/*
 * Project:  Petruchio
 * Filename: SimplerFormatter.java
 * Created:  24.08.2007
 * Author:   Tim Strazny
 */

package petruchio.common;

import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.LogRecord;

import petruchio.interfaces.SelfDescribing;


/**
 * A simple log formatter that simply returns the message text and, if it's not
 * an info level log record, the level text.
 *
 * @author Tim Strazny
 */
public class SimplerFormatter extends Formatter implements SelfDescribing {
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "A log formatter that prints the log level's name and the log message. The log level's name is omitted if it's INFO.";
    }
    

    /* (non-Javadoc)
     * @see java.util.logging.Formatter#format(java.util.logging.LogRecord)
     */
    @Override
    public String format(final LogRecord record) {
        if(record.getLevel() == Level.INFO) {
            return record.getMessage() + "\n";
        }
        return record.getLevel().getLocalizedName() + ": " 
                + record.getMessage() + "\n";
    }

}