/*
 * Project:  Petruchio
 * Filename: CommonTasks.java
 * Created:  29.01.2007
 * Author:   Tim Strazny
 */
package petruchio.compiler;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;

import petruchio.compiler.ProcessData.NameCard;
import petruchio.interfaces.pi.ActionPrefix;
import petruchio.interfaces.pi.Guard;
import petruchio.interfaces.pi.InputAction;
import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.OutputAction;
import petruchio.interfaces.pi.PrefixProcess;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessComposition;
import petruchio.interfaces.pi.ProcessCreator;
import petruchio.interfaces.pi.ProcessReference;

/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class CommonTasks {

    /** */
    public static final Comparator<Process<ProcessData>> PCOMPARATOR = ProcessComparator.getInstance();
    
    public static final Comparator<Process<ProcessData>> PCOMPARATORNOFN = ProcessComparatorNoFN.getInstance();
    
    
    /**
     * TODO: document
     *
     * @return A new ProcessData object.
     */
    private static ProcessData newProcessData() {
        return new Data();
//    	return new SoftDataWrapper(new Data());
    }
    
    
    /**
     * TODO: document
     *
     * @param root
     * @param search
     * @param replace
     * @param indices
     * @param index
     * @param pbuilder
     * @return
     */
    public static Process<ProcessData> replaceProcess(final Process<ProcessData> root,
                    final Process<ProcessData> search, final Process<ProcessData> replace,
                    final List<Integer> indices, final int index,
                    final ProcessCreator pbuilder) {
        final Process<ProcessData> result;
        
        if(root == search) {
            result = replace;
            
        } else {
            switch(root.getType()) {
                case NULL:
                case REFERENCE:
                    result = root;
                    break;
                case PREFIX: {
                    final PrefixProcess<ProcessData> prefix = (PrefixProcess<ProcessData>)root;
                    final Process<ProcessData> sub = prefix.getProcess();
                    final Process<ProcessData> subReplaced = CommonTasks.replaceProcess(sub,
                                    search, replace, indices, index, pbuilder);
                    if(sub == subReplaced) {
                        result = root;
                        
                    } else if(subReplaced.getRestrictions().isEmpty()
                            || prefix.getActionPrefixes().isEmpty()) {
                        result = CommonTasks.copyTopLevel(subReplaced, pbuilder);
                        result.addRestrictions(prefix.getRestrictions());
                        result.addActionPrefixes(prefix.getActionPrefixes());
                        
                    } else {
                        result = pbuilder.newPrefixProcess(subReplaced);
                        result.addActionPrefixes(root.getActionPrefixes());
                        result.addRestrictions(root.getRestrictions());
                        
                    }
                    break;
                }
                case COMPOSITION: {
                    final ProcessComposition<ProcessData> comp = (ProcessComposition<ProcessData>)root;
                    CommonTasks.ensureOrdering(comp);
                    final List<Process<ProcessData>> l = new ArrayList<Process<ProcessData>>(comp.getProcesses());
                    final Integer theIndex = indices.get(index);
                    final Process<ProcessData> sub = l.get(theIndex.intValue());
                    final Process<ProcessData> subReplaced = CommonTasks.replaceProcess(sub, search, replace, indices, index+1, pbuilder);
                    if(CommonTasks.isBareNull(subReplaced)) {
                        l.remove(theIndex.intValue());
                    } else {
                        l.set(theIndex.intValue(), subReplaced);
                    }
                    
                    if(l.isEmpty()) {
                        result = pbuilder.nullProcess();
                        result.addActionPrefixes(root.getActionPrefixes());
                        
                    } else if(l.size() == 1) {
                        result = CommonTasks.copyTopLevel(l.get(0), pbuilder);
                        result.addActionPrefixes(root.getActionPrefixes());
                        result.addRestrictions(root.getRestrictions());
                        
                    } else {
                        final ProcessComposition<ProcessData> compReplaced = pbuilder.newProcessComposition();
                        compReplaced.addActionPrefixes(root.getActionPrefixes());
                        compReplaced.addRestrictions(root.getRestrictions());
                        compReplaced.setOperator(comp.getOperator());
                        compReplaced.addProcesses(l);
                        
                        result = compReplaced;
                    }    
                    break;
                }
                default:
                    throw new InternalError("Unexpected type of process: " + root);
            }
        }
        
        return result;
    }
    
    
    /**
     * TODO: document
     *
     * @param root
     * @param search
     * @param replace
     * @param pbuilder
     * @return
     */
    public static Process<ProcessData> replaceProcess(final Process<ProcessData> root,
                    final List<Process<ProcessData>> search,
                    final List<Process<ProcessData>> replace,
                    final ProcessCreator pbuilder) {
        
        if(search.isEmpty()) {
            return root;
        }
        
        final Process<ProcessData> result;
        
        final int index = CommonTasks.indexOfThis(search, root);
        if(index >= 0) {
            final Process<ProcessData> inresult = replace.get(index);
            search.remove(index);
            replace.remove(index);
            result = CommonTasks.replaceProcess(inresult, search, replace, pbuilder);
            
        } else {
            switch(root.getType()) {
                case NULL:
                case REFERENCE:
                    result = root;
                    break;
                case PREFIX: {
                    final PrefixProcess<ProcessData> prefix = (PrefixProcess<ProcessData>)root;
                    final Process<ProcessData> sub = prefix.getProcess();
                    final Process<ProcessData> subReplaced = CommonTasks.replaceProcess(sub,
                                    search, replace, pbuilder);
                    if(sub == subReplaced) {
                        result = root;
                        
                    } else if(subReplaced.getRestrictions().isEmpty()
                            || prefix.getActionPrefixes().isEmpty()) {
                        result = CommonTasks.copyTopLevel(subReplaced, pbuilder);
                        result.addRestrictions(prefix.getRestrictions());
                        result.addActionPrefixes(prefix.getActionPrefixes());
                        
                    } else {
                        result = pbuilder.newPrefixProcess(subReplaced);
                        result.addActionPrefixes(root.getActionPrefixes());
                        result.addRestrictions(root.getRestrictions());
                        
                    }
                    break;
                }
                case COMPOSITION: {
                    final ProcessComposition<ProcessData> comp = (ProcessComposition<ProcessData>)root;
                    CommonTasks.ensureOrdering(comp);
                    final List<Process<ProcessData>> l = new ArrayList<Process<ProcessData>>(comp.getProcesses().size());
                    boolean changed = false;
                    for(final Process<ProcessData> sub : comp.getProcesses()) {
                        if(search.isEmpty()) {
                            l.add(sub);
                        } else {
                            final Process<ProcessData> subReplaced = CommonTasks.replaceProcess(sub, search, replace, pbuilder);
                            if((subReplaced.getType() != Process.Type.NULL) || !subReplaced.getActionPrefixes().isEmpty()) {
                                l.add(subReplaced);
                            }
                            
                            if(subReplaced != sub) {
                                changed = true;
                            }
                        }
                    }
                    
                    if(changed) {
                        if(l.isEmpty()) {
                            result = pbuilder.nullProcess();
                            result.addActionPrefixes(root.getActionPrefixes());
                            
                        } else if(l.size() == 1) {
                            result = CommonTasks.copyTopLevel(l.get(0), pbuilder);
                            result.addActionPrefixes(root.getActionPrefixes());
                            result.addRestrictions(root.getRestrictions());
                            
                        } else {
                            final ProcessComposition<ProcessData> compReplaced = pbuilder.newProcessComposition();
                            compReplaced.addActionPrefixes(root.getActionPrefixes());
                            compReplaced.addRestrictions(root.getRestrictions());
                            compReplaced.setOperator(comp.getOperator());
                            compReplaced.addProcesses(l);
                            
                            result = compReplaced;
                        }
                    } else {
                        result = root;
                        
                    }
                    break;
                }
                default:
                    throw new InternalError("Unexpected type of process: " + root);
            }
        }
        
        return result;
    }
    
    
    /**
     * TODO: document
     *
     * @param p
     */
    private static boolean updateData(final Process<ProcessData> p) {
        if((p.getData() != null) && p.getData().isAvailable()) {
            return false;
        }
        
        if(p.getData() == null) {
            p.setData(CommonTasks.newProcessData());
        } else {
            p.getData().clear();
        }
        
        final ProcessData data = p.getData();
        final Collection<Name> bn = data.getBn();
        final List<Name> fn = data.getFn();
        final Collection<Name> refIDs = data.getRefProcIDs();
        final Collection<NameCard> inChannels = data.getInChannels();
        final Collection<NameCard> outChannels = data.getOutChannels();
        final Collection<Name> fnWOR = data.getFnWOR();
        final Collection<Name> tempFN = new HashSet<Name>();
        
        int newCount = 0;
        int inParamCount = 0;
        int outParamCount = 0;
        int inCount = 0;
        int outCount = 0;
        int tauCount = 0;
        int guardCount = 0;
        int nullProcCount = 0;
        int refProcCount = 0;
        
        final String abstractionSuffix;
        
        switch(p.getType()) {
            case COMPOSITION: {
                final ProcessComposition<ProcessData> comp = (ProcessComposition<ProcessData>)p;
                final List<String> orders = new ArrayList<String>(comp.getProcesses().size());
                for(final Process<ProcessData> sub : comp.getProcesses()) {
                    CommonTasks.updateData(sub);
//                    for(final String name : sub.getData().getBn()) {
//                        if(bn.contains(name)) {
//                            throw new RuntimeException("Name " + name
//                                            + " is bound twice in process " + p);
//                        }
//                    }
                    bn.addAll(sub.getData().getBn());
                    tempFN.addAll(sub.getData().getFn());
                    refIDs.addAll(sub.getData().getRefProcIDs());
                    inChannels.addAll(sub.getData().getInChannels());
                    outChannels.addAll(sub.getData().getOutChannels());
                    orders.add(sub.getData().getProcessAbstraction());
                    
                    newCount += sub.getData().getNewCount();
                    inParamCount += sub.getData().getInParamCount();
                    outParamCount += sub.getData().getOutParamCount();
                    inCount += sub.getData().getInCount();
                    outCount += sub.getData().getOutCount();
                    tauCount += sub.getData().getTauCount();
                    guardCount += sub.getData().getGuardCount();
                    nullProcCount += sub.getData().getNullProcCount();
                    refProcCount += sub.getData().getRefProcCount();
                }
                if(comp.getOperator().isCommutative()) {
                    CommonTasks.sort(comp);
                    Collections.sort(orders);
                }
//                final StringBuffer b = new StringBuffer('(');
//                final char op;
//                switch(comp.getOperator()) {
//                    case PARALLEL : op = '|'; break;
//                    case CHOICE : op = '+'; break;
//                    case SEQUENCE : op = ';'; break;
//                    default: throw new IllegalArgumentException("Unknown type of operator: " + comp.getOperator());
//                }
//                for(final Iterator<String> it = orders.iterator(); it.hasNext(); ) {
//                    b.append(it.next());
//                    if(it.hasNext()) {
//                        b.append(op);
//                    }
//                }
//                b.append(')');
//                orderPostfix = b.toString();
                switch(comp.getOperator()) {
                    case PARALLEL : abstractionSuffix = "|"; break;
                    case CHOICE : abstractionSuffix = "+"; break;
                    case SEQUENCE : abstractionSuffix = ";"; break;
                    default: throw new IllegalArgumentException("Unknown type of operator: " + comp.getOperator());
                }
                break;
            }
            case PREFIX: {
                final PrefixProcess<ProcessData> prefix = (PrefixProcess<ProcessData>)p;
                final Process<ProcessData> sub = prefix.getProcess();
                CommonTasks.updateData(sub);
                
//                for(final String name : sub.getData().getBn()) {
//                    if(bn.contains(name)) {
//                        throw new RuntimeException("Name " + name
//                                        + " is bound twice in process " + p);
//                    }
//                }
                bn.addAll(sub.getData().getBn());
                tempFN.addAll(sub.getData().getFn());
                inChannels.addAll(sub.getData().getInChannels());
                outChannels.addAll(sub.getData().getOutChannels());
                refIDs.addAll(sub.getData().getRefProcIDs());
                
                abstractionSuffix = sub.getData().getProcessAbstraction();
                
                newCount += sub.getData().getNewCount();
                inParamCount += sub.getData().getInParamCount();
                outParamCount += sub.getData().getOutParamCount();
                inCount += sub.getData().getInCount();
                outCount += sub.getData().getOutCount();
                tauCount += sub.getData().getTauCount();
                guardCount += sub.getData().getGuardCount();
                nullProcCount += sub.getData().getNullProcCount();
                refProcCount += sub.getData().getRefProcCount();
                break;
            }
            case NULL: {
                nullProcCount = 1;
                abstractionSuffix = "";
                break;
            }
            case REFERENCE: {
                refProcCount = 1;
                // if the process is a reference, we have to add the parameters to the
                // set of free names
                final ProcessReference<ProcessData> ref = (ProcessReference<ProcessData>)p;
                tempFN.addAll(ref.getParameters().getParameters());
                refIDs.add(ref.getProcessID().getID());
                abstractionSuffix = "r" + ref.getProcessID().getID().toString() + "p" + ref.getParameters().size();
                break;
            }
            default:
                throw new InternalError("Unknown type of process: " + p.getType());
        }
        
        final StringBuilder abstraction = new StringBuilder();
        
        // now for the action prefixes
        for(final ActionPrefix ap : p.getActionPrefixes()) {
            switch(ap.getType()) {
                case INTERNAL:
                    abstraction.append('t');
                    ++tauCount;
                    // nothing to do
                    break;
                case INPUT: {
                    final InputAction in = (InputAction)ap;
                    abstraction.append('i');
                    abstraction.append(in.getNames().size());
                    inChannels.add(new NameCard(in.getChannel(), in.getNames().size()));
                    ++inCount;
                    inParamCount += in.getNames().size();
                    
//                    for(final String n : in.getNames().getParameters()) {
//                        if(bn.contains(n)) {
//                            throw new RuntimeException("Name " + n
//                                            + " is bound twice in process " + p);
//                        }
//                    }
                    bn.addAll(in.getNames().getParameters());
                    tempFN.add(in.getChannel());
                    break;
                }
                case OUTPUT: {
                    final OutputAction out = (OutputAction)ap;
                    abstraction.append('o');
                    abstraction.append(out.getNames().size());
                    outChannels.add(new NameCard(out.getChannel(), out.getNames().size()));
                    ++outCount;
                    outParamCount += out.getNames().size();
                    
                    tempFN.addAll(out.getNames().getParameters());
                    tempFN.add(out.getChannel());
                    break;
                }
                case GUARD: {
                    final Guard g = (Guard)ap;
                    tempFN.add(g.leftOperand());
                    tempFN.add(g.rightOperand());
                    if(!g.isSatisfied()) {
                        abstraction.append('g');
                        ++guardCount;
                    }
                    break;
                }
                default:
                    throw new InternalError("Unexpected type of prefix: " + ap);
            }
        }
        abstraction.append(abstractionSuffix);
        
        fnWOR.addAll(tempFN);
        fnWOR.removeAll(bn);
        
//        for(final String name : p.getRestrictions()) {
//            if(bn.contains(name)) {
//                throw new RuntimeException("Name " + name
//                                + " is bound twice in process " + p);
//            }
//        }
        
        bn.addAll(p.getRestrictions());
        newCount += p.getRestrictions().size();
        
        tempFN.removeAll(bn);
        
        fn.addAll(tempFN);
        
        Collections.sort(fn);
        
        data.setPrefixOrder(abstraction.toString());
        data.setNewCount(newCount);
        data.setInParamCount(inParamCount);
        data.setOutParamCount(outParamCount);
        data.setInCount(inCount);
        data.setOutCount(outCount);
        data.setTauCount(tauCount);
        data.setGuardCount(guardCount);
        data.setNullProcCount(nullProcCount);
        data.setRefProcCount(refProcCount);
        
        return true;
    }
    
    
    /**
     * @param p
     */
    public static void ensureOrdering(final ProcessComposition<ProcessData> p) {
        CommonTasks.updateData(p);
    }

    
    /**
     * @param p
     */
    private static void sort(final ProcessComposition<ProcessData> p) {
        Collections.sort(p.getProcesses(), CommonTasks.PCOMPARATOR);
    }
    
    
    /**
     * @param p1
     * @param p2
     * @return
     */
    public static boolean equalIndex(final Process<ProcessData> p1, final Process<ProcessData> p2) {
        return CommonTasks.PCOMPARATOR.compare(p1, p2) == 0;
    }
    
    
    public static boolean equalIndexNoFN(final Process<ProcessData> p1, final Process<ProcessData> p2) {
        return CommonTasks.PCOMPARATORNOFN.compare(p1, p2) == 0;
    }
    
    
    /**
     * @param p
     * @return
     */
    public static List<Name> freeNames(final Process<ProcessData> p) {
        CommonTasks.updateData(p);
        return p.getData().getFn();
    }
    
    
    /**
     * @param p
     * @return
     */
    public static Collection<Name> boundNames(final Process<ProcessData> p) {
        CommonTasks.updateData(p);
        return p.getData().getBn();
    }
    
    
    /**
     * TODO: document
     *
     * @param p
     * @return
     */
    public static Collection<NameCard> inputChannels(final Process<ProcessData> p) {
        CommonTasks.updateData(p);
        return p.getData().getInChannels();
    }
    
    
    /**
     * TODO: document
     *
     * @param p
     * @return
     */
    public static Collection<NameCard> outputChannels(final Process<ProcessData> p) {
        CommonTasks.updateData(p);
        return p.getData().getOutChannels();
    }
    
    
    /**
     * @param p
     * @return
     */
    public static Collection<Name> freeNamesWOR(final Process<ProcessData> p) {
        CommonTasks.updateData(p);
        return p.getData().getFnWOR();
    }
    
    
//    /**
//     * @param p
//     * @return
//     */
//    public static Collection<String> boundNamesWOR(final Process<ProcessData> p) {
//        CommonTasks.updateData(p);
//        return p.getData().getBnWOR();
//    }
    
    
    /**
     * @param p
     * @return
     */
    public static ProcessData getData(final Process<ProcessData> p) {
        CommonTasks.updateData(p);
        return p.getData();
    }

    
    /**
     * TODO: document
     *
     * @param <T>
     * @param <O>
     * @param c
     * @param o
     * @return
     */
    public static <T, O extends T> int indexOfThis(final List<T> c, final O o) {
        final int size = c.size();
        for(int i = 0; i < size; ++i) {
            if(c.get(i) == o) {
                return i;
            }
        }
        return -1;
    }
    
    
    /**
     * TODO: document
     *
     * @param <T>
     * @param <O>
     * @param c
     * @param o
     * @return
     */
    public static <T, O extends T> int lastIndexOfThis(final List<T> c, final O o) {
        for(int i = c.size()-1; i >= 0; --i) {
            if(c.get(i) == o) {
                return i;
            }
        }
        return -1;
    }
    
    
    /**
     * TODO: document
     *
     * @param <T>
     * @param <O>
     * @param start
     * @param c
     * @param o
     * @return
     */
    public static <T, O extends T> int indexOfThis(final int theStart, final List<T> c, final O o) {
        final int size = c.size();
        final int start;
        if((theStart >= size) || (theStart < 0)) {
            start = 0;
        } else {
            start = theStart;
        }
        for(int i = 0; i < size; ++i) {
            int index = start+i;
            if(index >= size) {
                index -= size;
            }
            if(c.get(index) == o) {
                return index;
            }
        }
        return -1;
    }
    
    
    /**
     * TODO: document
     *
     * @param <T>
     * @param <O>
     * @param c
     * @param o
     * @return
     */
    public static <T, O extends T> boolean containsThis(final List<T> c, O o) {
        return CommonTasks.indexOfThis(c, o) >= 0;
    }
    
    
    /**
     * @param <T>
     * @param <O>
     * @param c
     * @param o
     * @return
     */
    public static <T, O extends T> int removeAllOfThese(final List<T> c, List<O> o) {
        if(c.isEmpty()) {
            return -1;
        }
        
        int index = -1;
        int curStart = 0;
        for(int j = o.size()-1; j >= 0; --j) {
            final O current = o.get(j);
            final int r = CommonTasks.indexOfThis(curStart, c, current);
            if(r >= 0) {
                if(index < 0) {
                    index = r;
                }
                curStart = r;
                c.remove(r);
            }

            if(c.isEmpty()) {
                break;
            }
        }
        return index;
    }
    
    
    /**
     * TODO: document
     *
     * @param p
     * @param pbuilder
     * @param <T>
     * @return
     */
    @SuppressWarnings("unchecked")
    public static <T, P extends Process<T>> P copyTopLevel(final Process<T> p,
            final ProcessCreator pbuilder) {
        final Process<T> copy;
        switch(p.getType()) {
            case NULL:
                copy = pbuilder.nullProcess();
                break;
            case PREFIX:
                copy = pbuilder.newPrefixProcess(((PrefixProcess<T>)p).getProcess());
                break;
            case REFERENCE:
                final ProcessReference<T> ref = (ProcessReference<T>)p;
                copy = pbuilder.newProcessReference(ref.getProcessID(), ref.getParameters());
                break;
            case COMPOSITION:
                final ProcessComposition<T> comp = (ProcessComposition<T>)p;
                final ProcessComposition<T> compCopy = pbuilder.newProcessComposition();
                compCopy.addProcesses(comp.getProcesses());
                compCopy.setOperator(comp.getOperator());
                copy = compCopy;
                break;
            default:
                throw new InternalError("Unexpected type of process: " + p);
        }
        
        copy.addRestrictions(p.getRestrictions());
        copy.addActionPrefixes(p.getActionPrefixes());
        return (P)copy;
    }
    
    
    /**
     * TODO: document
     *
     * @param <T>
     * @param p
     * @param pbuilder
     * @return
     */
    public static <T> Process<T> liftRestrictions(final Process<T> p,
                    final ProcessCreator pbuilder) {
        final Collection<Name> restrictions = new ArrayList<Name>(0);
        final Process<T> stripped = CommonTasks.collectRestrictions(p,
                        restrictions, pbuilder);
        stripped.addRestrictions(restrictions);
        return stripped;
    }
    
    
    /**
     * TODO: document
     *
     * @param <T>
     * @param p
     * @param pbuilder
     * @return
     */
    @SuppressWarnings("unchecked")
    public static <T> ProcessComposition<T> flatten(
                    final ProcessComposition<ProcessData> p, final ProcessCreator pbuilder){
        return (ProcessComposition<T>)CommonTasks.liftRestrictions(p, pbuilder);
    }
    
    
    
    /**
     * TODO: document
     *
     * @param p
     * @param restrictions
     * @param pbuilder
     * @param <T>
     * @return
     */
    public static <T> Process<T> collectRestrictions(
                    final Process<T> p,
                    final Collection<Name> restrictions,
                    final ProcessCreator pbuilder) {

        final Process<T> newP;
        if(p.getType() == Process.Type.PREFIX) {
            final PrefixProcess<T> prefix = (PrefixProcess<T>)p;
            final Process<T> sub = prefix.getProcess();
            final Process<T> newSub = CommonTasks.liftRestrictions(sub, pbuilder);
            if(prefix.getActionPrefixes().isEmpty()) {
                newP = newSub;
                
            } else {
                newP = pbuilder.newPrefixProcess(newSub);
                newP.addActionPrefixes(p.getActionPrefixes());
                
            }
            
        } else if(p.getType() == Process.Type.COMPOSITION) {
            final ProcessComposition<T> comp = (ProcessComposition<T>)p;
            // not needed CommonTasks.ensureOrdering(comp);
            
            boolean changed = false;
            final List<Process<T>> l = new ArrayList<Process<T>>();
            for(final Process<T> sub : comp.getProcesses()) {
                final Process<T> newSub;
                if(comp.getOperator().allowsScopeExtrusion()) {
                    newSub = CommonTasks.collectRestrictions(sub, restrictions, pbuilder);
                } else {
                    newSub = CommonTasks.liftRestrictions(sub, pbuilder);
                }
                if(newSub.getActionPrefixes().isEmpty() && (newSub.getType() == Process.Type.COMPOSITION)
                                && (((ProcessComposition<T>)newSub).getOperator() == comp.getOperator())) {
                    l.addAll(((ProcessComposition<T>)newSub).getProcesses());
                } else {
                    l.add(newSub);
                }
                
                if(newSub != sub) {
                    changed = true;
                }
            }
            
            if(changed) {
                final ProcessComposition<T> newComp = pbuilder.newProcessComposition();
                newComp.setOperator(comp.getOperator());
                newComp.addProcesses(l);
                
                newP = newComp;
                newP.addActionPrefixes(p.getActionPrefixes());
                if(!comp.getOperator().allowsScopeExtrusion()) {
                    newP.addRestrictions(p.getRestrictions());
                }
            } else {
                newP = p;
            }
            
        } else {
            newP = p;
            
        }
        
        if(p.getRestrictions().isEmpty() || ((p.getType() == Process.Type.COMPOSITION)
                    && !((ProcessComposition<T>)p).getOperator().allowsScopeExtrusion())) {
            return newP;
            
        }
        restrictions.addAll(p.getRestrictions());
        
        final Process<T> clean = CommonTasks.copyTopLevel(newP, pbuilder);
        clean.getRestrictions().clear();
        
        return clean;
    }
    
    
    /**
     * TODO: document
     *
     * @param p
     * @return
     */
    public static boolean isBareNull(final Process<?> p) {
        return (p.getType() == Process.Type.NULL) && p.getActionPrefixes().isEmpty();
    }
    
    
    /**
     * TODO: document
     *
     * @param a
     * @param b
     * @return
     */
    public static int compareNameList(final List<Name> a, final List<Name> b) {
        if(a.size() != b.size()) {
            return a.size() - b.size();
        }
        for(int i = 0; i < a.size()-1; ++i) {
            final int c = a.get(i).compareTo(b.get(i));
            if(c != 0) {
                return c;
            }
        }
        return 0;
    }
    
    public static int compareIntegerList(final List<Integer> a, final List<Integer> b) {
        for(int i = 0; i < a.size()-1; ++i) {
            final int c = a.get(i).compareTo(b.get(i));
            if(c != 0) {
                return c;
            }
        }
        return 0;
    }
    
    
    /**
     * TODO: document
     *
     * @param a
     * @param b
     * @return
     */
    public static boolean equalFn(final List<Name> a, final List<Name> b) {
        return CommonTasks.compareNameList(a, b) == 0;
    }
 
}