/*
 * Project:  Petruchio
 * Filename: ProcessData.java
 * Created:  29.01.2007
 * Author:   Tim Strazny
 */
package petruchio.compiler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import petruchio.common.WeakIdentityHashMap;
import petruchio.interfaces.pi.ActionPrefix;
import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.Process;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class Data implements ProcessData {

    /** */
    private Map<Process<ProcessData>, Boolean> structCong = new WeakIdentityHashMap<Process<ProcessData>, Boolean>(0);
    
    /** */
    private Set<Name> bn = new HashSet<Name>(0);
    
    /** */
    private List<Name> fn = new ArrayList<Name>(0);
    
    /** */
    private Set<NameCard> inChannels = new HashSet<NameCard>(0);
    
    /** */
    private Set<NameCard> outChannels = new HashSet<NameCard>(0);
    
    private Set<ActionPrefix> communicators = null;
    
    private int h = 0;

//    /** */
//    private Set<String> bnWOR = new HashSet<String>();
    
    /** */
    private Set<Name> fnWOR = new HashSet<Name>(0);
    
    private Collection<Name> refIDs = new HashSet<Name>(0);

    /** */
    private int inCount = 0;
    
    /** */
    private int outCount = 0;
    
    /** */
    private int guardCount = 0;
    
    /** */
    private int tauCount = 0;
    
    /** */
    private int inParamCount = 0;
    
    /** */
    private int outParamCount = 0;
    
    /** */
    private int newCount = 0;
    
    /** */
    private int nullProcCount = 0;
    
    /** */
    private int refProcCount = 0;
    
    /** */
    private String prefixOrder = null;
    
    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#isAvailable()
     */
    @Override
    public boolean isAvailable() {
    	return true;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getBn()
     */
    @Override
    public Collection<Name> getBn() {
        return this.bn;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getFn()
     */
    @Override
    public List<Name> getFn() {
        return this.fn;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getInChannels()
     */
    @Override
    public Collection<NameCard> getInChannels() {
        return this.inChannels;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getOutChannels()
     */
    @Override
    public Collection<NameCard> getOutChannels() {
        return this.outChannels;
    }
    
    
//    /* (non-Javadoc)
//     * @see petruchio.compiler.ProcessData#getBnWOR()
//     */
//    public Collection<String> getBnWOR() {
//        return this.bnWOR;
//    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getFnWOR()
     */
    @Override
    public Collection<Name> getFnWOR() {
        return this.fnWOR;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getStructCong()
     */
    @Override
    public Map<Process<ProcessData>, Boolean> getStructCong() {
        return this.structCong;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getPrefixOrder()
     */
    @Override
    public String getProcessAbstraction() {
        return this.prefixOrder;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setPrefixOrder(java.lang.String)
     */
    @Override
    public void setPrefixOrder(String order) {
        this.prefixOrder = order;
    }
    

    /* (non-Javadoc)
     * @see java.lang.Object#clone()
     */
    @Override
    public ProcessData clone() {
        try {
            final ProcessData clone = (ProcessData)super.clone();
            this.structCong = (this.structCong == null) ? null
                            : new WeakIdentityHashMap<Process<ProcessData>, Boolean>(this.structCong);
            this.bn = (this.bn == null) ? null : new HashSet<Name>(this.bn);
            this.fn = (this.fn == null) ? null : new ArrayList<Name>(this.fn);
            this.refIDs = (this.refIDs == null) ? null : new ArrayList<Name>(this.refIDs);
            return clone;
        } catch(final CloneNotSupportedException e) {
            // cannot happen!
            throw new InternalError("NOW THIS IS UNEXPECTED! We implement "
                            + "Cloneable but the exception gets thrown!?");
        }
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getInCount()
     */
    @Override
    public int getInCount() {
        return this.inCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setInCount(int)
     */
    @Override
    public void setInCount(int inCount) {
        this.inCount = inCount;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getOutCount()
     */
    @Override
    public int getOutCount() {
        return this.outCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setOutCount(int)
     */
    @Override
    public void setOutCount(int outCount) {
        this.outCount = outCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getTauCount()
     */
    @Override
    public int getTauCount() {
        return this.tauCount;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setTauCount(int)
     */
    @Override
    public void setTauCount(int tauCount) {
        this.tauCount = tauCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getSeqProcCount()
     */
    @Override
    public int getNullProcCount() {
        return this.nullProcCount;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setSeqProcCount(int)
     */
    @Override
    public void setNullProcCount(int seqProcCount) {
        this.nullProcCount = seqProcCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getGuardCount()
     */
    @Override
    public int getGuardCount() {
        return this.guardCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setGuardCount(int)
     */
    @Override
    public void setGuardCount(int guardCount) {
        this.guardCount = guardCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getRefProcCount()
     */
    @Override
    public int getRefProcCount() {
        return this.refProcCount;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setRefProcCount(int)
     */
    @Override
    public void setRefProcCount(int refProcCount) {
        this.refProcCount = refProcCount;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getInParamCount()
     */
    @Override
    public int getInParamCount() {
        return this.inParamCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setInParamCount(int)
     */
    @Override
    public void setInParamCount(int inParamCount) {
        this.inParamCount = inParamCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getNewCount()
     */
    @Override
    public int getNewCount() {
        return this.newCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setNewCount(int)
     */
    @Override
    public void setNewCount(int newCount) {
        this.newCount = newCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getOutParamCount()
     */
    @Override
    public int getOutParamCount() {
        return this.outParamCount;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setOutParamCount(int)
     */
    @Override
    public void setOutParamCount(int outParamCount) {
        this.outParamCount = outParamCount;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getRefProcIDs()
     */
    @Override
    public Collection<Name> getRefProcIDs() {
        return this.refIDs;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#clear()
     */
    @Override
    public void clear() {
        this.bn.clear();
        this.fn.clear();
        this.fnWOR.clear();
        this.inChannels.clear();
        this.outChannels.clear();
        this.refIDs.clear();
        this.structCong.clear();
        if(this.communicators != null) {
            this.communicators.clear();
            this.communicators = null;
        }
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        if(this.h == 0) {
            this.h = (this.inCount << 0) ^ (this.outCount << 3) ^ (this.guardCount << 6)
                ^ (this.inParamCount << 9) ^ (this.outParamCount << 12) ^ (this.newCount << 15)
                ^ (this.nullProcCount << 18) ^ (this.tauCount << 21) ^ (this.refIDs.hashCode() << 24)
                ^ (this.prefixOrder == null ? 209436 : this.prefixOrder.hashCode())
                ^ (this.fn.hashCode() << 27) ^ this.fnWOR.hashCode();
        }
        return this.h;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object o) {
        return (o == this) || ((o != null) && (o instanceof Data) && this.equals((Data)o));
    }
    
    /**
     * @param d
     * @return
     */
    public boolean equals(final Data d) {
        return this.hashCode() == d.hashCode()
            && this.inCount == d.inCount && this.outCount == d.outCount
            && this.guardCount == d.guardCount && this.inParamCount == d.inParamCount
            && this.outParamCount == d.outParamCount && this.newCount == d.newCount
            && this.nullProcCount == d.nullProcCount && this.tauCount == d.tauCount
            && this.refIDs.equals(d.refIDs)
            && (this.prefixOrder == null ? d.prefixOrder == null : this.prefixOrder.equals(d.prefixOrder))
            && this.fn.equals(d.fn) && this.fnWOR.equals(d.fnWOR);
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#getCommunicators()
     */
    @Override
    public Collection<ActionPrefix> getCommunicators() {
        return this.communicators;
    }


    /* (non-Javadoc)
     * @see petruchio.compiler.ProcessData#setCommunicators(java.util.Collection)
     */
    @Override
    public void setCommunicators(Collection<ActionPrefix> communicators) {
        this.communicators = new HashSet<ActionPrefix>(communicators);
    }


}