package petruchio.compiler;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import petruchio.compiler.struct.DefaultStructChecker;
import petruchio.interfaces.algorithms.StructuralCongruence;
import petruchio.interfaces.pi.PiParser;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessDefinition;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.Transition;
import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.pi.ProcessCreator;
import petruchio.pn.Converter;


public class PNDiff {
    
    public static <E> Process<E> parseProcess(final PiParser parser, final String prc) {
        try {
            final String proc = parser.getDummyProcessDefinitionPrefix()
                    + prc
                    + parser.getDummyProcessDefinitionSuffix();
            parser.setInput(new ByteArrayInputStream(proc.getBytes()));
            final ProcessDefinition<E> def = parser.nextProcessDefinition();
            return def.getProcess();
        } catch(final RuntimeException e) {
            throw new RuntimeException("Error parsing process: " + prc, e);
        }
    }
    
    public static <E> Process<E> getProcess(final PiParser parser, final Place p, final Map<Place, Process<E>> cache) {
        Process<E> proc = cache == null ? null : cache.get(p);
        if(proc == null) {
            final String prc = p.getMeaning();
            final int start = prc.indexOf(petruchio.interfaces.Retriever.MEANING_SEP);
            proc = parseProcess(parser, prc.substring(start < 0 ? 0 : start+petruchio.interfaces.Retriever.MEANING_SEP.length()));
            if(cache != null) {
                cache.put(p, proc);
            }
        }
        return proc;
    }
    
    public static Map<Place, Place> diff(final PetriNet netA, final PetriNet netB, final Map<Place, Place> wicked) {
        final StructuralCongruence<ProcessData> struct = new DefaultStructChecker();
        final PiParser parser = new petruchio.pi.readers.cup.PiParser();
        final ProcessCreator creator = new petruchio.pi.ProcessCreator();
        struct.setProcessCreator(creator);
        parser.setProcessCreator(creator);
        
        final Map<Place, Process<ProcessData>> cache = new IdentityHashMap<Place, Process<ProcessData>>();
        final Map<Place, Place> matchedAB = new IdentityHashMap<Place, Place>();
        
        final Queue<Place> todo = new LinkedList<Place>();
        for(final Place p : netA.getPlaces()) {
            if(p.getMarking() != 0) {
                todo.add(p);
            }
        }
        for(final Place pB : netB.getPlaces()) {
            if(pB.getMarking() != 0) {
                final Process<ProcessData> pb = PNDiff.<ProcessData>getProcess(parser, pB, cache);
                Place matched = null;
                for(final Place pA : todo) {
                    final Process<ProcessData> pa = PNDiff.<ProcessData>getProcess(parser, pA, cache);
                    if(!matchedAB.containsKey(pA) && struct.structurallyCongruent(pa, pb)) {
                        matched = pA;
                        break;
                    }
                }
                if(matched == null) {
                    System.out.println("Unable to match initially marked place of second net: " + pB);
                    return matchedAB;
                }
                matchedAB.put(matched, pB);
            }
        }
        while(!todo.isEmpty()) {
            final Place pA = todo.remove();
            final Place pB = matchedAB.get(pA);
            if(pB == null) {
                System.out.println("Unable to match initially marked place: " + pA);
                return matchedAB;
            }
//            if(pA.getOutput().size() != pB.getOutput().size()) {
//                System.out.println("Different output degree. Unable to match: " + pA + " and " + pB);
//                wicked.put(pA, pB);
//                return wicked;
//            }
            final Queue<PTArc> arcsToMatchB = new LinkedList<PTArc>(pB.getOutput());
            for(final PTArc inA : pA.getOutput()) {
                boolean ptArcMatches = false;
                for(final Iterator<PTArc> it = arcsToMatchB.iterator(); it.hasNext(); ) {
                    final PTArc inB = it.next();
                    if(inA.getWeight() != inB.getWeight()) {
                        continue;
                    }
                    boolean transitionMatches = true;
                    for(final TPArc outA : inA.getTarget().getOutput()) {
                        final Process<ProcessData> pa = PNDiff.<ProcessData>getProcess(parser, outA.getTarget(), cache);
                        final Place match = matchedAB.get(outA.getTarget());
                        boolean tpArcMatches = false;
                        for(final TPArc outB : inB.getTarget().getOutput()) {
                            if(outA.getWeight() != outB.getWeight() || (match != null && outB.getTarget() != match)) {
                                continue;
                            }
                            if(outB.getTarget() == match) {
                                tpArcMatches = true;
                                break;
                            }
                            final Process<ProcessData> pb = PNDiff.<ProcessData>getProcess(parser, outB.getTarget(), cache);
                            if(struct.structurallyCongruent(pa, pb)) {
                                matchedAB.put(outA.getTarget(), outB.getTarget());
                                tpArcMatches = true;
                                todo.add(outA.getTarget());
//                                System.out.println("\n" + pa + "\n " + outA.getTarget() + " == " + outB.getTarget() + "\n" + pb);
                                break;
//                            } else {
//                                System.out.println("\n" + pa + "\n " + outA.getTarget() + " =/= " + outB.getTarget() + "\n" + pb);
                            }
                        }
                        if(!tpArcMatches) {
                            transitionMatches = false;
                            break;
                        }
                    }
                    if(transitionMatches) {
                        it.remove();
                        ptArcMatches = true;
                        break;
                    }
                }
                if(!ptArcMatches) {
                    System.out.println("Unable to match arc " + inA + " of " + pA + " and " + pB);
                    wicked.put(pA, pB);
                    return matchedAB;
                }
            }
            if(!arcsToMatchB.isEmpty()) {
                System.out.println("Unable to match arc(s) " + arcsToMatchB + " of " + pA + " and " + pB);
                wicked.put(pA, pB);
                return matchedAB;
            }
        }
        System.out.println("Reachable parts of nets match.");
        return matchedAB;
    }
    
    
    public static void defaultDiff(final String nameA, final String nameB) {
        final PetriNet netA = Converter.loadNet(nameA);
        final PetriNet netB = Converter.loadNet(nameB);
        
        final Map<Place, Place> wicked = new IdentityHashMap<Place, Place>();
        final Map<Place, Place> matching = PNDiff.diff(netA, netB, wicked);
        if(wicked.isEmpty()) {
            return;
        }
        
        final PetriNet redA = new petruchio.pn.PetriNet();
        final PetriNet redB = new petruchio.pn.PetriNet();
        final Map<Transition, Transition> aT = new IdentityHashMap<Transition, Transition>();
        final Map<Transition, Transition> bT = new IdentityHashMap<Transition, Transition>();
        final Map<Place, Place> aP = new IdentityHashMap<Place, Place>();
        final Map<Place, Place> bP = new IdentityHashMap<Place, Place>();
        for(final Map.Entry<Place, Place> e : matching.entrySet()) {
            {
                final Place p = e.getKey();
                final Place pRed = redA.addPlace(p.getName(), p.getMarking());
                aP.put(p, pRed);
                pRed.setBound(p.getBound());
                pRed.setCouldCommunicate(p.couldCommunicate());
                pRed.setFinal(p.isFinal());
                pRed.setMeaning((matching.containsKey(p) ? "___matched___" : "") + p.getMeaning());
                pRed.setNote(p.getNote());
                pRed.setX(p.getX());
                pRed.setY(p.getY());
                for(final PTArc a : p.getOutput()) {
                    final Transition t = a.getTarget();
                    Transition tRed = aT.get(t);
                    if(tRed == null) {
                        tRed = redA.addTransition(t.getName());
                        tRed.setMeaning(t.getMeaning());
                        tRed.setX(t.getX());
                        tRed.setY(t.getY());
                        aT.put(t, tRed);
                        
                        for(final TPArc b : t.getOutput()) {
                            final Place p2 = b.getTarget();
                            Place pRed2 = aP.get(p2);
                            if(pRed2 == null) {
                                pRed2 = redA.addPlace(p2.getName(), p2.getMarking());
                                pRed2.setBound(p2.getBound());
                                pRed2.setCouldCommunicate(p2.couldCommunicate());
                                pRed2.setFinal(p2.isFinal());
                                pRed2.setMeaning((matching.containsKey(p2) ? "___matched___" : "") + p2.getMeaning());
                                pRed.setNote(p2.getNote());
                                pRed2.setX(p2.getX());
                                pRed2.setY(p2.getY());
                                aP.put(p2, pRed2);
                            }
                            redA.addArc(tRed, pRed2, b.getWeight());
                        }
                        for(final PTArc b : t.getInput()) {
                            final Place p2 = b.getSource();
                            Place pRed2 = aP.get(p2);
                            if(pRed2 == null) {
                                pRed2 = redA.addPlace(p2.getName(), p2.getMarking());
                                pRed2.setBound(p2.getBound());
                                pRed2.setCouldCommunicate(p2.couldCommunicate());
                                pRed2.setFinal(p2.isFinal());
                                pRed2.setMeaning((matching.containsKey(p2) ? "___matched___" : "") + p2.getMeaning());
                                pRed.setNote(p2.getNote());
                                pRed2.setX(p2.getX());
                                pRed2.setY(p2.getY());
                                aP.put(p2, pRed2);
                            }
                            redA.addArc(pRed2, tRed, b.getWeight());
                        }
                    }
                }
            }
            {
                final Place p = e.getValue();
                final Place pRed = redB.addPlace(p.getName(), p.getMarking());
                bP.put(p, pRed);
                pRed.setBound(p.getBound());
                pRed.setCouldCommunicate(p.couldCommunicate());
                pRed.setFinal(p.isFinal());
                pRed.setMeaning((matching.containsValue(p) ? "___matched___" : "") + p.getMeaning());
                pRed.setNote(p.getNote());
                pRed.setX(p.getX());
                pRed.setY(p.getY());
                for(final PTArc a : p.getOutput()) {
                    final Transition t = a.getTarget();
                    Transition tRed = bT.get(t);
                    if(tRed == null) {
                        tRed = redB.addTransition(t.getName());
                        tRed.setMeaning(t.getMeaning());
                        tRed.setX(t.getX());
                        tRed.setY(t.getY());
                        bT.put(t, tRed);
                        
                        for(final TPArc b : t.getOutput()) {
                            final Place p2 = b.getTarget();
                            Place pRed2 = bP.get(p2);
                            if(pRed2 == null) {
                                pRed2 = redB.addPlace(p2.getName(), p2.getMarking());
                                pRed2.setBound(p2.getBound());
                                pRed2.setCouldCommunicate(p2.couldCommunicate());
                                pRed2.setFinal(p2.isFinal());
                                pRed2.setMeaning((matching.containsValue(p2) ? "___matched___" : "") + p2.getMeaning());
                                pRed.setNote(p2.getNote());
                                pRed2.setX(p2.getX());
                                pRed2.setY(p2.getY());
                                bP.put(p2, pRed2);
                            }
                            redB.addArc(tRed, pRed2, b.getWeight());
                        }
                        
                        for(final PTArc b : t.getInput()) {
                            final Place p2 = b.getSource();
                            Place pRed2 = bP.get(p2);
                            if(pRed2 == null) {
                                pRed2 = redB.addPlace(p2.getName(), p2.getMarking());
                                pRed2.setBound(p2.getBound());
                                pRed2.setCouldCommunicate(p2.couldCommunicate());
                                pRed2.setFinal(p2.isFinal());
                                pRed2.setMeaning((matching.containsValue(p2) ? "___matched___" : "") + p2.getMeaning());
                                pRed.setNote(p2.getNote());
                                pRed2.setX(p2.getX());
                                pRed2.setY(p2.getY());
                                bP.put(p2, pRed2);
                            }
                            redB.addArc(pRed2, tRed, b.getWeight());
                        }
                    }
                }
            }
        }
        
        Converter.writeNet(redA, nameA.substring(0, nameA.lastIndexOf('.')) + ".diff" + nameA.substring(nameA.lastIndexOf('.')));
        Converter.writeNet(redB, nameB.substring(0, nameB.lastIndexOf('.')) + ".diff" + nameB.substring(nameB.lastIndexOf('.')));
        System.out.println("Difference written.");
    }
    
    
    public static boolean remove(final List<Process<ProcessData>> list, final Process<ProcessData> key, final StructuralCongruence<ProcessData> struct) {
        int low = 0;
        int high = list.size()-1;
        
        while (low <= high) {
            final int mid = (low + high) >>> 1;
            final Process<ProcessData> midVal = list.get(mid);
            final int cmp = struct.getComparator().compare(midVal, key);

            if (cmp < 0) {
                low = mid + 1;
            } else if (cmp != 0) {
                high = mid - 1;
            } else {
                if(struct.structurallyCongruent(midVal, key)) {
                    list.remove(mid);
                    return true;
                }
                
                boolean equal = true;
                int i = mid-1;
                while(equal && (i >= low)) {
                    final Process<ProcessData> m = list.get(i);
                    equal = struct.getComparator().compare(m, key) == 0;
                    if(equal && struct.structurallyCongruent(m, key)) {
                        list.remove(i);
                        return true;
                    }
                    --i;
                }
                
                equal = true;
                i = mid+1;
                while(equal && (i <= high)) {
                    final Process<ProcessData> m = list.get(i);
                    equal = struct.getComparator().compare(m, key) == 0;
                    if(equal && struct.structurallyCongruent(m, key)) {
                        list.remove(i);
                        return true;
                    }
                    ++i;
                }

                return false; // key not found
            }
        }
        return false;  // key not found
    }
    
    public static void processesOnly(final String nameA, final String nameB) {
        final StructuralCongruence<ProcessData> struct = new DefaultStructChecker();
        final PiParser parser = new petruchio.pi.readers.cup.PiParser();
        final ProcessCreator creator = new petruchio.pi.ProcessCreator();
        struct.setProcessCreator(creator);
        parser.setProcessCreator(creator);
        
        final PetriNet netA = Converter.loadNet(nameA);
        final List<Process<ProcessData>> procA = new ArrayList<Process<ProcessData>>(netA.getPlaces().size());
        for(final Place p : netA.getPlaces()) {
            procA.add(PNDiff.<ProcessData>getProcess(parser, p, null));
        }
        netA.reset();
        
        final PetriNet netB = Converter.loadNet(nameB);
        final List<Process<ProcessData>> procB = new ArrayList<Process<ProcessData>>(netB.getPlaces().size());
        for(final Place p : netB.getPlaces()) {
            procB.add(PNDiff.<ProcessData>getProcess(parser, p, null));
        }
        netB.reset();
        
        Collections.sort(procB, struct.getComparator());
        
        for(final Iterator<Process<ProcessData>> it = procA.iterator(); it.hasNext(); ) {
            final Process<ProcessData> p = it.next();
            if(PNDiff.remove(procB, p, struct)) {
                it.remove();
            }
        }
        if(!procA.isEmpty() || !procB.isEmpty()) {
            Collections.sort(procA, struct.getComparator());
            try {
                final PrintWriter print = new PrintWriter(nameA.substring(0, nameA.lastIndexOf('.')) + ".diff" + nameA.substring(nameA.lastIndexOf('.')));
                int i = 0;
                for(final Process<ProcessData> p : procA) {
                    print.print(String.valueOf(i++));
                    print.print(": ");
                    print.println(p);
                }
                print.close();
            } catch(final IOException e) {
                e.printStackTrace();
                return;
            }
            try {
                final PrintWriter print = new PrintWriter(nameB.substring(0, nameB.lastIndexOf('.')) + ".diff" + nameB.substring(nameB.lastIndexOf('.')));
                int i = 0;
                for(final Process<ProcessData> p : procB) {
                    print.print(String.valueOf(i++));
                    print.print(": ");
                    print.println(p);
                }
                print.close();
            } catch(final IOException e) {
                e.printStackTrace();
                return;
            }
            System.out.println("Differences written.");
        } else {
            System.out.println("Process collections match.");
        }
    }
    
    public static void main(final String[] args) {
        if(args.length < 2 || args.length > 3 || !args[2].equals("-s")) {
            System.out.println("usage: PNDiff filenameA filenameB [-s]");
            return;
        }
        if(args.length > 2 && args[2].equals("-s")) {
            PNDiff.processesOnly(args[0], args[1]);
        } else {
            PNDiff.defaultDiff(args[0], args[1]);
        }
    }

}
