/*
 * Project:  Petruchio
 * Filename: Reaction.java
 * Created:  16.12.2006
 * Author:   Tim Strazny
 */
package petruchio.compiler.reactions;

import java.util.ArrayList;
import java.util.Collection;

import petruchio.interfaces.petrinet.Transition;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessComposition;


/**
 * <Description>
 * 
 * @param <E>
 * @author Tim Strazny
 */
public abstract class Reaction<E> implements petruchio.interfaces.algorithms.Reaction<E> {

    /** */
    private Process<E> process = null;
    
    private Transition trans = null;
    
    private Independent isSequentiallyIndependent = Independent.NOT_SET;
    
    private Collection<ProcessComposition<E>> choices = new ArrayList<ProcessComposition<E>>(0);
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#getAfterTop()
     */
    @Override
    public Process<E> getReactedProcess() {
        return this.process;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#setAfterTop(petruchio.interfaces.pi.Process)
     */
    @Override
    public void setReactedProcess(final Process<E> p) {
        this.process = p;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#setTransition(petruchio.interfaces.petrinet.Transition)
     */
    @Override
    public void setTransition(final Transition t) {
    	this.trans = t;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#getTransition()
     */
    @Override
    public Transition getTransition() {
    	return this.trans;
    }

    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "" + this.process;
    }


    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#isSequentiallyIndependent()
     */
    @Override
    public Independent isSequentiallyIndependent() {
        return this.isSequentiallyIndependent;
    }


    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#setSequentiallyIndependent(petruchio.interfaces.algorithms.Reaction.Independent)
     */
    @Override
    public void setSequentiallyIndependent(final Independent isSequentiallyIndependent) {
        this.isSequentiallyIndependent = isSequentiallyIndependent;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#addChoice(petruchio.interfaces.pi.ProcessComposition)
     */
    @Override
    public void addChoice(final ProcessComposition<E> choice) {
        this.choices.add(choice);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#getChoices()
     */
    @Override
    public Collection<ProcessComposition<E>> getChoices() {
        return this.choices;
    } 
    
}