/*
 * Project:  Petruchio
 * Filename: SingleReaction.java
 * Created:  16.12.2006
 * Author:   Tim Strazny
 */
package petruchio.compiler.reactions;

import java.util.Iterator;
import java.util.List;

import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.ProcessID;


/**
 * <Description>
 * 
 * @param <E>
 * @author Tim Strazny
 */
public class SingleReaction<E> extends Reaction<E> implements
                petruchio.interfaces.algorithms.SingleReaction<E> {
    
    /**
     * 
     */
    private ProcessID pid = null;
    
    /**
     * 
     */
    private petruchio.interfaces.algorithms.SingleReaction.StepType type = null;
    
    /**
     * 
     */
    private List<Name> namesBefore = null;
    
    /**
     * 
     */
    private List<Name> namesAfter = null;
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.Reaction#getType()
     */
    @Override
    public petruchio.interfaces.algorithms.Reaction.Type getType() {
        return petruchio.interfaces.algorithms.Reaction.Type.SINGLE;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#getNamesBefore()
     */
    @Override
    public List<Name> getNamesBefore() {
        return this.namesBefore;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#getNamesAfter()
     */
    @Override
    public List<Name> getNamesAfter() {
        return this.namesAfter;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#getType()
     */
    @Override
    public petruchio.interfaces.algorithms.SingleReaction.StepType getStepType() {
        return this.type;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#setType(petruchio.interfaces.algorithms.SingleReaction.StepType)
     */
    @Override
    public void setStepType(final petruchio.interfaces.algorithms.SingleReaction.StepType t) {
        this.type = t;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#setNamesAfter(java.util.List)
     */
    @Override
    public void setNamesAfter(final List<Name> names) {
        this.namesAfter = names;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#setNamesBefore(java.util.List)
     */
    @Override
    public void setNamesBefore(final List<Name> names) {
        this.namesBefore = names;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#getProcessID()
     */
    @Override
    public ProcessID getProcessID() {
        return this.pid;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SingleReaction#setProcessID(petruchio.interfaces.pi.ProcessID)
     */
    @Override
    public void setProcessID(final ProcessID pid) {
        this.pid = pid;
    }

    
    /* (non-Javadoc)
     * @see petruchio.compiler.reaction.Reaction#toString()
     */
    @Override
    public String toString() {
        if(this.getStepType() == null) {
            return "";
        }
        switch(this.getStepType()) {
            case TAU:
                return "tau";
                
            case RECURSE:
                final StringBuilder buf = new StringBuilder();
                final Iterator<Name> b = this.getNamesBefore().iterator();
                final Iterator<Name> a = this.getNamesAfter().iterator();
                buf.append(this.getProcessID().getID());
                buf.append("(");
                while(b.hasNext()) {
                    buf.append(a.next());
                    buf.append("/");
                    buf.append(b.next());
                    if(b.hasNext()) {
                        buf.append(",");
                    }
                }
                buf.append(")");
                return buf.toString();
                
            default:
                throw new InternalError("Unknown type of reaction: " + this.getType());
        }
    }

}