/*
 * Project:  Petruchio
 * Filename: LDigraph.java
 * Created:  13.01.2010
 * Author:   Tim Strazny
 */
package petruchio.compiler.struct;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class LDigraph<T extends Comparable<T>> implements Iterable<LDigraph<T>> {
    private int index = -1;
    private int size = 1;
    private final T label;
    private Collection<LDigraph<T>> children = new ArrayList<LDigraph<T>>(0);
    
    public void setSize(final int size) {
        this.size = size;
    }
    
    public int getSize() {
        return this.size;
    }
    
    public void setIndex(final int index) {
        this.index = index;
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public LDigraph(final T label) {
        this.label = label;
    }
    
    public void clear() {
        this.children.clear();
    }
    
    public T getLabel() {
        return this.label;
    }
    
    public void addChild(final LDigraph<T> child) {
        this.children.add(child);
    }
    
    public boolean removeChild(final LDigraph<T> child) {
        return this.children.remove(child);
    }
    
    public void addChildren(final LDigraph<T> child) {
        this.children.addAll(child.children);
    }
    
    public boolean removeChildren(final LDigraph<T> child) {
        return this.children.removeAll(child.children);
    }
    
    public boolean hasChild(final LDigraph<T> child) {
        return this.children.contains(child);
    }
    
    public boolean isLeaf() {
        return this.children.isEmpty();
    }
    
    public boolean hasChildren() {
        return !this.children.isEmpty();
    }
    
    public int outDegree() {
        return this.children.size();
    }
    
    @Override
    public Iterator<LDigraph<T>> iterator() {
        return this.children.iterator();
    }
    
    public Iterator<LDigraph<T>> children() {
        return this.iterator();
    }
    
    @Override
    public boolean equals(final Object o) {
        return o == this;
    }
    
    @Override
    public String toString() {
        final StringBuilder a = new StringBuilder(this.label == null ? "null" : this.label.toString());
        a.append('[');
        for(final Iterator<LDigraph<T>> it = this.iterator(); it.hasNext();) {
            final LDigraph<T> g = it.next();
            a.append(g.label == null ? "null" : g.label.toString());
            if(it.hasNext()) {
                a.append(", ");
            }
        }
        a.append(']');
        return a.toString();
    }
    
}
