/*
 * Project:  Petruchio
 * Filename: Marking.java
 * Created:  18.02.2007
 * Author:   Tim Strazny
 */
package petruchio.cov;


import net.jcip.annotations.Immutable;
import petruchio.interfaces.petrinet.Place;

/**
 * <Description>
 * 
 * @author Tim Strazny
 */
@Immutable
public class Marking implements Comparable<Marking> {
    
    /**
     * <Description>
     * 
     * @author Tim Strazny
     */
    public static enum Comparison { INVALID, LESS, EQUAL, GREATER, UNKNOWN }
    
    /** */
    public static final int OMEGA = Integer.MAX_VALUE;
    
    /** */
    private final Place place;
    
    /** */
    private final int marking;
    
    
    /**
     * @param place
     * @param marking
     */
    public Marking(final Place place, final int marking) {
        this.place = place;
        this.marking = marking;
    }
    
    
    /**
     * @return the marking
     */
    public int getMarking() {
        return this.marking;
    }
    
    
    /**
     * @return the place
     */
    public Place getPlace() {
        return this.place;
    }
    

    /**
     * @return <code>true</code> iff this marking has value <code>omega</code>.
     */
    public boolean isOmega() {
        return this.marking == Marking.OMEGA;
    }

    
    /**
     * @param m
     * @return
     */
    public Marking.Comparison covCompareTo(final Marking m) {
        if(this.place != m.place) {
            return Marking.Comparison.INVALID;
        } else if(this.marking == m.marking){
            return Marking.Comparison.EQUAL;
        } else if(this.isOmega()) {
            return Marking.Comparison.GREATER;
        } else if(m.isOmega()) {
            return Marking.Comparison.LESS;
        } else if(this.marking < m.marking) {
            return Marking.Comparison.LESS;
        } else {
            return Marking.Comparison.GREATER;
        }
    }
    
    public Marking.Comparison covCompareToNoOmega(final Marking m) {
        if(this.place != m.place) {
            return Marking.Comparison.INVALID;
        } else if(this.marking == m.marking){
            return Marking.Comparison.EQUAL;
        } else if(this.marking < m.marking) {
            return Marking.Comparison.LESS;
        } else {
            return Marking.Comparison.GREATER;
        }
    }
    

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(final Marking o) {
        return PlaceComparator.getInstance().compare(this.getPlace(), o.getPlace());
    }

    

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return this.place.getName() + "[" + (this.isOmega() ? "w" : String.valueOf(this.marking)) + "]"; 
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return 31*(31*17+System.identityHashCode(this.place)) + this.marking;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object o) {
        return (o instanceof Marking) && this.equals((Marking)o);
    }
    
    
    /**
     * TODO: document
     *
     * @param m
     * @return
     */
    public boolean equals(final Marking m) {
        return (m == this) || ((this.place == m.place) && (this.marking == m.marking));
    }
    
}