/*
 * Project:  Petruchio
 * Filename: Node.java
 * Created:  18.02.2007
 * Author:   Tim Strazny
 */
package petruchio.cov;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class Node {
    
    private static final PlaceComparator COMPARATOR = PlaceComparator.getInstance();
    
    SimpleList<Transition> path = null;
    
    final List<Marking> places;
    
    int tokenCount;
    
//    int maxTokens;
    
    int transitionDistance;
    
    int boundedPlaces;
    
    int distToBounds;
    
    int preTrans;
    
    boolean accelerated;
    
    int hash;
    
    public static Node newNode() {
        return new Node();
    }
    
    public void copyFromNode(final Node n) {
        this.tokenCount = n.tokenCount;
//        this.maxTokens = n.maxTokens;
        this.boundedPlaces = n.boundedPlaces;
        this.distToBounds = n.distToBounds;
        this.transitionDistance = n.transitionDistance;
        this.hash = n.hash;
        this.path = n.path;
        this.preTrans = n.preTrans;
        this.accelerated = n.accelerated;
        this.places.clear();
        this.places.addAll(n.places);
    }
    
    public static Node newNode(final Node n) {
        return new Node(n);
    }
    
    protected Node() {
        this.places = new ArrayList<Marking>();
        this.clear();
    }
    
    protected Node(final Node n) {
      this.places = new ArrayList<Marking>(n.places);
//      this.maxTokens = n.maxTokens;
      this.tokenCount = n.tokenCount;
      this.boundedPlaces = n.boundedPlaces;
      this.distToBounds = n.distToBounds;
      this.transitionDistance = n.transitionDistance;
      this.hash = n.hash;
      this.path = n.path;
      this.preTrans = n.preTrans;
      this.accelerated = n.accelerated;
  }

    
    /**
     * @return
     */
    public List<Marking> getMarking() {
        return this.places;
    }
    
    
    /**
     * @return
     */
    public int getBoundedPlaces() {
        return this.boundedPlaces;
    }
    
    public int getDistanceToBounds() {
        return this.distToBounds;
    }
    
    public void clearLists() {
        this.places.clear();
    }
    
    /**
     * 
     */
    public void clear() {
        this.places.clear();
        this.hash = 0;
//        this.maxTokens = 0;
        this.tokenCount = 0;
        this.boundedPlaces = 0;
        this.distToBounds = 0;
        this.preTrans = 0;
        this.accelerated = false;
        this.transitionDistance = -1;
    }
    
    
    public int getTokens() {
        return this.tokenCount;
    }
    
//    public int getMaxTokens() {
//        if(this.maxTokens < 0) {
//            this.recalculateMaxTokens();
//        }
//        return this.maxTokens;
//    }
    
    static <T> int binarySearch(List<? extends Comparable<? super T>> list, T key, int low) {
        return binarySearch(list, key, low, list.size()-1);
    }
    
    static <T> int binarySearch(List<? extends Comparable<? super T>> list, T key, int low, int high) {
        while(low <= high) {
            int mid = (low + high) >>> 1;
            Comparable<? super T> midVal = list.get(mid);
            int cmp = midVal.compareTo(key);

            if(cmp < 0) {
                low = mid + 1;
            } else if(cmp > 0) {
                high = mid - 1;
            } else {
                return mid; // key found
            }
        }
        return -(low + 1); // key not found
    }
    
    
    public Node add(final Node n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index < 0) {
                this.tokenCount += m.getMarking();
//                if(m.getMarking() > this.maxTokens) {
//                    this.maxTokens = m.getMarking();
//                }
                if(m.getPlace().boundKnown()) {
                    ++this.boundedPlaces;
                    this.distToBounds += m.getPlace().getBound() - m.getMarking();
                }
                index = -(index+1);
                this.places.add(index, m);
                this.hash = 0;
                this.preTrans += m.getPlace().getInput().size();
            } else {
                final Marking myM = this.places.get(index);
                this.tokenCount += m.getMarking();
                if(m.getPlace().boundKnown()) {
                    this.distToBounds += myM.getMarking() - m.getMarking();
                }
                final int mark = myM.getMarking() + m.getMarking();
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                this.places.set(index, new Marking(m.getPlace(), mark));
                this.hash = 0;
            }
        }
        return this;
    }
    
    public boolean addWithBoundsChecking(final Node n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index < 0) {
                if(m.getPlace().boundKnown() && m.getPlace().getBound() < m.getMarking()) {
                    return false;
                }
                this.tokenCount += m.getMarking();
//                if(m.getMarking() > this.maxTokens) {
//                    this.maxTokens = m.getMarking();
//                }
                index = -(index+1);
                this.places.add(index, m);
                if(m.getPlace().boundKnown()) {
                    ++this.boundedPlaces;
                    this.distToBounds += m.getPlace().getBound() - m.getMarking();
                }
                this.preTrans += m.getPlace().getInput().size();
                this.hash = 0;
            } else {
                final Marking myM = this.places.get(index);
                final int mark = myM.getMarking() + m.getMarking();
                if(m.getPlace().boundKnown() && m.getPlace().getBound() < mark) {
                    return false;
                }
                this.tokenCount += m.getMarking();
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                if(m.getPlace().boundKnown()) {
                    this.distToBounds += myM.getMarking() - m.getMarking();
                }
                this.places.set(index, new Marking(m.getPlace(), mark));
                this.hash = 0;
            }
        }
        return true;
    }
    
    public Node add(final Node n, final int mult) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index < 0) {
                final int mark = mult*m.getMarking();
                this.tokenCount += mark;
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                if(m.getPlace().boundKnown()) {
                    ++this.boundedPlaces;
                    this.distToBounds += m.getPlace().getBound() - mark;
                }
                index = -(index+1);
                this.places.add(index, new Marking(m.getPlace(), mark));
                this.hash = 0;
                this.preTrans += m.getPlace().getInput().size();
            } else {
                final Marking myM = this.places.get(index);
                this.tokenCount += mult*m.getMarking();
                if(m.getPlace().boundKnown()) {
                    this.distToBounds += myM.getMarking() - mult*m.getMarking();
                }
                final int mark = myM.getMarking() + mult*m.getMarking();
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                this.places.set(index, new Marking(m.getPlace(), mark));
                this.hash = 0;
            }
        }
        return this;
    }
    
    public boolean addWithBoundsChecking(final Node n, final int mult) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index < 0) {
                final int mark = mult*m.getMarking();
                if(m.getPlace().boundKnown() && m.getPlace().getBound() < mark) {
                    return false;
                }
                this.tokenCount += mark;
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                index = -(index+1);
                this.places.add(index, new Marking(m.getPlace(), mark));
                if(m.getPlace().boundKnown()) {
                    ++this.boundedPlaces;
                    this.distToBounds += m.getPlace().getBound() - mark;
                }
                this.preTrans += m.getPlace().getInput().size();
                this.hash = 0;
            } else {
                final Marking myM = this.places.get(index);
                final int mark = myM.getMarking() + mult*m.getMarking();
                if(m.getPlace().boundKnown() && m.getPlace().getBound() < mark) {
                    return false;
                }
                this.tokenCount += mult*m.getMarking();
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                if(m.getPlace().boundKnown()) {
                    this.distToBounds += myM.getMarking() - mult*m.getMarking();
                }
                this.places.set(index, new Marking(m.getPlace(), mark));
                this.hash = 0;
            }
        }
        return true;
    }
    
    public void subtractInitial() {
//        this.maxTokens = 0;
        for(int index = 0; index < this.places.size(); ++index) {
            final Marking m = this.places.get(index);
            final int init = m.getPlace().getMarking(); 
            if(init > 0) {
                if(init > m.getMarking()) {
                    this.tokenCount -= m.getMarking();
                    this.places.remove(index);
                    if(m.getPlace().boundKnown()) {
                        --this.boundedPlaces;
                        this.distToBounds -= m.getPlace().getBound() - m.getMarking();
                    }
                    this.preTrans -= m.getPlace().getInput().size();
                } else {
                    this.tokenCount -= init;
                    if(m.getPlace().boundKnown()) {
                        this.distToBounds -= m.getMarking() - init;
                    }
                    this.places.set(index, new Marking(m.getPlace(), m.getMarking() - init));
                }
                this.hash = 0;
            }
//            if(m.getMarking() > maxTokens) {
//                this.maxTokens = m.getMarking();
//            }
        }
    }
    
    /**
     * @param n
     */
    public boolean subtract(final Node n) {
        boolean exactMinus = true;
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index >= 0) {
                final Marking myM = this.places.get(index);
                final int mark = myM.getMarking()-m.getMarking();
                if(mark <= 0) {
                    if(exactMinus && mark < 0) {
                        exactMinus = false;
                    }
                    this.tokenCount -= myM.getMarking();
                    this.places.remove(index);
                    if(m.getPlace().boundKnown()) {
                        --this.boundedPlaces;
                        this.distToBounds -= m.getPlace().getBound() - myM.getMarking();
                    }
                    this.preTrans -= m.getPlace().getInput().size();
                } else if(mark > 0) {
                    this.tokenCount -= m.getMarking();
                    if(m.getPlace().boundKnown()) {
                        this.distToBounds -= myM.getMarking() - m.getMarking();
                    }
                    this.places.set(index, new Marking(m.getPlace(), mark));
                }
                this.hash = 0;
//                if(myM.getMarking() == this.maxTokens) {
//                    this.maxTokens = -1;
//                }
            } else {
                index = -(index+1);
                exactMinus = false;
            }
        }
        return exactMinus;
    }
    
    public boolean subtract(final Node n, final int mult) {
        int index = 0;
        boolean exactMinus = true;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index >= 0) {
                final Marking myM = this.places.get(index);
                final int mark = myM.getMarking()-mult*m.getMarking();
                if(mark <= 0) {
                    if(exactMinus && mark < 0) {
                        exactMinus = false;
                    }
                    this.tokenCount -= myM.getMarking();
                    this.places.remove(index);
                    if(m.getPlace().boundKnown()) {
                        --this.boundedPlaces;
                        this.distToBounds -= m.getPlace().getBound() - myM.getMarking();
                    }
                    this.preTrans -= m.getPlace().getInput().size();
                } else if(mark > 0) {
                    this.tokenCount -= mult*m.getMarking();
                    if(m.getPlace().boundKnown()) {
                        this.distToBounds -= myM.getMarking() - mult*m.getMarking();
                    }
                    this.places.set(index, new Marking(m.getPlace(), mark));
                }
//                if(myM.getMarking() == this.maxTokens) {
//                    this.maxTokens = -1;
//                }
                this.hash = 0;
            } else {
                index = -(index+1);
                exactMinus = false;
            }
        }
        return exactMinus;
    }
    
    public Node minimize(final Node n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index >= 0) {
                final Marking myM = this.places.get(index);
                if(m.getMarking() < myM.getMarking()) {
//                    if(myM.getMarking() == this.maxTokens) {
//                        this.maxTokens = -1;
//                    }
                    this.tokenCount -= myM.getMarking() - m.getMarking();
                    if(m.getPlace().boundKnown()) {
                        this.distToBounds -= m.getMarking() - myM.getMarking();
                    }
                    this.places.set(index, new Marking(m.getPlace(), m.getMarking()));
                }
                this.hash = 0;
            } else {
                index = -(index+1);
            }
        }
        return this;
    }
    
    public Node maximize(final Node n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index >= 0) {
                final Marking myM = this.places.get(index);
                if(m.getMarking() > myM.getMarking()) {
                    this.tokenCount += m.getMarking() - myM.getMarking();
                    if(m.getPlace().boundKnown()) {
                        this.distToBounds += myM.getMarking() - m.getMarking();
                    }
                    this.places.set(index, new Marking(m.getPlace(), m.getMarking()));
                }
                this.hash = 0;
            } else {
                this.tokenCount += m.getMarking();
                if(m.getPlace().boundKnown()) {
                    ++this.boundedPlaces;
                    this.distToBounds += m.getPlace().getBound() - m.getMarking();
                }
                index = -(index+1);
                this.places.add(index, new Marking(m.getPlace(), m.getMarking()));
                this.hash = 0;
                this.preTrans += m.getPlace().getInput().size();
            }
//            if(m.getMarking() > this.maxTokens) {
//                this.maxTokens = m.getMarking();
//            }
        }
        return this;
    }
    
    
    public void checkTokenCount() {
        int t = 0;
        for(final Marking m : this.places) {
            t += m.getMarking();
        }
        if(t != this.tokenCount) {
            throw new RuntimeException(t + " != " + this.tokenCount);
        }
    }
    
    
//    public void checkMaxTokens() {
//        int t = 0;
//        for(final Marking m : this.places) {
//            if(m.getMarking() > t) {
//                t = m.getMarking();
//            }
//        }
//        if(t != this.maxTokens) {
//            throw new RuntimeException(t + " != " + this.maxTokens);
//        }
//    }
    
    
    public Node subtractWithSanityCheck(final Node n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index >= 0) {
                final Marking myM = this.places.get(index);
//                if(myM.getMarking() == this.maxTokens) {
//                    this.maxTokens = -1; 
//                }
                final int mark = myM.getMarking()-m.getMarking();
                this.tokenCount -= m.getMarking();
                if(mark == 0) {
                    this.places.remove(index);
                    if(m.getPlace().boundKnown()) {
                        --this.boundedPlaces;
                        this.distToBounds -= m.getPlace().getBound() - m.getMarking();
                    }
                    this.preTrans -= m.getPlace().getInput().size();
                } else if(mark > 0) {
                    if(m.getPlace().boundKnown()) {
                        this.distToBounds -= myM.getMarking() - m.getMarking();
                    }
                    this.places.set(index, new Marking(m.getPlace(), mark));
                } else if(mark < 0) {
                    throw new RuntimeException("Cannot subtract " + n + " from " + this + ", Problem: " + m);
                }
                this.hash = 0;
            } else {
                throw new RuntimeException("Cannot subtract " + n + " from " + this + ", Problem: " + m);
            }
        }
        return this;
    }
    
    
    public int semisubtract(final Place p, final int t) {
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            final Marking myM = this.places.get(index);
            if(!myM.isOmega()) {
                final int mark = (Marking.OMEGA == t) ? Marking.OMEGA : (myM.getMarking() - t);
                if(mark <= 0) {
                    this.hash = 0;
                    this.places.remove(index);
                    if(p.boundKnown()) {
                        --this.boundedPlaces;
                        this.distToBounds -= p.getBound() - myM.getMarking();
                    }
                    this.preTrans -= p.getInput().size();
                    this.tokenCount -= myM.getMarking();
//                    if(myM.getMarking() == this.maxTokens) {
//                        this.maxTokens = -1;
//                    }
                    return myM.getMarking();
                }
                this.tokenCount -= t; 
                if(p.boundKnown()) {
                    this.distToBounds += p.getBound() + t;
                }
                this.places.set(index, new Marking(p, mark));
                this.hash = 0;
//                if(myM.getMarking() == this.maxTokens) {
//                    this.maxTokens = -1;
//                }
            }
            return t;
        }
        return 0;
    }
    
    public void subtract(final Place p, final int t) {
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            final Marking myM = this.places.get(index);
            if(!myM.isOmega()) {
                final int mark = (Marking.OMEGA == t) ? Marking.OMEGA : (myM.getMarking() - t);
                if(mark < 0) {
                    throw new IllegalArgumentException("Cannot subtract " + p + "=" + t + " from " + this);
                }
                if(mark == 0) {
                    this.hash = 0;
                    this.places.remove(index);
                    if(p.boundKnown()) {
                        --this.boundedPlaces;
                        this.distToBounds -= p.getBound() - myM.getMarking();
                    }
                    this.preTrans -= p.getInput().size();
                    this.tokenCount -= myM.getMarking();
//                    if(myM.getMarking() == this.maxTokens) {
//                        this.maxTokens = -1;
//                    }
                    return;
                }
                this.tokenCount -= t;
                if(p.boundKnown()) {
                    this.distToBounds += p.getBound() + t;
                }
                this.places.set(index, new Marking(p, mark));
//                if(myM.getMarking() == this.maxTokens) {
//                    this.maxTokens = -1;
//                }
                this.hash = 0;
            }
            return;
        }
        throw new IllegalArgumentException("Cannot subtract " + p + "=" + t + " from " + this);
    }
    
    
    public boolean add(final Place p, final int t) {
        if(t == 0) {
            return false;
        }
        if(t < 0) {
            throw new RuntimeException("Will not add negative token number on " + this + ": " + p + " += " + t);
        }
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            final Marking myM = this.places.get(index);
            if(!myM.isOmega()) {
                final int mark = (Marking.OMEGA == t) ? Marking.OMEGA : (myM.getMarking() + t);
                this.tokenCount += t;
//                if(mark > this.maxTokens) {
//                    this.maxTokens = mark;
//                }
                if(p.boundKnown()) {
                    this.distToBounds += p.getBound() - t;
                }
                this.places.set(index, new Marking(p, mark));
                this.hash = 0;
            }
            return true;
        }
        this.preTrans += p.getInput().size();
        this.tokenCount += t; 
//        if(t > this.maxTokens) {
//            this.maxTokens = t;
//        }
        this.places.add(-(index+1), new Marking(p, t));
        if(p.boundKnown()) {
            ++this.boundedPlaces;
            this.distToBounds += p.getBound() - t;
        }
        this.hash = 0;
        return false;
    }
    
    public boolean set(final Place p, final int t) {
        if(t == 0) {
            return this.remove(p) != 0;
        }
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            final Marking myM = this.places.get(index);
            if(!myM.isOmega()) {
                this.tokenCount += t - myM.getMarking();
                if(p.boundKnown()) {
                    this.distToBounds += p.getBound() - (t-myM.getMarking());
                }
//                if(t > this.maxTokens) {
//                    this.maxTokens = t;
//                }
                this.places.set(index, new Marking(p, t));
//                if(myM.getMarking() == this.maxTokens) {
//                    this.maxTokens = -1;
//                }
                this.hash = 0;
            }
            return true;
        }
        this.preTrans += p.getInput().size();
        this.tokenCount += t;
//        if(t > this.maxTokens) {
//            this.maxTokens = t;
//        }
        this.places.add(-(index+1), new Marking(p, t));
        if(p.boundKnown()) {
            ++this.boundedPlaces;
            this.distToBounds += p.getBound() - t;
        }
        this.hash = 0;
        return false;
    }
    
    
    public boolean add(final Place p) {
        return this.add(p, 1);
    }
    
    
    public int remove(final Place p) {
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            this.hash = 0;
            final Marking myM = this.places.remove(index);
            if(p.boundKnown()) {
                --this.boundedPlaces;
                this.distToBounds -= p.getBound() - myM.getMarking();
            }
            this.preTrans -= p.getInput().size();
            this.tokenCount -= myM.getMarking();
//            if(myM.getMarking() == this.maxTokens) {
//                this.maxTokens = -1;
//            }
            return myM.getMarking();
        }
        return 0;
    }
    
    
//    private void recalculateMaxTokens() {
//        this.maxTokens = 0;
//        for(final Marking m : this.places) {
//            if(m.getMarking() > this.maxTokens) {
//                this.maxTokens = m.getMarking();
//            }
//        }
//    }
    
    
    /**
     * @param list
     * @param key
     * @return
     */
    static int binarySearch(final List<Marking> list, final Place key) {
        int low = 0;
        int high = list.size() - 1;

        while(low <= high) {
            int mid = (low + high) >>> 1;
            Marking midVal = list.get(mid);
            int cmp = COMPARATOR.compare(midVal.getPlace(), key);

            if(cmp < 0) {
                low = mid + 1;
            } else if(cmp > 0) {
                high = mid - 1;
            } else {
                return mid; // key found
            }
        }
        return -(low + 1); // key not found
    }


    private boolean quickEquals(final Node n, int i, int j) {
        final int size = this.places.size();
        if(size - i != n.places.size() - j) {
            return false;
        }
        while(i < size) {
            if(!this.places.get(i).equals(n.places.get(j))) {
                return false;
            }
            ++i;
            ++j;
        }
        return true;
    }
    
    public boolean greaterEquals(final Node n) {
    	int thisTokens = this.tokenCount;
        int thatTokens = n.tokenCount;
        if(thisTokens == thatTokens) {
            // check for equality
            return quickEquals(n, 0, 0);
        }
        // assert thisTokens != thatTokens
    	if(thisTokens < thatTokens) { //|| (this.maxTokens >= 0 && n.maxTokens >= 0 && this.maxTokens < n.maxTokens)) {
    		return false;
    	}
        // assert thisTokens > thatTokens
        final List<Marking> thismarkings = this.places;
        final List<Marking> thatmarkings = n.places;
        final int thisPlaces = thismarkings.size();
        final int thatPlaces = thatmarkings.size();
        if(thisPlaces < thatPlaces) {
        	return false;
        }
        // assert thisTokens > thatTokens && thisPlaces >= thatPlaces
        
        int i = 0;
        int j = 0;
        while((i < thisPlaces) && (j < thatPlaces)) {
            Marking a = thismarkings.get(i);
            thisTokens -= a.getMarking();
            final Marking b = thatmarkings.get(j);
            thatTokens -= b.getMarking();
            
            if(thisTokens < thatTokens) {
                return false;
            }
            // assert thisTokens >= thatTokens
            if(thisTokens == thatTokens) {
                if(a.getPlace() != b.getPlace()) {
                    return false;
                }
                return quickEquals(n, i+1, j+1);
            }
            // assert thisTokens > thatTokens
            
            while((a.getPlace() != b.getPlace())) {
                if(PlaceComparator.getInstance().isLess(a.getPlace(), b.getPlace())) {
                    // the current place a is not contained in n
                    ++i;
                    if(i < thisPlaces) {
                        a = thismarkings.get(i);
                        thisTokens -= a.getMarking();
                        if(thisTokens < thatTokens) {
                            return false;
                        }
                        // assert thisTokens >= thatTokens
                        if(thisPlaces - i < thatPlaces - j) {
                            // not all remaining places covered
                            return false;
                        }
                        // assert thisTokens >= thatTokens && thisPlaces - i >= thatPlaces - j
                    } else {
                        // current place b is not contained in this
                        return false;
                    }
                    
                } else {
                    // the current place b is not contained in this
                    return false;
                }
            }
            
            // assert thisTokens > thatTokens && thisplaces - i >= thatplaces - j
            // assert a.getPlace() == b.getPlace();
            
            if(a.getMarking() < b.getMarking()) {
                return false;
            }
            
            // assert thisTokens > thatTokens && thisplaces - i >= thatplaces - j
            // assert a.getPlace() == b.getPlace();
            // assert a.getMarking() >= b.getMarking()
            // place b covered by a
                        
            ++i;
            ++j;
        }
        
        return true;
    }
    
    public boolean greaterEqualsIgnoreParametricPlaces(final Node n) {
        final List<Marking> thismarkings = this.places;
        final List<Marking> thatmarkings = n.places;
        final int thisPlaces = thismarkings.size();
        final int thatPlaces = thatmarkings.size();
        
        int i = 0;
        int j = 0;
        while((i < thisPlaces) && (j < thatPlaces)) {
            Marking a = thismarkings.get(i);
            final Marking b = thatmarkings.get(j);
            while((a.getPlace() != b.getPlace())) {
                if(PlaceComparator.getInstance().isLess(a.getPlace(), b.getPlace())) {
                    // the current place a is not contained in n
                    ++i;
                    if(i < thisPlaces) {
                        a = thismarkings.get(i);
                    } else if(!b.getPlace().isParametric()) {
                        // current place b is not contained in this
                        return false;
                    }
                    
                } else if(!b.getPlace().isParametric()) {
                    // the current place b is not contained in this
                    return false;
                }
            }
            
            if(!a.getPlace().isParametric() && a.getMarking() < b.getMarking()) {
                return false;
            }
            
            // assert thisTokens > thatTokens && thisplaces - i >= thatplaces - j
            // assert a.getPlace() == b.getPlace();
            // assert a.getMarking() >= b.getMarking()
            // place b covered by a
                        
            ++i;
            ++j;
        }
        
        while(j < thatPlaces) {
            final Marking b = thatmarkings.get(j);
            if(!b.getPlace().isParametric()) {
                return false;
            }
            ++j;
        }
        
        return true;
    }
    
    
    public int divide(final Node n) {
        int thisTokens = this.tokenCount;
        int thatTokens = n.tokenCount;
        if(thisTokens < thatTokens) { // || (this.maxTokens >= 0 && n.maxTokens >= 0 && this.maxTokens < n.maxTokens)) {
            return 0;
        }
        // assert thisTokens >= thatTokens
        if(thisTokens == thatTokens) {
            // check for equality
            return quickEquals(n, 0, 0) ? 1 : 0;
        }
        // assert thisTokens > thatTokens
        final List<Marking> thismarkings = this.places;
        final List<Marking> thatmarkings = n.places;
        final int thisPlaces = thismarkings.size();
        final int thatPlaces = thatmarkings.size();
        if(thisPlaces < thatPlaces) {
            return 0;
        }
        // assert thisTokens > thatTokens && thisplaces >= thatplaces
        
        int mult = -1;
        int i = 0;
        int j = 0;
        while((i < thisPlaces) && (j < thatPlaces)) {
            Marking a = thismarkings.get(i);
            thisTokens -= a.getMarking();
            final Marking b = thatmarkings.get(j);
            thatTokens -= b.getMarking();
            
            if(thisTokens < thatTokens) {
                return 0;
            }
            // assert thisTokens >= thatTokens
            if(thisTokens == thatTokens) {
                if(a.getPlace() != b.getPlace()) {
                    return 0;
                }
                final int div = a.getMarking() / b.getMarking();
                if(mult < 0 || div < mult) {
                    mult = div;
                }
                if(thisTokens == 0) {
                    return mult;
                }
                return quickEquals(n, i+1, j+1) ? 1 : 0;
            }
            // assert thisTokens > thatTokens
            
            while((a.getPlace() != b.getPlace())) {
                if(PlaceComparator.getInstance().isLess(a.getPlace(), b.getPlace())) {
                    // the current place a is not contained in n
                    ++i;
                    if(i < thisPlaces) {
                        a = thismarkings.get(i);
                        thisTokens -= a.getMarking();
                        if(thisTokens < thatTokens) {
                            return 0;
                        }
                        // assert thisTokens >= thatTokens
                        if(thisPlaces - i < thatPlaces - j) {
                            // not all remaining places covered
                            return 0;
                        }
                        // assert thisTokens >= thatTokens && thisPlaces - i >= thatPlaces - j
                    } else {
                        // current place b is not contained in this
                        return 0;
                    }
                    
                } else {
                    // the current place b is not contained in this
                    return 0;
                }
            }
            
            // assert thisTokens > thatTokens && thisplaces - i >= thatplaces - j
            // assert a.getPlace() == b.getPlace();
            
            if(a.getMarking() < b.getMarking()) {
                return 0;
            }
            
            // assert thisTokens > thatTokens && thisplaces - i >= thatplaces - j
            // assert a.getPlace() == b.getPlace();
            // assert a.getMarking() >= b.getMarking()
            // place b covered by a
                        
            ++i;
            ++j;
            
            final int div = a.getMarking() / b.getMarking();
            if(mult < 0 || div < mult) {
                mult = div;
            }
        }
        
        return mult < 0 ? 0 : mult;
    }
        

    
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        if(this.hash == 0) {
//            this.hash ^= this.omegaCount;
//            this.hash ^= this.tokenCount;
            final int size = this.places.size();
            this.hash = 17;
            for(int i = 0; i < size; ++i) {
                this.hash = 31*this.hash + this.places.get(i).hashCode();
            }
        }
        return this.hash;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object o) {
        return (o instanceof Node) && this.equals((Node)o);
    }
    
    
    public boolean equals(final Node n) {
        if(n == this) {
            return true;
        }
        if(n.tokenCount != this.tokenCount) {
            return false;
        }
        return this.quickEquals(n, 0, 0);
    }
    
    
    public Marking violatesBound() {
        for(final Marking m : this.places) {
            if(m.getPlace().boundKnown() && m.getPlace().getBound() < m.getMarking()) {
                return m;
            }
        }
        return null;
    }
    
    
    public Marking getMarking(final Place p) {
        final int index = Node.binarySearch(this.places, p);
        if(index < 0) {
            return null;
        }
        return this.places.get(index);
    }
    
    
    public Collection<Place> toMultiSet() {
        final Collection<Place> ms = new ArrayList<Place>();
        for (final Marking m : this.places) {
            if (m.isOmega()) {
                throw new IllegalArgumentException("Cannot build multi set from node with omega entries.");
            }
            for (int i = m.getMarking(); i > 0; --i) {
                ms.add(m.getPlace());
            }
        }
        return ms;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        final int size = this.places.size();
        final StringBuilder b = new StringBuilder();
        b.append("(");
        b.append(this.tokenCount);
        b.append(", ");
        for(int i = 0; i < size; ++i) {
            final Marking m = this.places.get(i);
            if(!m.isOmega() && m.getMarking() != 1) {
                b.append(m.isOmega() ? "w" : "" + m.getMarking());
                b.append('*');
            }
            //b.append(System.identityHashCode(m.getPlace()));
            b.append(m.getPlace());
            if(i < size-1) {
                b.append('+');
            }
        }
        b.append(")");
        return b.toString();
    }
    
    public int getTransitionDistance() {
        return this.transitionDistance;
    }

    public void setTransitionDistance(int transitionDistance) {
        this.transitionDistance = transitionDistance;
    }
    
}
