/*
 * File:    SemiBackward.java
 * Project: petruchio
 * Author:  Tim Strazny
 * Created: 15.03.2009
 */
package petruchio.cov;


/**
 * Semi-decision procedure for coverability.
 * It tries to disprove the coverability of a marking but takes a shortcut to the
 * "marking is coverable" result when the search inspects "too many" states.
 * 
 * @author Tim Strazny
 */
public class SemiBackward extends Backward {

    /* (non-Javadoc)
     * @see petruchio.cov.Backward#shortcutIsCoverable(petruchio.cov.Node)
     */
    @Override
    synchronized boolean shortcutIsCoverable(final Node marking) {
        return this.markingsToProcess.size() > this.transitionCount + 10 + (this.transitionCount*marking.getTokens() >> 2);
    }
    
    /* (non-Javadoc)
     * @see petruchio.cov.Backward#shortcutIsNotCoverable(petruchio.cov.Node)
     */
    @Override
    boolean shortcutIsNotCoverable(final Node marking) {
        return false;
    }
    
    /* (non-Javadoc)
     * @see petruchio.cov.Backward#shortcutDoNotGenerateMorePredecessors(petruchio.cov.Node)
     */
    @Override
    boolean shortcutDoNotGenerateMorePredecessors(final Node marking) {
        return false;
    }
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "Searches backward from possible communication pairs if a covering "
            + "marking can be reached from the intial marking. The algorithm does "
            + "not follow every path. When the problem becomes to complicated it "
            + "simply returns _true_ (which is safe for absolutely structurally stationary processes).";
    }
    
}
