package petruchio.cov;

import java.util.Iterator;

import net.jcip.annotations.Immutable;

@Immutable
public class SimpleList<E> implements Iterable<E> {
    private final E value;
    private final int length;
    private final SimpleList<E> next;
    
    public SimpleList(final E value) {
        this(value, null);
    }
    
    public SimpleList(final E value, final SimpleList<E> next) {
        this.value = value;
        this.next = next;
        this.length = next == null ? 1 : next.length + 1;
    }
    
    public static <T> SimpleList<T> prepend(final T value, final SimpleList<T> next) {
        return new SimpleList<T>(value, next);
    }
    
    public SimpleList<E> reverse() {
        SimpleList<E> cur = this;
        SimpleList<E> r = null;
        while(cur != null) {
            r = SimpleList.prepend(cur.getValue(), cur);
            cur = cur.getNext();
        }
        return r;
    }
    
    @Override
    public String toString() {
        final StringBuilder b = new StringBuilder();
        SimpleList<E> cur = this;
        do {
            b.append(cur.value);
            cur = cur.next;
            if(cur != null) {
                b.append(", ");
            }
        } while(cur != null);
        return b.toString();
    }

    @Override
    public Iterator<E> iterator() {
        return new Iterator<E>() {
            SimpleList<E> cur = SimpleList.this;
            
            @Override
            public boolean hasNext() {
                return cur != null;
            }

            @Override
            public E next() {
                final E e = cur.value;
                cur = cur.next;
                return e;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
            
        };
    }

    /**
     * @return
     */
    public int getLength() {
        return this.length;
    }
    
    /**
     * @return
     */
    public E getValue() {
        return this.value;
    }
    
    public SimpleList<E> getNext() {
        return this.next;
    }
    
    public SimpleList<E> prepend(final E e) {
        return new SimpleList<E>(e, this);
    }
}
