package petruchio.cov;

import java.util.ArrayList;
import java.util.Collection;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;

import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;

public class StructuredNode extends Node implements Comparable<StructuredNode> {
    public static final StructuredNode EMPTY = newNode();
    
    private final Map<Transition, StructuredNode> followers;
    
    private final Map<Transition, Collection<StructuredNode>> predecessors;
    
    private boolean isCovered;
    
    int omegaCount;
    
    private final List<Integer> coveredIndex;
    
    private final List<Place> covered;
    
    private boolean cleared;
    
    public static StructuredNode newNode() {
        return new StructuredNode();
    }
    
    public void copyFromNode(final StructuredNode n) {
        super.copyFromNode(n);
        this.omegaCount = n.omegaCount;
        this.isCovered = n.isCovered;
        this.cleared = n.cleared;
    }
    
    public static StructuredNode copyNode(final StructuredNode n) {
        final StructuredNode node = newNode();
        node.copyFromNode(n);
        return node;
    }
    
    StructuredNode() {
        super();
        this.omegaCount = 0;
        this.isCovered = false;
        this.coveredIndex = new ArrayList<Integer>(0);
        this.covered = new ArrayList<Place>(0);
        this.followers = new IdentityHashMap<Transition, StructuredNode>(0);
        this.predecessors = new IdentityHashMap<Transition, Collection<StructuredNode>>(0);
    }

    
    
    /**
     * @return
     */
    public boolean isCovered() {
        return this.isCovered;
    }
    
    
    public boolean addOmega(final Place p) {
        return this.add(p, Marking.OMEGA);
    }
    
    
    /**
     * @param isCovered
     */
    public void setIsCovered(final boolean isCovered) {
        this.isCovered = isCovered;
    }
    
    /**
     * TODO: document
     *
     * @param t
     * @param n
     */
    public void removePredecessor(final Transition t, final StructuredNode n) {
        final Collection<StructuredNode> p = this.predecessors.get(t);
        if(p != null) {
            p.remove(n);
            if(p.isEmpty()) {
                this.predecessors.remove(t);
            }
        }
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     * @param n
     */
    public void addPredecessor(final Transition t, final StructuredNode n) {
        if(this.cleared) {
            throw new RuntimeException();
        }
        Collection<StructuredNode> c = this.predecessors.get(t);
        if(c == null) {
            c = new ArrayList<StructuredNode>();
            this.predecessors.put(t, c);
        }
        c.add(n);
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     * @param n
     */
    public StructuredNode addSuccessor(final Transition t, final StructuredNode n) {
        if(this.cleared) {
            throw new RuntimeException();
        }
        return this.followers.put(t, n);
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void removeSuccessor(final Transition t) {
        this.followers.remove(t);
    }

    
    /**
     * 
     */
    public void clear() {
        this.cleared = true;
        super.clear();
        this.omegaCount = 0;
        for(final Map.Entry<Transition, Collection<StructuredNode>> e : this.predecessors.entrySet()) {
            final Transition t = e.getKey();
            for (final StructuredNode p : e.getValue()) {
                p.removeSuccessor(t);
            }
        }
        this.predecessors.clear();
        for (final Map.Entry<Transition, StructuredNode> e : this.followers.entrySet()) {
            final StructuredNode s = e.getValue();
            s.removePredecessor(e.getKey(), this);
        }
        this.followers.clear();
        this.isCovered = false;
    }
    

    /**
     * TODO: document
     *
     * @return
     */
    public Map<Transition, StructuredNode> getSuccessors() {
        return this.followers;
    }
    
    public boolean hasOmegas() {
        return this.omegaCount != 0;
    }    
    
    public int getOmegas() {
        return this.omegaCount;
    }
    
    
    /**
     * TODO: document
     *
     * @return
     */
    public Map<Transition, Collection<StructuredNode>> getPredecessors() {
        return this.predecessors;
    }
    
    public Marking.Comparison covMaxCompareTo(final StructuredNode n) {
        if(n == this) {
            return Marking.Comparison.EQUAL;
        }
        final List<Marking> m1 = this.places;
        final List<Marking> m2 = n.places;
        final int m1size = m1.size();
        final int m2size = m2.size();
        int thisOmegas = this.omegaCount;
        int thatOmegas = n.omegaCount;
        int thisTokens = this.tokenCount;
        int thatTokens = n.tokenCount;
        // first approximations that have to be proven or disproved
        // if it gets disproved the nodes are not comparable
        boolean greater = (thisOmegas > thatOmegas)
                || (m1size > m2size)
                || ((thisOmegas == thatOmegas)
                                && (thisTokens > thatTokens));
        boolean less = (thisOmegas < thatOmegas)
                || (m1size < m2size)
                || ((thisOmegas == thatOmegas)
                                && (thisTokens < thatTokens));
        
        this.coveredIndex.clear();
        this.covered.clear();
        
        int i = 0;
        int j = 0;
        while((i < m1size) && (j < m2size)) {
            Marking a = m1.get(i);
            if(a.isOmega()) {
                --thisOmegas;
            } else {
                thisTokens -= a.getMarking();
            }
            Marking b = m2.get(j);
            if(b.isOmega()) {
                --thatOmegas;
            } else {
                thatTokens -= b.getMarking();
            }
            
            while((a.getPlace() != b.getPlace())) {
                if(PlaceComparator.getInstance().isLess(a.getPlace(), b.getPlace())) {
                    // the current place a is not contained in n
                    if(less) {
                        return Marking.Comparison.UNKNOWN;
                    }
                    greater = true;
                    ++i;
                    if(i < m1size) {
                        this.coveredIndex.add(Integer.valueOf(i-1));
                        this.covered.add(a.getPlace());
                        
                        a = m1.get(i);
                        if(a.isOmega()) {
                            --thisOmegas;
                        } else {
                            thisTokens -= a.getMarking();
                        }
                    } else if(greater) {
                        return Marking.Comparison.UNKNOWN;
                    } else {
                        return Marking.Comparison.LESS;
                    }
                } else {
                    // the current place b is not contained in this
                    if(greater) {
                        return Marking.Comparison.UNKNOWN;
                    }
                    less = true;
                    ++j;
                    if(j < m2size) {
                        b = m2.get(j);
                        if(b.isOmega()) {
                            --thatOmegas;
                        } else {
                            thatTokens -= b.getMarking();
                        }
                    } else if(less) {
                        return Marking.Comparison.UNKNOWN;
                    } else {
                        return Marking.Comparison.GREATER;
                    }
                }
            }
            
            if(greater &&
                    (((thisOmegas == thatOmegas) && (thisTokens < thatTokens))
                            || (thisOmegas < thatOmegas))) {
                return Marking.Comparison.UNKNOWN;
            }
            if(less &&
                    (((thisOmegas == thatOmegas) && (thisTokens > thatTokens))
                            || (thisOmegas > thatOmegas))) {
                return Marking.Comparison.UNKNOWN;
            }
            
            final Marking.Comparison mcmp = a.covCompareTo(b);
            switch(mcmp) {
                case EQUAL:
                    break;
                case LESS:
                    if(greater) {
                        return Marking.Comparison.UNKNOWN;
                    }
                    less = true;
                    break;
                case GREATER:
                    if(less) {
                        return Marking.Comparison.UNKNOWN;
                    }
                    greater = true;
                    this.coveredIndex.add(Integer.valueOf(i));
                    this.covered.add(b.getPlace());
                    break;
                case INVALID:
                default:
                    throw new InternalError("Unexpected behavior. Dumb.");
            }
            
            ++i;
            ++j;
        }
        
        if(i < m1size) {
            // n has less places than this
            if(less) {
                return Marking.Comparison.UNKNOWN;
            }
            for(; i < m1size; ++i) {
                this.coveredIndex.add(Integer.valueOf(i));
                this.covered.add(m1.get(i).getPlace());
            }
            greater = true;
        } else if(j < m2size) {
            // this has less places than n
            if(greater) {
                return Marking.Comparison.UNKNOWN;
            }
            return Marking.Comparison.LESS;
        }
        
        
        if(greater) {
            for(int k = this.covered.size() - 1; k >= 0; --k) {
                final Place p = this.covered.get(k);
                final int index = this.coveredIndex.get(k).intValue();
                final Marking myM = this.places.get(index);
                if(!myM.isOmega()) {
                    ++this.omegaCount;
                    this.tokenCount -= myM.getMarking();
                    this.places.set(index, new Marking(p, Marking.OMEGA));
                }
            }
            this.hash = 0;
            return Marking.Comparison.GREATER;
        } else if(less) {
            return Marking.Comparison.LESS;
        } else {
            return Marking.Comparison.EQUAL;
        }
    }
    
    public void maxOmegas(final Node n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index < 0) {
                throw new InternalError();
            }
            final Marking myM = this.places.get(index);
            if(!myM.isOmega() && (myM.covCompareTo(m) == Marking.Comparison.GREATER)) {
                ++this.omegaCount;
                this.tokenCount -= myM.getMarking();
                this.places.set(index, new Marking(m.getPlace(), Marking.OMEGA));
                this.hash = 0;
            }
        }
    }
    
    public void add(final StructuredNode n) {
        int index = 0;
        for(final Marking m : n.places) {
            index = binarySearch(this.places, m, index);
            if(index < 0) {
                if(m.isOmega()) {
                    ++this.omegaCount;
                } else {
                    this.tokenCount += m.getMarking();
                    if(m.getPlace().boundKnown()) {
                        ++this.boundedPlaces;
                        this.distToBounds += m.getPlace().getBound() - m.getMarking();
                    }
                }
                index = -(index+1);
                this.places.add(index, m);
                this.hash = 0;
            } else {
                final Marking myM = this.places.get(index);
                if(!myM.isOmega()) {
                    if(m.isOmega()) {
                        ++this.omegaCount;
                        this.tokenCount -= myM.getMarking();
                        if(m.getPlace().boundKnown()) {
                            this.distToBounds -= m.getPlace().getBound() - myM.getMarking();
                        }
                    } else {
                        this.tokenCount += m.getMarking();
                        if(m.getPlace().boundKnown()) {
                            this.distToBounds += myM.getMarking() - m.getMarking();
                        }
                    }
                    final int mark = m.isOmega() ? Marking.OMEGA : (myM.getMarking() + m.getMarking());
                    this.places.set(index, new Marking(m.getPlace(), mark));
                    this.hash = 0;
                }
            }
        }
    }
    
    public boolean add(final Place p, final int t) {
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            final Marking myM = this.places.get(index);
            if(!myM.isOmega()) {
                final int mark = (Marking.OMEGA == t) ? Marking.OMEGA : (myM.getMarking() + t);
                if(t == Marking.OMEGA) {
                    ++this.omegaCount;
                    this.tokenCount -= myM.getMarking();
                } else {
                    this.tokenCount += t; 
                }
                this.places.set(index, new Marking(p, mark));
                this.hash = 0;
            }
            return true;
        }
        if(t == Marking.OMEGA) {
            ++this.omegaCount;
        } else {
            this.tokenCount += t; 
        }
        this.places.add(-(index+1), new Marking(p, t));
        if(p.boundKnown()) {
            ++this.boundedPlaces;
            this.distToBounds += p.getBound() - t;
        }
        this.hash = 0;
        return false;
    }
    
    public int remove(final Place p) {
        final int index = Node.binarySearch(this.places, p);
        if(index >= 0) {
            this.hash = 0;
            final Marking myM = this.places.remove(index);
            if(myM.isOmega()) {
                --this.omegaCount;
                return -1;
            }
            if(p.boundKnown()) {
                --this.boundedPlaces;
                this.distToBounds -= p.getBound() - myM.getMarking();
            }
            this.tokenCount -= myM.getMarking();
            return myM.getMarking();
        }
        return 0;
    }
    
    public boolean enables(final StructuredNode n) {
        assert n.omegaCount == 0 : "A transition cannot request omega many tokens.";
        
        int thisOmegas = this.omegaCount;
        int thisTokens = this.tokenCount;
        int thatTokens = n.tokenCount;
        if((thisOmegas == 0) && (thisTokens < thatTokens)) {
            // no omegas but n requires more token than this node contains
            return false;
        }
        final List<Marking> m1 = this.places;
        final List<Marking> m2 = n.places;
        int m1size = m1.size();
        int m2size = m2.size();
        if(m2size > m1size) {
            return false;
        }
        int i = 0;
        int j = 0;
        while(j < m2size) {
            if((i >= m1size)) {
                return false;
            }
            final Marking toSatisfy = m2.get(j);
            final Place p = toSatisfy.getPlace();
            
            Marking m = m1.get(i);
            while(PlaceComparator.getInstance().isLess(m.getPlace(), p)) {
                ++i;
                if(i >= m1size) {
                    return false;
                }
                m = m1.get(i);
                if(m.isOmega()) {
                    --thisOmegas;
                } else {
                    thisTokens -= m.getMarking();
                }
            }
            
            final Marking.Comparison mcmp = m.covCompareTo(toSatisfy);
            if((mcmp == Marking.Comparison.INVALID) || (mcmp == Marking.Comparison.LESS)) {
                return false;
            }
            
            thatTokens -= toSatisfy.getMarking();
            if((thisOmegas == 0) && (thisTokens < thatTokens)) {
                return false;
            }
            
            ++i;
            ++j;
        }
        return true;
    }
    
    public boolean covers(final StructuredNode n) {
        if(n == this) {
            return true;
        }
        final List<Marking> m1 = this.places;
        final List<Marking> m2 = n.places;
        final int m1size = m1.size();
        final int m2size = m2.size();
        int thisOmegas = this.omegaCount;
        int thatOmegas = n.omegaCount;
        int thisTokens = this.tokenCount;
        int thatTokens = n.tokenCount;
        // first approximations that have to be proven or disproven
        // if it gets disproven the nodes are not comparable
        boolean greater = (thisOmegas > thatOmegas)
                || (m1size > m2size)
                || ((thisOmegas == thatOmegas)
                                && (thisTokens > thatTokens));
        boolean less = (thisOmegas < thatOmegas)
                || (m1size < m2size)
                || ((thisOmegas == thatOmegas)
                                && (thisTokens < thatTokens));
        
        int i = 0;
        int j = 0;
        while((i < m1size) && (j < m2size)) {
            Marking a = m1.get(i);
            if(a.isOmega()) {
                --thisOmegas;
            } else {
                thisTokens -= a.getMarking();
            }
            Marking b = m2.get(j);
            if(b.isOmega()) {
                --thatOmegas;
            } else {
                thatTokens -= b.getMarking();
            }
            
            while((a.getPlace() != b.getPlace())) {
                if(PlaceComparator.getInstance().isLess(a.getPlace(), b.getPlace())) {
                    // the current place a is not contained in n
                    if(less) {
                        return false;
                    }
                    greater = true;
                    ++i;
                    if(i < m1size) {
                        a = m1.get(i);
                        if(a.isOmega()) {
                            --thisOmegas;
                        } else {
                            thisTokens -= a.getMarking();
                        }
                    } else {
                        return false;
                    }
                } else {
                    // the current place b is not contained in this
                    return false;
                }
            }
            
            final Marking.Comparison mcmp = a.covCompareTo(b);
            switch(mcmp) {
                case EQUAL:
                    break;
                case LESS:
                    return false;
                case GREATER:
                    if(less) {
                        return false;
                    }
                    greater = true;
                    break;
                case INVALID:
                default:
                    throw new InternalError("Unexpected behavior. Dumb.");
            }
            
            if(greater &&
                    (((thisOmegas == thatOmegas) && (thisTokens < thatTokens))
                            || (thisOmegas < thatOmegas))) {
                return false;
            }
            if(less &&
                    (((thisOmegas == thatOmegas) && (thisTokens > thatTokens))
                            || (thisOmegas > thatOmegas))) {
                return false;
            }
            
            ++i;
            ++j;
        }
        
        if(i < m1size) {
            // n has less places than this
            return !less;
        } else if(j < m2size) {
            // this has less places than n
            return false;
        }
        
        
        return greater;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(final StructuredNode o) {
        if(o == this) {
            return 0;
        }
        // sort in descending order by omegaCount and by tokenCount
        if(this.omegaCount < o.omegaCount) {
            return 1;
        } else if(this.omegaCount != o.omegaCount) {
            return -1;
        }
        
        if(this.tokenCount < o.tokenCount) {
            return 1;
        } else if (this.tokenCount != o.tokenCount) {
            return -1;
        }
        
        if(this.places.size() < o.places.size()) {
            return 1;
        } else if (this.places.size() != o.places.size()) {
            return -1;
        }
        
        return 0;
    }
    
    
    @Override
    public boolean equals(final Object o) {
        return o instanceof StructuredNode && this.equals((StructuredNode)o);
    }
    
    
    public boolean equals(final StructuredNode n) {
        if(n == this) {
            return true;
        }
        if((n.tokenCount != this.tokenCount) || (n.omegaCount != this.omegaCount)
                || (n.places.size() != this.places.size())) {
            return false;
        }
        int thisTokens = this.tokenCount;
        int thatTokens = n.tokenCount;
        int thisOmegas = this.omegaCount;
        int thatOmegas = n.omegaCount;
        for(int i = this.places.size()-1; i >= 0; --i) {
            final Marking a = this.places.get(i);
            final Marking b = n.places.get(i);
            if(!a.equals(b)) {
                return false;
            }
            if(a.isOmega()) {
                --thisOmegas;
            } else {
                thisTokens -= a.getMarking();
            }
            if(b.isOmega()) {
                --thatOmegas;
            } else {
                thatTokens -= b.getMarking();
            }
            if((thisTokens != thatTokens) || (thisOmegas != thatOmegas)) {
                return false;
            }
        }
        return true;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        final int size = this.places.size();
        final StringBuilder b = new StringBuilder();
        b.append("(");
        b.append(this.omegaCount);
        b.append(", ");
        b.append(this.tokenCount);
        b.append(", ");
        for(int i = 0; i < size; ++i) {
            final Marking m = this.places.get(i);
            if(!m.isOmega() && m.getMarking() != 1) {
                b.append(m.isOmega() ? "w" : "" + m.getMarking());
                b.append('*');
            }
            //b.append(System.identityHashCode(m.getPlace()));
            b.append(m.getPlace());
            if(i < size-1) {
                b.append('+');
            }
        }
        b.append(")");
        return b.toString();
    }
    
}
