/*
 * File:    CommunicatorCache.java
 * Project: petruchio
 * Author:  Tim Strazny
 * Created: 15.03.2009
 */
package petruchio.interfaces;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import petruchio.compiler.ProcessData.NameCard;
import petruchio.compiler.reactions.SingleReactionTask;
import petruchio.interfaces.algorithms.ReactionFinder;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.pi.ActionPrefix;
import petruchio.interfaces.pi.Process;

/**
 * @author Tim Strazny
 */
public interface CommunicatorCache<V> extends Resettable {

    /**
     * @param reactionFinder
     */
    public void setReactionFinder(ReactionFinder<V> reactionFinder);
    
    /**
     * @param p
     * @return
     */
    public Process<V> getProcess(Place p);
    
    /**
     * @param p
     * @param proc
     */
    public void associateWithProcess(Place p, Process<V> proc);
    
    /**
     * @return
     */
    public Set<Map.Entry<Place, Process<V>>> placeProcAssociations();
    
    /**
     * @return
     */
    public Map<Place, Process<V>> placeProcMap();
    
    /**
     * @param p
     */
    public Collection<ActionPrefix> add(SingleReactionTask<V> p);
    
    /**
     * @param p
     */
    public void addAll(Collection<SingleReactionTask<V>> p);
    
    /**
     * @param channel
     * @return
     */
    public Collection<SingleReactionTask<V>> getSendersOn(NameCard channel);
    
    /**
     * @param channel
     * @return
     */
    public Collection<SingleReactionTask<V>> getReceiversOn(NameCard channel);
    
    /**
     * @param p
     * @return
     */
    public Collection<SingleReactionTask<V>> getCommunicators(Process<V> p);
    
    /**
     * @param p
     * @return
     */
    public Collection<SingleReactionTask<V>> getCommunicators(Place p);
    
    /**
     * @param prefixes
     * @return
     */
    public Collection<SingleReactionTask<V>> getCommunicators(Collection<ActionPrefix> prefixes);
    
    /**
     * @param a
     * @param b
     * @return
     */
    public boolean canCommunicate(SingleReactionTask<V> a, SingleReactionTask<V> b);
    
    /**
     * @param a
     * @param b
     * @return
     */
    public boolean canCommunicate(Place a, Place b);
    
}
