/*
 * Project:  Petruchio
 * Filename: Compiler.java
 * Created:  30.04.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces;

import java.util.List;
import java.util.Properties;


/**
 * A compiler.
 * 
 * @author Tim Strazny
 */
public interface Compiler extends Resettable {
    
    
    /**
     * The characters of which a pi-calculus name may consist.
     */
    public static final String NAME_CHARS = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz*_'~#$@%!?&-\"";

    
    /**
     * The name of this compiler.
     *
     * @return The compiler's name
     */
    public String getName();
    
    
    /**
     * Loads the given hierarchy of properties to the compiler.
     *
     * @param props A hierarchy of properties
     */
    public void loadProperties(List<Properties> props);
    
    
    /**
     * Translates the pi processes from the input file to a petri net that will
     * be written to the output file.
     * Both file names have to be absolute.
     *
     * @param inputFile  The file with pi process definitions to read.
     *                   The file name has to be absolute.  
     * @param outputFile The generated Petri net file to write.
     *                   The file name has to be absolute.
     */
    public void compile(String inputFile, String outputFile);
    
    
    /**
     * Reads in a Petri net and writes it out again. The input and output formats
     * are determined by the compiler options as well as the reductions and
     * everything that happens between the reading and writing of the net.
     *
     * @param inputFile A file containing a Petri net definition.
     * @param outputFile The file to write to.
     */
    public void convert(String inputFile, String outputFile);
    
    
    /**
     * Returns some information about status of the current compilation operation. 
     */
    public String getStatus();
    
    
    /**
     * Gives the set of described properties the compiler accepts and
     * understands.
     * 
     * @return The properties the compiler accepts
     */
    public CompilerProperty[] getOptions();
    
    
    /**
     * Tells the compiler to stop its computations.
     */
    public void pleaseStop();
    
}