/*
 * Project:  Petruchio
 * Filename: LocalConfigurationParser.java
 * Created:  21.09.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces;

import java.io.InputStream;
import java.util.Properties;


/**
 * A parser that reads the local configuration from a pi file. 
 *
 * @author Tim Strazny
 */
public interface LocalConfigurationParser extends Resettable {
	
	/**
     * Adds an error handler to be used to report errors.
     *
     * @param handler An error handler to use during the parsing
     */
    public void addErrorHandler(ParserErrorHandler handler);
    
    
    /**
     * Sets the input stream for the next parsing task.
     *
     * @param input The new input stream
     */
    public void setInput(InputStream input);
    
    
    /**
     * Returns the (optional) compiler properties defined in the given input.
     * The compiler may insert these on second place in the properties hierarchy.
     * If this operation is not supported, an empty Properties object may be
     * returned.
     *
     * @return The compiler properties defined in the input.
     */
    public Properties getLocalConfiguration();
    
}