/*
 * Project:  Petruchio
 * Filename: ParserErrorHandler.java
 * Created:  20.08.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces;



/**
 * An error handler for pi calculus process parsers.
 * 
 * @author Tim Strazny
 */
public interface ParserErrorHandler extends Resettable {

    /**
     * Reports an error with the specified message.
     *
     * @param msg The error message to be reported
     */
    public void reportError(String msg);
    
    
    /**
     * Reports an error with the specified message and the position of the token
     * on which the error occurred.
     *
     * @param line The line number of the token where the error occurred
     * @param column The position of the first character in the line of the
     *            token where the error occurred
     * @param msg The error message to be reported
     */
    public void reportError(int line, int column, String msg);
    
    
    /**
     * Reports an error with the specified message and the position of the token
     * on which the error occurred, as well as the token itself.
     *
     * @param line The line number of the token where the error occurred
     * @param column The position of the first character in the line of the
     *            token where the error occurred
     * @param token The token on which the error occurred
     * @param msg The error message to be reported
     */
    public void reportError(int line, int column, String token, String msg);
    
}