/*
 * Project:  Petruchio
 * Filename: CommunicationReaction.java
 * Created:  15.12.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.algorithms;

import java.util.List;

import petruchio.interfaces.pi.Name;


/**
 * A reaction of two processes that communicate.
 * 
 * @param <E> The data type for the processes.
 * @author Tim Strazny
 */
public interface CommunicationReaction<E> extends Reaction<E> {
    
    /**
     * Sets the channel name this communication uses.
     *
     * @param channel The channel
     */
    public void setChannel(Name channel);
    
    
    /**
     * Sets the names that where bound by the input action.
     *
     * @param names The names bound by the input action
     */
    public void setNamesBefore(List<Name> names);
    
    
    /**
     * Sets the names that where sent by the output action.
     *
     * @param names The names sent
     */
    public void setNamesAfter(List<Name> names);
    
    
    /**
     * The channel name this communication uses.
     *
     * @return The channel
     */
    public Name getChannel();
    
    
    /**
     * The names that where bound by the input action.
     *
     * @return The names bound by the input action
     */
    public List<Name> getNamesBefore();
    
    
    /**
     * The names that where sent by the output action.
     *
     * @return The names sent
     */
    public List<Name> getNamesAfter();
    
}