/*
 * Project:  Petruchio
 * Filename: RestrictedForm.java
 * Created:  30.11.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.algorithms;

import petruchio.interfaces.Resettable;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessCreator;


/**
 * A class that implements this interface has to be able to bring a process to
 * its strict restricted form.
 * 
 * @param <E> The data type of processes.
 * @author Tim Strazny
 */
public interface RestrictedForm<E> extends Resettable {
    
    /**
     * Sets the process creator to be used when processes have to be created.
     *
     * @param pbuilder The process creator to use
     */
    public void setProcessCreator(ProcessCreator pbuilder);
    
    
    /**
     * Clears internal caches.
     */
    public void clearCaches();
    
    
    /**
     * Brings a process to its strict restricted form.
     * To simplify this task, the free and bound names of the given process have
     * to be disjoint and bound names are unique.
     *
     * @param p A process in strict restricted form, unique bound names and
     *            disjoint sets of free and bound names
     * @return A process that is structurally congruent to the given process and
     *            which is in strict restricted form. Still the bound and free
     *            names are disjoint and the bound names are unique.
     */
    public Process<E> restrict(Process<E> p);
    
}