/*
 * Project:  Petruchio
 * Filename: SingleReaction.java
 * Created:  15.12.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.algorithms;

import java.util.List;

import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.ProcessID;


/**
 * A reaction that is no communication.
 * 
 * @param <E> The process' data type. 
 * @author Tim Strazny
 */
public interface SingleReaction<E> extends Reaction<E> {
    
    /**
     * Reaction step type.
     * An enum is used to avoid the expensive instanceof-operator.
     * 
     * @author Tim Strazny
     */
    public enum StepType {
        /** A tau reaction. */
        TAU,
        
        /** A recursion reaction. */
        RECURSE
    }
    
    
    /**
     * Sets the formal parameters of the referenced process.
     *
     * @param names The formal parameters of the referenced process
     */
    public void setNamesBefore(List<Name> names);
    
    
    /**
     * Sets the current parameters of the referenced process.
     *
     * @param names The current parameters of the referenced process
     */
    public void setNamesAfter(List<Name> names);
    
    
    /**
     * Returns the formal parameters of the referenced process.
     *
     * @return The formal parameters of the referenced process
     */
    public List<Name> getNamesBefore();
    
    
    /**
     * The current parameters of the referenced process.
     *
     * @return The current parameters of the referenced process
     */
    public List<Name> getNamesAfter();
    
    
    /**
     * Sets the id of the referenced process.
     *
     * @param pid The id of the referenced process
     */
    public void setProcessID(ProcessID pid);
    
    
    /**
     * Returns the id of the referenced process.
     *
     * @return The id of the referenced process
     */
    public ProcessID getProcessID();
    
    
    /**
     * Sets the type of this reaction.
     *
     * @param type The type of this reaction
     */
    public void setStepType(StepType type);
    
    
    /**
     * Returns the type of this reaction.
     *
     * @return The type of this reaction
     */
    public StepType getStepType();
    
}