/*
 * Project:  Petruchio
 * Filename: PTArc.java
 * Created:  02.08.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces.petrinet;


/**
 * A Petri net arc from a place to a transition.
 * 
 * @author Tim Strazny
 */
public interface PTArc {
    
    /**
     * Returns the place that is the source of this arc.
     *
     * @return The place that is the source of this arc.
     */
    public Place getSource();
    
    
    /**
     * Returns the transition that is the target of this arc.
     *
     * @return The transition that is the target of this arc.
     */
    public Transition getTarget();
    
    
    /**
     * Returns the weight of this arc.
     * 
     * @return This arc's weight
     */
    public int getWeight();
    
    
    /**
     * Sets the weight of this arc.
     *
     * @param weight The new weight.
     */
    public void setWeight(int weight);
    
}