/*
 * Project:  Petruchio
 * Filename: PNReductor.java
 * Created:  09.08.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces.petrinet;

import java.util.Collection;

import petruchio.interfaces.Resettable;
import petruchio.interfaces.SelfDescribing;


/**
 * A petri net reduction.
 * 
 * @author Tim Strazny
 */
public interface PetriNetReductor extends Resettable, SelfDescribing {

    /**
     * Gives the name of the reduction.
     *
     * @return The reduction's name
     */
    public String getName();
    
    
    /**
     * Gives a short description of what is done by the reduction.
     *
     * @return A short description of the reduction
     */
    @Override
    public String getDescription();
    
    
    /**
     * The reduction applied to the given Petri net.
     *
     * @param net The net to reduce.
     * @return The number of elements that were removed. 
     */
    public int reduce(PetriNet net);
    
    
    /**
     * The reduction applied to the given Petri net but the given places and
     * transitions are not reduced.
     *
     * @param net The net to reduce.
     * @param ignorePlaces Places that will not be reduced.
     * @param ignoreTransitions Transitions that will not be reduced.
     * @return The number of elements that were removed. 
     */
    public int reduce(PetriNet net, Collection<Place> ignorePlaces,
    		Collection<Transition> ignoreTransitions);
    
}