/*
 * Project:  Petruchio
 * Filename: PetriNetReductorIgnorer.java
 * Created:  25.09.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces.petrinet;

import java.io.InputStream;
import java.util.Collection;

import petruchio.interfaces.Resettable;

/**
 * A class implementing this interface can tell reductor classes not to reduce
 * certain places and transitions. It may decide which places and transitions
 * will be ignored by the reductions by analyzing the Petri net and by reading
 * information from a given input stream. 
 *
 * @author Tim Strazny
 */
public interface PetriNetReductorIgnorer extends Resettable {
	
	/**
	 * Sets the input that may be analyzed to determine the places and transitions
	 * to be ignored.
	 *
	 * @param in The input.
	 */
	public void setInput(InputStream in);
	
	
	/**
	 * Returns the set of places that may not be reduced.
	 *
	 * @param net A Petri net that will be reduced.
	 * @return A set of places that may not be reduced.
	 */
	public Collection<Place> getPlacesToIgnore(PetriNet net);
	
	
	/**
	 * Returns the set of transitions that may not be reduced.
	 *
	 * @param net A Petri net that will be reduced.
	 * @return A set of transitions that may not be reduced.
	 */
	public Collection<Transition> getTransitionsToIgnore(PetriNet net);

}