/*
 * Project:  Petruchio
 * Filename: PetriNetStreamConverter.java
 * Created:  09.10.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces.petrinet;

import java.io.InputStream;

import petruchio.interfaces.ParserErrorHandler;
import petruchio.interfaces.Resettable;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public interface PetriNetStreamReader extends Resettable {

    /**
     * Returns the file name extension for this Petri net format.
     *
     * @return The file name extension for this Petri net format
     */
    public String getDefaultExtensions();
    
    
    /**
     * Reads the Petri net's elements and passes them to the given writer
     * without creating the whole Petri net in memory.
     *
     * @param writer The writer to output the Petri net elements to.
     */
    public void readTo(PetriNetStreamWriter writer);
    
    
    /**
     * Sets the input for the reader.
     *
     * @param in The input stream to the Petri net representation
     */
    public void setInput(InputStream in);
    
    
    /**
     * Adds an error handler to report errors to.
     *
     * @param handler The error handler to add.
     */
    public void addErrorHandler(ParserErrorHandler handler);
    
}