/*
 * Project:  Petruchio
 * Filename: PetriNetWriter.java
 * Created:  02.08.2007
 * Author:   Tim Strazny
 */
package petruchio.interfaces.petrinet;

import java.io.OutputStream;

import petruchio.interfaces.Resettable;


/**
 * Given a specific Petri net an instance of this interface maps the petri net
 * to a textual representation in a specified format.
 * 
 * @author Tim Strazny
 */
public interface PetriNetWriter extends Resettable {
    
    /**
     * Returns the file name extension for this Petri net format.
     *
     * @return The file name extension for this Petri net format
     */
    public String getDefaultExtension();
    
    
    /**
     * Writes a textual representation of the Petri net to the given
     * OutputStream.
     *
     * @param net The Petri net to output
     * @param output The stream to output to
     */
    public void write(PetriNet net, OutputStream output);
    
}