/*
 * Project:  Petruchio
 * Filename: Transition.java
 * Created:  01.12.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.petrinet;

import java.util.Collection;


/**
 * A Petri net transition.
 * 
 * @author Tim Strazny
 */
public interface Transition extends NetElement {
    
    /**
     * Returns the name of this transition.
     *
     * @return This transition's name
     */
    public String getName();
    
    
    /**
     * Sets the name of this transition.
     *
     * @param name The new name for this transition
     */
    public void setName(String name);
    
    
    /**
     * Returns the meaning of this transition.
     *
     * @return The meaning of this transition.
     */
    public String getMeaning();
    
    
    /**
     * Sets the meaning of this transition.
     *
     * @param meaning The new meaning of this transition.
     */
    public void setMeaning(String meaning);

    
    /**
     * Returns the x coordinate of this transition's position.
     *
     * @return The x coordinate of this transition
     */
    public int getX();
    
    
    /**
     * Returns the y coordinate of this transition's position.
     *
     * @return The y coordinate of this transition
     */
    public int getY();
    
    
    /**
     * Sets the x coordinate of this transition's position.
     *
     * @param x The new x coordinate of this transition
     */
    public void setX(int x);
    
    
    /**
     * Sets the y coordinate of this transition's position.
     *
     * @param y The new y coordinate of this transition
     */
    public void setY(int y);

    
    /**
     * Returns this transition's input arcs.
     * 
     * @return The input arcs of this transition
     */
    public Collection<PTArc> getInput();
    
    
    /**
     * Returns this transition's output arcs.
     * 
     * @return The output arcs of this transition
     */
    public Collection<TPArc> getOutput();
    
}