/*
 * Project:  Petruchio
 * Filename: ActionPrefix.java
 * Created:  30.11.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.pi;


/**
 * A pi calculus action prefix.
 * 
 * @author Tim Strazny
 */
public interface ActionPrefix {
    
    /**
     * The type of action prefix.
     * An enum is used to avoid the expensive instanceof-operator.
     * 
     * @author Tim Strazny
     */
    public static enum Type {
        /** An input action. */
        INPUT,
        
        /** An output action. */
        OUTPUT,
        
        /** An unobservable, internal action. */
        INTERNAL,
        
        /** A guard. */
        GUARD
    }
    
    
    /**
     * Returns the type of this action prefix.
     * 
     * @return The type of this action prefix.
     */
    public Type getType();
    
    /**
     * @return
     */
    @Override
    public int hashCode();
    
    /**
     * @param o
     * @return
     */
    @Override
    public boolean equals(Object o);
    
    
    /**
     * Returns a String representation of this prefix.
     *
     * @return A String representation of this prefix.
     */
    @Override
    public String toString();
    
}