/*
 * Project:  Petruchio
 * Filename: Guard.java
 * Created:  05.12.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.pi;


/**
 * A pi calculus guard prefix. [a=b]
 * 
 * @author Tim Strazny
 */
public interface Guard extends ActionPrefix {
    
    /**
     * Returns the left operand of the guard equation. 
     *
     * @return The left operand of the guard equation.
     */
    public Name leftOperand();
    
    
    /**
     * Returns the right operand of the guard equation.
     *
     * @return The right operand of the guard equation.
     */
    public Name rightOperand();
    
    
    /**
     * Returns <code>true</code> if the guard is satisfied and may be removed
     * under structural congruence. That is, if the left and the right operand
     * are the same name.
     *
     * @return <code>true</code> if the guard is satisfied and may be removed
     *         under structural congruence
     */
    public boolean isSatisfied();
    
}
