/*
 * Project:  Petruchio
 * Filename: Parameters.java
 * Created:  05.12.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.pi;

import java.util.List;


/**
 * A list of parameters to use with a process reference (values) or a process
 * definition (formal parameters).
 * 
 * @author Tim Strazny
 */
public interface Parameters {

    /**
     * Adds a new parameter to the end of the list.
     *
     * @param param The parameter to append
     */
    public void add(Name param);
    
    
    /**
     * Returns a list representation of this parameters.
     *
     * @return A list representation of this parameters
     */
    public List<Name> getParameters();
    
    
    /**
     * Returns whether this is an empty parameter list.
     *
     * @return <code>true</code> if there are no parameters
     */
    public boolean isEmpty();
    
    
    /**
     * Returns the number of parameters.
     *
     * @return The number of parameters
     */
    public int size();
    
}