/*
 * Project: Petruchio
 * File:    PiWriter.java
 * Author:  Tim Strazny
 * Created: 16.10.2007
 */
package petruchio.interfaces.pi;

import java.io.OutputStream;

import petruchio.interfaces.Resettable;


/**
 * Given a collection of pi process definitions, an instance of this interface
 * maps the definitions to a textual representation in a specified format.
 * 
 * @author Tim Strazny
 */
public interface PiWriter<V> extends Resettable {
    
    /**
     * Returns the file name extension for this pi format.
     *
     * @return The file name extension for this pi format.
     */
    public String getDefaultExtension();
    
    
    /**
     * Writes a textual representation of the pi calculus process definitions to
     * the given OutputStream.
     *
     * @param comment The header comment to output.
     * @param header The header to output.
     * @param defs The processes to output.
     * @param output The stream to write to.
     */
    public void write(String comment, String header,
            ProcessManager<V> pm, OutputStream output);
    
}