/*
 * Project:  Petruchio
 * Filename: ProcessDefinition.java
 * Created:  30.11.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.pi;


/**
 * A pi calculus process definition that associates a process with a process
 * identifier and formal parameters.
 * 
 * @param <E> The data type of the process.
 * @author Tim Strazny
 */
public interface ProcessDefinition<E> {

    /**
     * Returns the process that is defined.
     *
     * @return The defined process
     */
    public Process<E> getProcess();
    
    
    /**
     * Returns the formal parameters of this definition.
     *
     * @return The formal parameters of this definition
     */
    public Parameters getParameters();
    
    
    /**
     * Set to true if this process definition is a main definition.
     *
     * @param main <code>true</code> if this is a main/system process
     */
    public void setMain(boolean main);
    
    
    /**
     * Returns the process identifier for this definition.
     *
     * @return The process identifier for this definition.
     */
    public ProcessID getProcessID();

    
    /**
     * Returns whether this process definition is marked as the main/system
     * process definition in the set of definitions. 
     *
     * @return <code>true</code> if this is a main/system process
     */
    public boolean isMain();
    
    
    /**
     * Returns the String representation of this process definition.
     *
     * @return The String representation of this process definition
     */
    @Override
    public String toString();
    
}