/*
 * Project:  Petruchio
 * Filename: ProcessID.java
 * Created:  30.11.2006
 * Author:   Tim Strazny
 */
package petruchio.interfaces.pi;


/**
 * A pi calculus process identifier as used in process definitions and process
 * references.
 *
 * Note that instances of this class may be used as keys for hash tables, so the
 * methods {@link #hashCode()} and {@link #equals(Object)} will have to be
 * implemented as specified in the {@link Object} class.
 * 
 * @author Tim Strazny
 */
public interface ProcessID extends Comparable<ProcessID> {
    
    /**
     * Returns the process identifier.
     *
     * @return The process identifier
     */
    public Name getID();
 
    
    /**
     * Compares this identifier to another Object and returns <code>true</code>
     * if the other object is a process identifier that represents the same
     * identifier.
     *
     * @param o An object
     * @return <code>true</code> if the object is a process identifier that
     *         identifies the same process as this identifier
     */
    @Override
    public boolean equals(Object o);
    
    
    /**
     * Returns a String representation of this identifier.
     *
     * @return A String representation of this identifier.
     */
    @Override
    public String toString();
    
    
    /**
     * Returns a hash code value for the object. This method is 
     * supported for the benefit of hashtables such as those provided by 
     * <code>java.util.Hashtable</code>. 
     * <p>
     * The general contract of <code>hashCode</code> is: 
     * <ul>
     * <li>Whenever it is invoked on the same object more than once during 
     *     an execution of a Java application, the <tt>hashCode</tt> method 
     *     must consistently return the same integer, provided no information 
     *     used in <tt>equals</tt> comparisons on the object is modified.
     *     This integer need not remain consistent from one execution of an
     *     application to another execution of the same application. 
     * <li>If two objects are equal according to the <tt>equals(Object)</tt>
     *     method, then calling the <code>hashCode</code> method on each of 
     *     the two objects must produce the same integer result. 
     * <li>It is <em>not</em> required that if two objects are unequal 
     *     according to the {@link java.lang.Object#equals(java.lang.Object)} 
     *     method, then calling the <tt>hashCode</tt> method on each of the 
     *     two objects must produce distinct integer results.  However, the 
     *     programmer should be aware that producing distinct integer results 
     *     for unequal objects may improve the performance of hashtables.
     * </ul>
     * <p>
     * As much as is reasonably practical, the hashCode method defined by 
     * class <tt>Object</tt> does return distinct integers for distinct 
     * objects. (This is typically implemented by converting the internal 
     * address of the object into an integer, but this implementation 
     * technique is not required by the 
     * Java<font size="-2"><sup>TM</sup></font> programming language.)
     *
     * @return A hash code value for this object.
     */
    @Override
    public int hashCode();
    
}