/*
 * Project: Petruchio
 * File:    ProcessModifier.java
 * Author:  Tim Strazny
 * Created: 03.01.2008
 */
package petruchio.interfaces.pi;

import petruchio.interfaces.Resettable;


/**
 * A process modifier may modify the processes prior to compilation, after they
 * are brought to (strict) restricted form and the bound and free names are made
 * disjoint and unique.
 * 
 * @author Tim Strazny
 * @param <V> The type of data attached to the processes.
 */
public interface ProcessModifier<V> extends Resettable {
    
    /**
     * Sets the process creator to be used when processes have to be created.
     *
     * @param pbuilder The process creator to use
     */
    public void setProcessCreator(ProcessCreator pbuilder);
    
    
    /**
     * The modification of the processes.
     *
     * @param pmanager The manager that holds all the process definitions to modify.
     * @param main The main process' id.
     * @return A message that will be output and written to the compiled file.
     */
    public String modify(ProcessManager<V> pmanager, ProcessID main);
    
    
    /**
     * The name of this modification.
     *
     * @return The name of this modification.
     */
    public String getName();
    
}