/*
 * Project:  Petruchio
 * Filename: DuplicateProcessIDException.java
 * Created:  08.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pi;


import petruchio.interfaces.pi.ProcessID;


/**
 * When looking up a process ID in the process manager that already has an
 * associated process, an instance of this exception will be thrown. 
 * 
 * @author Tim Strazny
 */
public class DuplicateProcessIDException extends RuntimeException {

    /** The serial version UID */
    private static final long serialVersionUID = -3150439749345281061L;
    
    /** The process ID that no process was associated with. */
    private final ProcessID pid;
    
    
    /**
     * Initializes the exception by saving the process ID.
     * 
     * @param pid The process ID that no process was associated with.
     */
    public DuplicateProcessIDException(final ProcessID pid) {
        this.pid = pid;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Throwable#toString()
     */
    @Override
    public String toString() {
        return "A process with id " + this.pid + " has already been defined.";
    }

}