/*
 * Project:  DA
 * Filename: Process.java
 * Created:  01.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pi;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import petruchio.interfaces.pi.ActionPrefix;
import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.InputAction;
import petruchio.interfaces.pi.InternalAction;
import petruchio.interfaces.pi.OutputAction;
import petruchio.interfaces.pi.Guard;


/**
 * <Description>
 * 
 * @param <E>
 * @author Tim Strazny
 */
public abstract class Process<E> implements petruchio.interfaces.pi.Process<E> {
    
    /**
     * 
     */
    private final List<ActionPrefix> prefixes;
    
    /**
     * 
     */
    private final Collection<Name> boundNames;
    
    
    /**
     * 
     */
    private String string;
    
    
    /**
     * 
     */
    private E data;
    
    
    /**
     * 
     */
    public Process() {
        this.prefixes = new ArrayList<ActionPrefix>(0);
        this.boundNames = new ArrayList<Name>(0);
        this.data = null;
        this.string = null;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#addActionPrefix(petruchio.interfaces.pi.ActionPrefix)
     */
    @Override
    public void addActionPrefix(final ActionPrefix ap) {
        this.prefixes.add(0, ap);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#addActionPrefixes(java.util.List)
     */
    @Override
    public void addActionPrefixes(final List<ActionPrefix> aps) {
        this.prefixes.addAll(0, aps);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#getActionPrefixes()
     */
    @Override
    public List<ActionPrefix> getActionPrefixes(){
        return this.prefixes;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#addRestriction(java.lang.String)
     */
    @Override
    public void addRestriction(final Name name) {
        if(!this.boundNames.contains(name)) {
            this.boundNames.add(name);
        }
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#addRestrictions(java.util.Collection)
     */
    @Override
    public void addRestrictions(final Collection<Name> names) {
        for(final Name name : names) {
            this.addRestriction(name);
        }
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#getRestrictions()
     */
    @Override
    public Collection<Name> getRestrictions(){
        return this.boundNames;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#getData()
     */
    @Override
    public E getData() {
        return this.data;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#setData(java.lang.Object)
     */
    @Override
    public void setData(final E data) {
        this.data = data;
        if(data == null) {
            this.string = null;
        }
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#changed()
     */
    @Override
    public void changed() {
        this.setData(null);
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#hasRealPrefixes()
     */
    @Override
    public boolean hasRealPrefixes() {
        for(final ActionPrefix a : this.getActionPrefixes()) {
            if(a instanceof InputAction) {
                return true;
            } else if(a instanceof OutputAction) {
                return true;
            } else if(a instanceof InternalAction) {
                return true;
            } else if(a instanceof Guard) {
                // do not return true
            } else {
                throw new InternalError("Unknown type of prefix: " + a);
            }
        }
        return false;
    }
    

    /**
     * TODO: document
     *
     * @return
     */
    public abstract String getSuffix();
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        if(this.string == null) {
            final StringBuilder buf = new StringBuilder();
            if(!this.boundNames.isEmpty()) {
                buf.append("new ");
                for(final Iterator<Name> it = this.boundNames.iterator(); it.hasNext(); ) {
                    final Name name = it.next();
                    buf.append(name);
                    if(it.hasNext()) {
                        buf.append(", ");
                    } else {
                        buf.append(".");
                    }
                }
            }
            for(final ActionPrefix ap : this.prefixes) {
                buf.append(ap.toString());
                buf.append(".");
            }
            buf.append(this.getSuffix());
            this.string = buf.toString();
        }
        return this.string;
    }

}