/*
 * Project:  Petruchio
 * Filename: ProcessComposition.java
 * Created:  01.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pi;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import petruchio.compiler.LexProcessComparator;


/**
 * <Description>
 * 
 * @param <E>
 * @author Tim Strazny
 */
public class ProcessComposition<E> extends Process<E> implements petruchio.interfaces.pi.ProcessComposition<E> {

    /** The collection of child processes. No ordering needed. */
    private final List<petruchio.interfaces.pi.Process<E>> processes;
    
    /** The operator of the composition. */
    private Operator op;
    
    
    /**
     * Initializes the new instance.
     */
    public ProcessComposition() {
        this.processes = new ArrayList<petruchio.interfaces.pi.Process<E>>(0);
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#getType()
     */
    @Override
    public petruchio.interfaces.pi.Process.Type getType() {
        return petruchio.interfaces.pi.Process.Type.COMPOSITION;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessComposition#addProcess(petruchio.interfaces.pi.Process)
     */
    @Override
    public void addProcess(petruchio.interfaces.pi.Process<E> p) {
        if(p == null) {
            throw new IllegalArgumentException("Will not add 'null' process.");
        }
        this.processes.add(p);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessComposition#addProcesses(java.util.Collection)
     */
    @Override
    public void addProcesses(final List<petruchio.interfaces.pi.Process<E>> p) {
        this.processes.addAll(p);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessComposition#getOperator()
     */
    @Override
    public Operator getOperator() {
        return this.op;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessComposition#getProcesses()
     */
    @Override
    public List<petruchio.interfaces.pi.Process<E>> getProcesses() {
        return this.processes;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessComposition#setOperator(petruchio.interfaces.pi.ProcessComposition.Operator)
     */
    @Override
    public void setOperator(final Operator op) {
        if(op == null) {
            throw new IllegalArgumentException("The operator may not be null.");
        }
        this.op = op;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pi.Process#getSuffix()
     */
    @Override
    public String getSuffix() {
    	final String opString;
    	switch(this.op) {
    		case PARALLEL:
    			opString = " | ";
    			break;
    		case CHOICE:
				opString = " + ";
				break;
    		case SEQUENCE:
                opString = " ; ";
                break;
			default:
				opString = " (?) ";
    	}
    	
    	final StringBuilder b = new StringBuilder();
    	b.append("(");
    	final List<petruchio.interfaces.pi.Process<E>> procs;
    	if(this.processes.size() > 1 && this.op != ProcessComposition.Operator.SEQUENCE) {
    	    procs = new ArrayList<petruchio.interfaces.pi.Process<E>>(this.processes);
    	    Collections.sort(procs, LexProcessComparator.getInstance());
    	} else {
    	    procs = this.processes;
    	}
    	for(final Iterator<petruchio.interfaces.pi.Process<E>> it = procs.iterator(); it.hasNext(); ) {
        	b.append(it.next().toString());
            if(it.hasNext()) {
                b.append(opString);
            }
    	}
    	b.append(")");
    	return b.toString();
    }

}