/*
 * Project:  Petruchio
 * Filename: ProcessBuilder.java
 * Created:  01.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pi;

import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.pi.Guard;
import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.Parameters;
import petruchio.interfaces.pi.NullProcess;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessComposition;
import petruchio.interfaces.pi.ProcessDefinition;
import petruchio.interfaces.pi.ProcessID;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class ProcessCreator implements SelfDescribing, petruchio.interfaces.pi.ProcessCreator {

    /** The internal action */
    private static final InternalAction INTERNAL_ACTION = new petruchio.pi.InternalAction();
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "The default process creator that controls the instantiation of pi calculus process elements.";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newInputAction(java.lang.String, petruchio.interfaces.pi.Parameters)
     */
    @Override
    public InputAction newInputAction(final Name channel, final Parameters names) {
        return new petruchio.pi.InputAction(channel, names);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newInternalAction()
     */
    @Override
    public InternalAction newInternalAction() {
        return ProcessCreator.INTERNAL_ACTION;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newOutputAction(java.lang.String, petruchio.interfaces.pi.Parameters)
     */
    @Override
    public OutputAction newOutputAction(final Name channel, final Parameters names) {
        return new petruchio.pi.OutputAction(channel, names);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newProcessComposition()
     */
    @Override
    public <T> ProcessComposition<T> newProcessComposition() {
        return new petruchio.pi.ProcessComposition<T>();
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newProcessDefinition(petruchio.interfaces.pi.ProcessID, petruchio.interfaces.pi.Process, boolean)
     */
    @Override
    public <T> ProcessDefinition<T> newProcessDefinition(final ProcessID pid,
                    final Process<T> p, final Parameters params,
                    final boolean isMain) {
        return new petruchio.pi.ProcessDefinition<T>(pid, p, params, isMain);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newProcessID(java.lang.String)
     */
    @Override
    public ProcessID newProcessID(final Name name) {
        return new petruchio.pi.ProcessID(name);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newPrefixProcess(petruchio.interfaces.pi.Process)
     */
    @Override
    public <T> PrefixProcess<T> newPrefixProcess(final Process<T> p) {
        return new petruchio.pi.PrefixProcess<T>(p);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newProcessReference(petruchio.interfaces.pi.ProcessID)
     */
    @Override
    public <T> ProcessReference<T> newProcessReference(final ProcessID pid, final Parameters params) {
        return new ProcessReference<T>(pid, params);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#nullProcess()
     */
    @Override
    public <T> NullProcess<T> nullProcess() {
        return new petruchio.pi.NullProcess<T>();
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newGuard(java.lang.String, java.lang.String)
     */
    @Override
    public Guard newGuard(Name left, Name right) {
        return new petruchio.pi.Guard(left, right);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newParameterList()
     */
    @Override
    public Parameters newParameterList() {
        return new petruchio.pi.Parameters();
    }


	/* (non-Javadoc)
	 * @see petruchio.interfaces.Resettable#reset()
	 */
	@Override
    public void reset() {
		this.clearCaches();
	}
	
	
	/* (non-Javadoc)
	 * @see petruchio.interfaces.pi.ProcessCreator#clearCaches()
	 */
	@Override
    public void clearCaches() {
		// no caches...
	}


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessCreator#newName(java.lang.String)
     */
    @Override
    public Name newName(final String name) {
        return new petruchio.pi.Name(name);
    }
    
}