/*
 * Project:  DA
 * Filename: ProcessDefinition.java
 * Created:  01.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pi;

import java.util.HashSet;
import java.util.Set;

import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.Parameters;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessID;


/**
 * <Description>
 * 
 * @param <E>
 * @author Tim Strazny
 */
public class ProcessDefinition<E> implements petruchio.interfaces.pi.ProcessDefinition<E> {

    /**
     * 
     */
    private final ProcessID pid;
    
    /**
     * 
     */
    private final Process<E> p;
    
    /**
     * 
     */
    private final Parameters params;
    
    /**
     * 
     */
    private boolean isMain;
    
    
    /**
     * @param pid
     * @param p
     * @param params
     * @param isMain
     */
    public ProcessDefinition(final ProcessID pid, final Process<E> p,
                    final Parameters params, final boolean isMain) {
        this.pid = pid;
        this.p = p;
        this.params = params;
        this.isMain = isMain;
        if(!ProcessDefinition.uniqueParameters(params)) {
            throw new IllegalParametersException(this);
        }
    }
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessDefinition#getProcess()
     */
    @Override
    public Process<E> getProcess() {
        return this.p;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessDefinition#getProcessID()
     */
    @Override
    public ProcessID getProcessID() {
        return this.pid;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessDefinition#getParameters()
     */
    @Override
    public Parameters getParameters() {
        return this.params;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessDefinition#isMain()
     */
    @Override
    public boolean isMain() {
        return this.isMain;
    }

    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        if(this.params.isEmpty()) {
            return this.pid.toString() + " := " + this.p.toString() + ";";
        }
        return this.pid.toString() + "(" + this.params.toString() + ") := " + this.p.toString() + ";";
    }
    
    /**
     * TODO: document
     *
     * @param params
     * @return
     */
    private static boolean uniqueParameters(final Parameters params) {
        final Set<Name> p = new HashSet<Name>(params.size());
        for(final Name n : params.getParameters()) {
            if(!p.add(n)) {
                return false;
            }
        }
        return true;
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessDefinition#setMain(boolean)
     */
    @Override
    public void setMain(final boolean main) {
        this.isMain = main;
    }
}