/*
 * Project:  Petruchio
 * Filename: ProcessReference.java
 * Created:  04.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pi;

import petruchio.interfaces.pi.Parameters;
import petruchio.interfaces.pi.ProcessID;


/**
 * <Description>
 * 
 * @param <E>
 * @author Tim Strazny
 */
public class ProcessReference<E> extends Process<E> implements petruchio.interfaces.pi.ProcessReference<E> {
    
    /** The ID of the referenced process */
    private final ProcessID pid;
    
    /** The input parameters for the reference */
    private final Parameters params;
    
    
    /**
     * Initializes the new instance with process ID and input parameters.
     * 
     * @param pid    The ID of the referenced process.
     * @param params The input parameters for the reference.
     */
    public ProcessReference(final ProcessID pid, final Parameters params) {
        this.pid = pid;
        this.params = params;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.Process#getType()
     */
    @Override
    public petruchio.interfaces.pi.Process.Type getType() {
        return petruchio.interfaces.pi.Process.Type.REFERENCE;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessReference#getProcessID()
     */
    @Override
    public ProcessID getProcessID() {
        return this.pid;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessReference#getParameters()
     */
    @Override
    public Parameters getParameters() {
        return this.params;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pi.Process#getSuffix()
     */
    @Override
    public String getSuffix() {
        if(this.params.isEmpty()) {
            return this.pid.toString();
        }
        return this.pid.toString() + "(" + this.params.toString() + ")";
    }
    
}