/*
 * Project: Petruchio
 * File:    SafeProcess.java
 * Author:  Tim Strazny
 * Created: 07.12.2007
 */
package petruchio.pi.modifiers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import petruchio.compiler.CommonTasks;
import petruchio.compiler.ProcessData;
import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.PrefixProcess;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessComposition;
import petruchio.interfaces.pi.ProcessCreator;
import petruchio.interfaces.pi.ProcessDefinition;
import petruchio.interfaces.pi.ProcessID;
import petruchio.interfaces.pi.ProcessManager;
import petruchio.interfaces.pi.ProcessReference;


/**
 * TODO: Document
 * 
 * @author Tim Strazny
 */
public class MarkCallers implements SelfDescribing, petruchio.interfaces.pi.ProcessModifier<ProcessData> {

    private static final String ZERO_ID = "ZERO";
    
    private ProcessCreator pbuilder = null;
    
    private Name zeroID = null;
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        this.pbuilder = null;
        this.zeroID = null;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessModifier#modify(petruchio.interfaces.pi.ProcessManager, petruchio.interfaces.pi.ProcessID)
     */
    @Override
    public String modify(final ProcessManager<ProcessData> pm, final ProcessID main) {
        final Map<Name, Collection<Name>> calledBy = new HashMap<Name, Collection<Name>>(pm.getProcessIDs().size());
        
        String zID = MarkCallers.ZERO_ID;
        int i = 0;
        while(true) {
            this.zeroID = this.pbuilder.newName(zID);
            if(pm.getProcessIDs().contains(this.pbuilder.newProcessID(this.zeroID))) {
                zID = MarkCallers.ZERO_ID + "_" + ++i; 
            } else {
                break;
            }
        }
        
        
        boolean containsZero = false;
        
        for(final ProcessDefinition<ProcessData> def : pm.getProcessDefinitions()) {
            final Process<ProcessData> p = def.getProcess();
            
            final ProcessID id = def.getProcessID();
            final Name caller = id.getID();
            
            if(CommonTasks.getData(p).getNullProcCount() != 0) {
                containsZero = true;
                Collection<Name> callers = calledBy.get(this.zeroID);
                if(callers == null) {
                    callers = new ArrayList<Name>();
                    calledBy.put(this.zeroID, callers);
                }
                callers.add(caller);
            }
            
            for(final Name call : CommonTasks.getData(p).getRefProcIDs()) {
                Collection<Name> callers = calledBy.get(call);
                if(callers == null) {
                    callers = new ArrayList<Name>();
                    calledBy.put(call, callers);
                }
                callers.add(caller);
            }
        
            final Process<ProcessData> newP = this.annotateReferences(p, caller);
            pm.updateProcess(id, newP);
        }
        
        if(containsZero) {
            final Process<ProcessData> stop = this.pbuilder.nullProcess();
            pm.addProcessDefinition(this.pbuilder.newProcessDefinition(this.pbuilder.newProcessID(this.zeroID),
                    stop, this.pbuilder.newParameterList(), false));
        }
        
        for(final Map.Entry<Name, Collection<Name>> e : calledBy.entrySet()) {
            final Name call = e.getKey();
            final Collection<Name> callers = e.getValue();
            final ProcessID id = this.pbuilder.newProcessID(call);
            final ProcessDefinition<ProcessData> def = pm.getProcessDefinition(id);
            
            if(def != null) {
                for(final Name caller : callers) {
                    final ProcessID newID = this.pbuilder.newProcessID(this.newName(call, caller));
                    pm.addProcessDefinition(this.pbuilder.newProcessDefinition(newID, def.getProcess(), def.getParameters(), false));
                }
            
                if(!call.equals(main.getID())) {
                    pm.removeProcess(id);
                }
            }
        }
        
        return null;
    }
    
    
    private Name newName(final Name prefix, final Name suffix) {
        return this.pbuilder.newName(prefix.toString() + "__" + suffix.toString());
    }
    
    
    /**
     * TODO: Document
     *
     * @param p
     * @param caller
     */
    private Process<ProcessData> annotateReferences(final Process<ProcessData> p, final Name caller) {
        switch(p.getType()) {
            case REFERENCE: {
                final ProcessReference<ProcessData> ref = (ProcessReference<ProcessData>)p;
                final Name curID = ref.getProcessID().getID();
                final ProcessID id = this.pbuilder.newProcessID(this.newName(curID, caller));
                final ProcessReference<ProcessData> newP = this.pbuilder.newProcessReference(id, ref.getParameters());
                newP.addActionPrefixes(p.getActionPrefixes());
                newP.addRestrictions(p.getRestrictions());
                return newP;
            }
            
            case COMPOSITION: {
                final ProcessComposition<ProcessData> comp = (ProcessComposition<ProcessData>)p;
                final List<Process<ProcessData>> l = new ArrayList<Process<ProcessData>>(comp.getProcesses().size());
                boolean changed = false;
                for(final Process<ProcessData> sub : comp.getProcesses()) {
                    final Process<ProcessData> subReplaced = this.annotateReferences(sub, caller);
                    if(subReplaced != sub) {
                        changed = true;
                    }
                    l.add(subReplaced);
                }
                if(changed) {
                    final ProcessComposition<ProcessData> c = this.pbuilder.newProcessComposition();
                    c.setOperator(comp.getOperator());
                    c.addActionPrefixes(comp.getActionPrefixes());
                    c.addRestrictions(comp.getRestrictions());
                    c.addProcesses(l);
                    c.changed();
                    return c;
                }
                return p;
            }
                
            case PREFIX: {
                final PrefixProcess<ProcessData> prefix = (PrefixProcess<ProcessData>)p;
                final Process<ProcessData> r = this.annotateReferences(prefix.getProcess(), caller);
                if(r != prefix.getProcess()) {
                    final PrefixProcess<ProcessData> rep = this.pbuilder.newPrefixProcess(r);
                    rep.addActionPrefixes(prefix.getActionPrefixes());
                    rep.addRestrictions(prefix.getRestrictions());
                    rep.changed();
                    return rep;
                }
                return p;
            }
            
            case NULL: {
                final Name curID = this.zeroID;
                final ProcessID id = this.pbuilder.newProcessID(this.newName(curID, caller));
                final ProcessReference<ProcessData> newP = this.pbuilder.newProcessReference(id, this.pbuilder.newParameterList());
                newP.addActionPrefixes(p.getActionPrefixes());
                newP.addRestrictions(p.getRestrictions());
                return newP;
            }
            
            default:
                throw new InternalError("Unexpected type of process: " + p);
        }
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "Default implementation of an algorithm that annotates all references with the ID of the caller process.";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.algorithms.SafeProcess#setProcessCreator(petruchio.interfaces.pi.ProcessCreator)
     */
    @Override
    public void setProcessCreator(final ProcessCreator pbuilder) {
        this.pbuilder = pbuilder;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessModifier#getName()
     */
    @Override
    public String getName() {
        return "annotate caller processes";
    }

}