/*
 * Project: Petruchio
 * File:    Polyadic2Monadic.java
 * Author:  Tim Strazny
 * Created: 02.11.2008
 */
package petruchio.pi.modifiers;

import java.util.Collection;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import petruchio.common.IdentityHashSet;
import petruchio.compiler.CommonTasks;
import petruchio.compiler.ProcessData;
import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.pi.ActionPrefix;
import petruchio.interfaces.pi.InputAction;
import petruchio.interfaces.pi.Name;
import petruchio.interfaces.pi.PrefixProcess;
import petruchio.interfaces.pi.Process;
import petruchio.interfaces.pi.ProcessCreator;
import petruchio.interfaces.pi.ProcessDefinition;
import petruchio.interfaces.pi.ProcessID;
import petruchio.interfaces.pi.ProcessManager;
import petruchio.interfaces.pi.ProcessModifier;

/**
 * Not implemented yet.
 * 
 * @author Tim Strazny
 */
public class Polyadic2Monadic implements SelfDescribing, ProcessModifier<ProcessData> {

    @SuppressWarnings("unused")
    private ProcessCreator pbuilder = null;
    
    private Collection<Name> usedNames = new HashSet<Name>();
    private Map<Name, Map<Integer, Name>> renaming = new HashMap<Name, Map<Integer, Name>>();
    
    public Polyadic2Monadic() {
    	throw new UnsupportedOperationException("Not implemented yet.");
    }
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessModifier#getName()
     */
    @Override
    public String getName() {
        return "polyadic to monadic";
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessModifier#modify(petruchio.interfaces.pi.ProcessManager, petruchio.interfaces.pi.ProcessID)
     */
    @Override
    public String modify(final ProcessManager<ProcessData> pmanager, final ProcessID main) {
        final Collection<ProcessDefinition<ProcessData>> todo = new IdentityHashSet<ProcessDefinition<ProcessData>>(pmanager.getProcessDefinitions().size());
        for(final ProcessDefinition<ProcessData> d : pmanager.getProcessDefinitions()) {
            this.usedNames.addAll(d.getParameters().getParameters());
            this.usedNames.addAll(CommonTasks.getData(d.getProcess()).getBn());
            this.usedNames.addAll(CommonTasks.getData(d.getProcess()).getFn());
            todo.add(d);
        }

        final Collection<ProcessDefinition<ProcessData>> defs = new IdentityHashSet<ProcessDefinition<ProcessData>>();
        final Map<Name, Process<ProcessData>> origin = new HashMap<Name, Process<ProcessData>>();
        
        while(!todo.isEmpty()) {
            final Iterator<ProcessDefinition<ProcessData>> it = todo.iterator();
            final ProcessDefinition<ProcessData> d = it.next();
            it.remove();
            final ProcessDefinition<ProcessData> nd = this.makeMonadic(d, origin, todo);
            if(d != nd) {
                defs.add(nd);
            }
        }
        for(final ProcessDefinition<ProcessData> d : defs) {
            pmanager.updateProcess(d);    
        }
        return null;
    }
    
    /**
     * @param def
     * @return
     */
    private ProcessDefinition<ProcessData> makeMonadic(final ProcessDefinition<ProcessData> def,
            final Map<Name, Process<ProcessData>> origin,
            final Collection<ProcessDefinition<ProcessData>> todo) {
        final Process<ProcessData> p = def.getProcess();
        for(final Name n : def.getParameters().getParameters()) {
            origin.put(n, null);
        }
        final Process<ProcessData> np = this.makeMonadic(p, origin, todo);
        origin.clear();
        if(p != np) {
            return new petruchio.pi.ProcessDefinition<ProcessData>(def.getProcessID(),
                    np, def.getParameters(), def.isMain());
        }
        return def;
    }
    
    /**
     * @param p
     * @param origin
     * @param todo
     * @return
     */
    private Process<ProcessData> makeMonadic(final Process<ProcessData> p,
            final Map<Name, Process<ProcessData>> origin,
            final Collection<ProcessDefinition<ProcessData>> todo) {
        for(final Name n : p.getRestrictions()) {
            origin.put(n, p);
        }
        switch(p.getType()) {
            case NULL: {
                return p;
            }
            case REFERENCE: {
                return p;
            }
            case PREFIX: {
                final PrefixProcess<ProcessData> a = (PrefixProcess<ProcessData>)p;
                for(final ActionPrefix ap : a.getActionPrefixes()) {
                    switch(ap.getType()) {
                        case INPUT: {
                            final InputAction iap = (InputAction)ap;
                            for(final Name n : iap.getNames().getParameters()) {
                                origin.put(n, p);
                            }
                            break;
                        }
                        case OUTPUT: {
//                            final OutputAction oap = (OutputAction)ap;
                            throw new UnsupportedOperationException("not implemented yet.");
                        }
                        default: {
                            throw new IllegalArgumentException("Unknown process type: " + p);
                        }
                    }
                }
                break;
            }
            case COMPOSITION: {
                throw new UnsupportedOperationException("not implemented yet.");
            }
            default: {
                throw new IllegalArgumentException("Unknown process type: " + p);
            }
        }
//        return null;
        throw new UnsupportedOperationException("not implemented yet.");
    }
    
//    /**
//     * @param old
//     * @param arity
//     * @return
//     */
//    private Name getNewName(final Name old, final int arity) {
//        Map<Integer, Name> ni = this.renaming.get(old);
//        final Integer ar = Integer.valueOf(arity);
//        Name n = ni != null ? ni.get(ar) : null;
//        
//        if(n == null) {
//            n = new petruchio.pi.Name(old.toString() + "_" + arity);
//            int i = 1;
//            while(this.usedNames.contains(n)) {
//                n = new petruchio.pi.Name(old.toString() + "_" + arity + "_" + (i++));
//            }
//            
//            this.usedNames.add(n);
//            if(ni == null) {
//                ni = new HashMap<Integer, Name>();
//                this.renaming.put(old, ni);
//            }
//            ni.put(ar, n);
//        }
//        return n;
//    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.ProcessModifier#setProcessCreator(petruchio.interfaces.pi.ProcessCreator)
     */
    @Override
    public void setProcessCreator(final ProcessCreator pbuilder) {
        this.pbuilder = pbuilder;
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        this.pbuilder = null;
        this.renaming.clear();
        this.usedNames.clear();
    }
    
    /**
     * @return
     */
    @Override
    public String getDescription() {
        return "Converts processes that use the polyadic extension of the pi-calculus to the basic monadic"
            + " version of the calculus without changing the semantics of the processes.";
    }

}
