/*
 * Project:  Petruchio
 * Filename: PiParserErrorHandler.java
 * Created:  20.08.2007
 * Author:   Tim Strazny
 */
package petruchio.pi.readers;

import java.util.logging.Logger;

import petruchio.interfaces.SelfDescribing;


/**
 * A default console error handler.
 * 
 * @author Tim Strazny
 */
public class ConsoleErrorHandler implements SelfDescribing, petruchio.interfaces.ParserErrorHandler {
    
    private final Logger logger = Logger.getLogger(System.getProperty("Logger", this.getClass().getName()));
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "Logs error reports as severe errors to the console.";
    }
    

    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // nothing to do
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParserErrorHandler#reportError(java.lang.String)
     */
    @Override
    public void reportError(final String msg) {
        this.logger.severe("Parsing error: "
                        + ConsoleErrorHandler.maskNullMessage(msg));
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParserErrorHandler#reportError(int, int, java.lang.String)
     */
    @Override
    public void reportError(final int line, final int column, final String msg) {
        this.logger.severe("Parsing error at line " + line + ", column " + column
                        + ": " + ConsoleErrorHandler.maskNullMessage(msg));
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParserErrorHandler#reportError(int, int, java.lang.String, java.lang.String)
     */
    @Override
    public void reportError(final int line, final int column,
                    final String token, final String msg) {
        this.logger.severe("Parsing error at line " + line + ", column " + column
                        + ", token '" + token + "': "
                        + ConsoleErrorHandler.maskNullMessage(msg));
    }
    
    
    /**
     * TODO: document
     *
     * @param msg
     * @return
     */
    private static String maskNullMessage(final String msg) {
        return (msg == null || msg.length() == 0) ? "No message." : msg;
    }

}