/*
 * Project:  Petruchio
 * Filename: MWBPi.flex
 * Created:  15.08.2007
 * Author:   Tim Strazny
 * Remarks:  A JFlex (jflex.de) lexer for pi processes 
 */

package petruchio.pi.readers.cup;

import java_cup.runtime.ComplexSymbolFactory.ComplexSymbol;
import java_cup.runtime.Scanner;

%%

%public
%class MWBPiLexer
%implements petruchio.pi.readers.cup.MWBPiSymbols, petruchio.pi.readers.cup.CUPLexer

%unicode

%line
%column

%cup

%{
  private PiSymbol symbol(int type) {
    return new PiSymbol(type, yyline+1, yycolumn+1, yytext());
  }
%}

LineTerminator = \r | \n| \r\n
InputCharacter = [^\r\n]

WhiteSpace = {LineTerminator} | [ \t\f]

Comment = {MLComment} | {MLCommentMWB} | {EOLComment}
MLComment = "/*" ~"*/"
MLCommentMWB = "(*" ~"*)"
EOLComment = "//" {InputCharacter}* {LineTerminator}?

// first letter of a name, no apostrophe
Cha1 = [0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz*_~#$@%!?&-]
// inner letters of a name
Cha = [0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz*_'~#$@%!?&-]
StringCha = [^\"]

%%

<YYINITIAL> {
  "{"			{ return symbol(LBRACE); }
  "}"			{ return symbol(RBRACE); }
  "("			{ return symbol(LPAREN); }
  ")"			{ return symbol(RPAREN); }
  "<"			{ return symbol(LANGLE); }
  ">"			{ return symbol(RANGLE); }
  "["			{ return symbol(LBRACK); }
  "]"			{ return symbol(RBRACK); }
  ";"			{ return symbol(SEMICOLON); }
  "."			{ return symbol(DOT); }
  ","			{ return symbol(COMMA); }
  "="			{ return symbol(EQUAL); }
  "|"			{ return symbol(VERTICAL); }
  "+"			{ return symbol(PLUS); }
  "^"			{ return symbol(HAT); }
  ":="			{ return symbol(DEF); }
  "0"			{ return symbol(ZERO); }
  "main"		{ return symbol(MAIN); }
  "agent"		{ return symbol(AGENT); }
  "tau"			{ return symbol(TAU); }
  "in"			{ return symbol(IN); }
  "out"			{ return symbol(OUT); }
  "new"			{ return symbol(NEW); }
  "nu"			{ return symbol(NEW); }
  "options"		{ return symbol(OPTIONS); }
  "'"			{ return symbol(APOSTROPHE); }

  "\\"{LineTerminator}  { /* ignore */ }  
  {Comment}		{ /* ignore */ }
  {WhiteSpace}  { /* ignore */ }
  
  \" {StringCha}* \"	{ return symbol(NAME); }
  {Cha1}{Cha}*  { return symbol(NAME); }
}

/* error fallback */
.|\n			{ throw new RuntimeException("Illegal character \"" + yytext() + "\" at line " + yyline + ", column " + yycolumn); } 

<<EOF>>	{return symbol(EOF); }