/*
 * Project:  Petruchio
 * Filename: MWBPiParser.java
 * Created:  15.08.2007
 * Author:   Tim Strazny
 */
package petruchio.pi.readers.cup;

import java.io.Reader;
import java.util.Collection;
import java.util.Properties;

import petruchio.interfaces.ParserErrorHandler;
import petruchio.interfaces.pi.ProcessCreator;
import petruchio.interfaces.pi.ProcessDefinition;
import petruchio.pi.readers.cup.CUPLexer;
import petruchio.pi.readers.cup.CUPMWBPiParser;
import petruchio.pi.readers.cup.CUPParser;
import petruchio.pi.readers.cup.MWBPiLexer;


/**
 * A wrapper class for the cup generated parser for pi calculus processes
 * written in the mobile workbench syntax.
 * 
 * @author Tim Strazny
 */
public class MWBPiParser extends CUPParser {
	
	private Properties properties = null;
    
    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#getLexer(java.io.Reader)
     */
    @Override
    public CUPLexer getLexer(final Reader reader) {
        return new MWBPiLexer(reader);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParser#getDefaultExtensions()
     */
    @Override
    public String getDefaultExtensions() {
        return "mwbpi";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#getDescription()
     */
    @Override
    public String getDescription() {
        return "The default parser for pi calculus process definitions in the "
            + "mobility workbench (MWB) tool's format. (Using the ANTLR parser "
            + "generator.) NOTE that syntactical constructs (\\x,y,...,z)P and "
            + "[x,y,...z] are NOT supported.";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParser#getDummyProcessDefinitionPrefix()
     */
    @Override
    public String getDummyProcessDefinitionPrefix() {
        return "agent X = ";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParser#getDummyProcessDefinitionSuffix()
     */
    @Override
    public String getDummyProcessDefinitionSuffix() {
        return "";
    }
    

    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#parseProcessDefinitions(petruchio.pi.readers.cup.CUPLexer, java.util.Collection, petruchio.interfaces.pi.ProcessCreator)
     */
    @Override
    public Collection<ProcessDefinition<?>> parseProcessDefinitions(
                    final CUPLexer lexer,
                    final Collection<ParserErrorHandler> handlers,
                    final ProcessCreator pbuilder) {
    	final CUPMWBPiParser parser = new CUPMWBPiParser(lexer);
        final Collection<ProcessDefinition<?>> defs = parser.parseProcessDefinitions(pbuilder, handlers);
        this.properties = parser.getProperties();
        return defs;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#getProperties()
     */
    @Override
    public Properties getProperties() {
        return this.properties;
    }
    
}