/*
 * Project:  Petruchio
 * Filename: PiParser.java
 * Created:  13.08.2007
 * Author:   Tim Strazny
 */
package petruchio.pi.readers.cup;

import java.io.Reader;
import java.util.Collection;
import java.util.Properties;

import petruchio.interfaces.ParserErrorHandler;
import petruchio.interfaces.pi.ProcessCreator;
import petruchio.interfaces.pi.ProcessDefinition;
import petruchio.pi.readers.cup.CUPLexer;
import petruchio.pi.readers.cup.CUPParser;
import petruchio.pi.readers.cup.CUPPiParser;
import petruchio.pi.readers.cup.PiLexer;


/**
 * A wrapper class for the cup generated parser for pi calculus processes
 * written in the petruchio syntax.
 * 
 * @author Tim Strazny
 */
public class PiParser extends CUPParser {

	private Properties properties = null;
	
    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#getLexer(java.io.Reader)
     */
    @Override
    public CUPLexer getLexer(final Reader reader) {
        return new PiLexer(reader);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParser#getDefaultExtensions()
     */
    @Override
    public String getDefaultExtensions() {
        return "pi";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParser#getDummyProcessDefinitionPrefix()
     */
    @Override
    public String getDummyProcessDefinitionPrefix() {
        return "X := ";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.pi.PiParser#getDummyProcessDefinitionSuffix()
     */
    @Override
    public String getDummyProcessDefinitionSuffix() {
        return ";";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#getDescription()
     */
    @Override
    public String getDescription() {
        return "The default parser for pi calculus process definitions. (Using the CUP parser generator.)";
    }
    

    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#parseProcessDefinitions(petruchio.pi.readers.cup.CUPLexer, java.util.Collection, petruchio.interfaces.pi.ProcessCreator)
     */
    @Override
    public Collection<ProcessDefinition<?>> parseProcessDefinitions(
                    final CUPLexer lexer,
                    final Collection<ParserErrorHandler> handlers,
                    final ProcessCreator pbuilder) {
    	final CUPPiParser parser = new CUPPiParser(lexer);
        final Collection<ProcessDefinition<?>> defs = parser.parseProcessDefinitions(pbuilder, handlers);
        this.properties = parser.getProperties();
        return defs;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pi.readers.cup.CUPParser#getProperties()
     */
    @Override
    public Properties getProperties() {
        return this.properties;
    }
    
}