package petruchio.pn;

import petruchio.interfaces.petrinet.IntMatrix;
import petruchio.interfaces.petrinet.Place;

public class InvariantAnalysis {
    /**
     * Returns a matrix where the columns form minimal-support non-negative integer solutions
     * of the integer linear equation system defined by given matrix.
     * 
     * @param incidence A matrix (which will be altered).
     * @return Matrix where columns form minimal-support non-negative integer solutions.
     */
    public static IntMatrix findInvariants(final IntMatrix incidence) {
        return findInvariants(incidence, true, 0, null);
    }
    
    
    /**
     * Returns a matrix where the columns form minimal-support (possibly non-negative) integer solutions
     * of the integer linear equation system defined by given matrix.
     * 
     * @param incidence A matrix (which will be altered).
     * @param semipositive Whether the solutions shall be non-negative.
     * @return Matrix where columns form minimal-support (possibly non-negative) integer solutions.
     */
    public static IntMatrix findInvariants(final IntMatrix incidence, final boolean semipositive) {
        return findInvariants(incidence, semipositive, -1, null);
    }
    
    /**
     * Returns a matrix where the columns form minimal-support (possibly non-negative) integer solutions
     * of the integer linear equation system defined by given matrix.
     * 
     * @param incidence A matrix (which will be altered).
     * @param semipositive Whether the solutions shall be non-negative.
     * @param count Number of rounds when attempting to make solutions non-negative.
     * @param indices 
     * @return Matrix where columns form minimal-support (possibly non-negative) integer solutions.
     */
    public static IntMatrix findInvariants(final IntMatrix incidence, final boolean semipositive,
            final int count, final Place[] indices) {
        final IntMatrix copy = IntPNMatrix.fromMatrix(incidence);
        try {
            return IntInvariantAnalysis.findInvariants(incidence, semipositive, count, indices);
        } catch(final ArithmeticException e) {
            System.out.println("Problem during invariant computation: " + e + "\n retrying with higher precision.");
        }
        try {
            return LongInvariantAnalysis.findInvariants(copy, semipositive, count, indices);
        } catch(final ArithmeticException e) {
            System.out.println("Problem during invariant computation: " + e + "\n retrying with unlimited precision.");
        }
        return BigIntegerInvariantAnalysis.findInvariants(copy, semipositive, count, indices);
    }
}
