/*
 * Project:  Petruchio
 * Filename: PTArc.java
 * Created:  05.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pn;


import net.jcip.annotations.ThreadSafe;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;

/**
 * <Description>
 * 
 * @author Tim Strazny
 */
@ThreadSafe
public class PTArc implements petruchio.interfaces.petrinet.PTArc {
    
    /**
     * 
     */
    private int weight;
    
    /**
     * 
     */
    private final Place p;
    
    /**
     * 
     */
    private final Transition t;
    
    
    /**
     * @param p
     * @param t
     * @param weight
     */
    public PTArc(final Place p, final Transition t, final int weight) {
        if(t == null) {
            throw new RuntimeException("The transition connected to an arc may not be null!");
        }
        if(p == null) {
            throw new RuntimeException("The place connected to an arc may not be null!");
        }
        this.p = p;
        this.t = t;
        this.weight = weight;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PTArc#getSource()
     */
    @Override
    public Place getSource() {
        return this.p;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PTArc#getTarget()
     */
    @Override
    public Transition getTarget() {
        return this.t;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object o) {
        return (o instanceof PTArc) && this.equals((PTArc)o);
    }
    
    
    /**
     * TODO: document
     *
     * @param arc
     * @return
     */
    public boolean equals(final PTArc arc) {
        return (arc != null) && (arc.p == this.p) && (arc.t == this.t);
    }

    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return this.getSource() + " =" + this.getWeight() + "=> " + this.getTarget();
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PTArc#getWeight()
     */
    @Override
    public int getWeight() {
        return this.weight;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PTArc#setWeight(int)
     */
    @Override
    public void setWeight(final int weight) {
        this.weight = weight;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return this.p.hashCode() ^ this.t.hashCode() ^ this.weight;
    }
    
    
    /**
     * TODO: document
     *
     * @param w
     */
    public void addWeight(final int w) {
        this.weight += w;
    }
    
}
