/*
 * Project:  Petruchio
 * Filename: Place.java
 * Created:  05.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pn;

import java.util.Collection;
import java.util.concurrent.CopyOnWriteArrayList;

import net.jcip.annotations.GuardedBy;
import net.jcip.annotations.ThreadSafe;

import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.TPArc;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
@ThreadSafe
public class Place implements petruchio.interfaces.petrinet.Place {
    
    private int marking;
    
    private String name;
    
    private String note;
    
    private String meaning;
   
    private int bound;
    
    private int x;
    
    private boolean isFinal;
    
    private boolean isParametric;
    
    private int y;
    
    @GuardedBy("threadsafe")
    private final Collection<TPArc> input;
    
    @GuardedBy("threadsafe")
    private final Collection<PTArc> output;
    
    private boolean couldCommunicate;
    
    public volatile Object data = null;
    
    public volatile boolean known = false;
    
    
    /**
     * @param name
     * @param marking
     */
    public Place(final String name, final int marking) {
        this.name = name;
        this.marking = marking;
        this.input = new CopyOnWriteArrayList<TPArc>();
        this.output = new CopyOnWriteArrayList<PTArc>();
        this.note = "";
        this.meaning = "";
        this.bound = petruchio.interfaces.petrinet.Place.BOUND_UNKNOWN;
        this.couldCommunicate = true;
        this.isFinal = false;
        this.x = 0;
        this.y = 0;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setMeaning(java.lang.String)
     */
    @Override
    public void setMeaning(final String meaning) {
    	if(meaning == null) {
    	    this.meaning = "";
    	} else {
    	    this.meaning = meaning;
    	}
    }
    
    @Override
    public boolean boundKnown() {
        return this.bound != petruchio.interfaces.petrinet.Place.BOUND_UNKNOWN
            && this.bound != petruchio.interfaces.petrinet.Place.BOUND_UNBOUNDED;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getMeaning()
     */
    @Override
    public String getMeaning() {
    	return this.meaning;
    }
   
    
    /**
     * 
     */
    public void clear() {
        this.input.clear();
        this.output.clear();
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getName()
     */
    @Override
    public String getName() {
        return this.name;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getMarking()
     */
    @Override
    public int getMarking() {
        return this.marking;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setMarking(int)
     */
    @Override
    public void setMarking(final int tokens) {
        this.marking = tokens;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setBound(int)
     */
    @Override
    public void setBound(final int bound) {
        if(bound < 0 && bound != Place.BOUND_UNBOUNDED && bound != Place.BOUND_UNKNOWN) {
            throw new IllegalArgumentException("Negative bound: " + bound);
        }
        this.bound = bound;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getBound()
     */
    @Override
    public int getBound() {
        return this.bound;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#addToken()
     */
    @Override
    public void addToken() {
        ++this.marking;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#addTokens(int)
     */
    @Override
    public void addTokens(final int tokens) {
        if(this.marking + tokens < 0) {
            throw new IllegalArgumentException("Negative token count!");
        }
        this.marking += tokens;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getInput()
     */
    @Override
    public Collection<TPArc> getInput(){
        return this.input;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getOutput()
     */
    @Override
    public Collection<PTArc> getOutput(){
        return this.output;
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void addInput(final TPArc t) {
        this.input.add(t);
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void addOutput(final PTArc t) {
        this.output.add(t);
    }
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void removeInput(final TPArc t) {
        this.input.remove(t);
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void removeOutput(final PTArc t) {
        this.output.remove(t);
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return this.getName();
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getX()
     */
    @Override
    public int getX() {
        return this.x;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setX(int)
     */
    @Override
    public void setX(final int x) {
        this.x = x;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getY()
     */
    @Override
    public int getY() {
        return this.y;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setY(int)
     */
    @Override
    public void setY(final int y) {
        this.y = y;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#getNote()
     */
    @Override
    public String getNote() {
        return this.note;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setNote(java.lang.String)
     */
    @Override
    public void setNote(final String note) {
        this.note = note;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setName(java.lang.String)
     */
    @Override
    public void setName(final String name) {
        this.name = name;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#couldCommunicate()
     */
    @Override
    public boolean couldCommunicate() {
        return this.couldCommunicate;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setCouldCommunicate(boolean)
     */
    @Override
    public void setCouldCommunicate(final boolean b) {
        this.couldCommunicate = b;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#isFinal()
     */
    @Override
    public boolean isFinal() {
        return this.isFinal;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setFinal(boolean)
     */
    @Override
    public void setFinal(final boolean isFinal) {
        this.isFinal = isFinal;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#isParametric()
     */
    @Override
    public boolean isParametric() {
        return this.isParametric;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Place#setParametric(boolean)
     */
    @Override
    public void setParametric(boolean isParametric) {
        this.isParametric = isParametric;
    }

}