/*
 * Project:  Petruchio
 * Filename: Transition.java
 * Created:  05.12.2006
 * Author:   Tim Strazny
 */
package petruchio.pn;

import java.util.Collection;
import java.util.concurrent.CopyOnWriteArrayList;

import net.jcip.annotations.GuardedBy;
import net.jcip.annotations.ThreadSafe;

import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.petrinet.PTArc;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
@ThreadSafe
public class Transition implements petruchio.interfaces.petrinet.Transition {
    
    private String name;
    
    private String meaning;
    
    private int x;
    
    private int y;
    
    @GuardedBy("threadsafe")
    private final Collection<PTArc> input;
    
    @GuardedBy("threadsafe")
    private final Collection<TPArc> output;
    
    public volatile Object data = null;
    
    public volatile boolean known = false;
    
    
    /**
     * @param name
     */
    public Transition(final String name) {
        this.name = name;
        this.meaning = "";
        this.input = new CopyOnWriteArrayList<PTArc>();
        this.output = new CopyOnWriteArrayList<TPArc>();
        this.x = 0;
        this.y = 0;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#setMeaning(java.lang.String)
     */
    @Override
    public void setMeaning(final String meaning) {
    	if(meaning == null) {
    		this.meaning = "";
    	} else {
    		this.meaning = meaning;
    	}
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#getMeaning()
     */
    @Override
    public String getMeaning() {
    	return this.meaning;
    }
    
    
    /**
     * 
     */
    public void clear() {
        this.input.clear();
        this.output.clear();
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void addInput(final PTArc t) {
        this.input.add(t);
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void addOutput(final TPArc t) {
        this.output.add(t);
    }
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void removeInput(final PTArc t) {
        this.input.remove(t);
    }
    
    
    /**
     * TODO: document
     *
     * @param t
     */
    public void removeOutput(final TPArc t) {
        this.output.remove(t);
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#setName(java.lang.String)
     */
    @Override
    public void setName(final String name) {
        this.name = name;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#getName()
     */
    @Override
    public String getName() {
        return this.name;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return this.getName();
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#getInput()
     */
    @Override
    public Collection<PTArc> getInput() {
        return this.input;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#getOutput()
     */
    @Override
    public Collection<TPArc> getOutput() {
        return this.output;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#getX()
     */
    @Override
    public int getX() {
        return this.x;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#setX(int)
     */
    @Override
    public void setX(final int x) {
        this.x = x;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#getY()
     */
    @Override
    public int getY() {
        return this.y;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.Transition#setY(int)
     */
    @Override
    public void setY(final int y) {
        this.y = y;
    }
    
}