/*
 * Project:  Petruchio
 * Filename: FileInput.java
 * Created:  25.09.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.rdiginput;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.petrinet.PetriNetReductorIgnorerInput;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public class FileInput implements SelfDescribing, PetriNetReductorIgnorerInput {
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "Interprets the given PetriNetReductorIgnorerInput to be a file.";
    }
    
	
	/* (non-Javadoc)
	 * @see petruchio.interfaces.Resettable#reset()
	 */
	@Override
    public void reset() {
		// no caches
	}
	

	/* (non-Javadoc)
	 * @see petruchio.interfaces.petrinet.PetriNetReductorIgnorerInput#getInput(java.lang.String)
	 */
	@Override
    public InputStream getInput(String value) {
		if(value == null) {
			return null;
		}
		try {
			return new FileInputStream(FileInput.getAbsolutePath(value));
		} catch(final FileNotFoundException e) {
			throw new RuntimeException("Unable to open file: " + value, e);
		}
	}
	
	
	/**
     * Builds an absolute path from a given file and the current user directory.
     *
     * @param file A relative or absolute path to the file
     * @return The absolute path for the given file name
     */
    private static String getAbsolutePath(final String file) {
        final File f = new File(file);
        if(!f.isAbsolute()) {
            final File f2 = new File(System.getProperty("user.dir")
                            + File.separator + file);
            try {
                return f2.getCanonicalPath();
            } catch(final IOException e) {
                return f2.getAbsolutePath();
            }
        }
        return file;
    }

}