/*
 * Project:  Petruchio
 * Filename: CMReader.java
 * Created:  02.10.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.readers;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.IdentityHashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Scanner;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import petruchio.common.IdentityHashSet;
import petruchio.interfaces.LocalConfigurationParser;
import petruchio.interfaces.ParserErrorHandler;
import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.petrinet.ExtendedTransition;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReader;
import petruchio.interfaces.petrinet.PetriNetStreamReader;
import petruchio.interfaces.petrinet.PetriNetStreamWriter;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.TransferArc;
import petruchio.interfaces.petrinet.Transition;
import petruchio.pi.readers.LocalConfigParser;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public class CMReader implements SelfDescribing, PetriNetReader, PetriNetStreamReader, LocalConfigurationParser {
    
    private static final String COMMENT = "(?:\\s*#.*?(?:$|\n))*";
    private static final boolean SINGLE_TRANSITION_FOR_PARAMETERS = false;
    private static final boolean PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS = false;
    private static final boolean USE_PARAMETRIC_PLACES = false;
    
    private InputStream in = null;
    private Collection<ParserErrorHandler> errorHandlers = new ArrayList<ParserErrorHandler>();

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "A Petri net reader for files in a simple counter machine format "
            + "as used by G. Geeraerts, J.-F. Raskin, L. Van Begin on "
            + "http://www.ulb.ac.be/di/ssd/ggeeraer/eec/";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReader#addErrorHandler(petruchio.interfaces.ParserErrorHandler)
     */
    @Override
    public void addErrorHandler(final ParserErrorHandler handler) {
        this.errorHandlers.add(handler);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReader#getDefaultExtension()
     */
    @Override
    public String getDefaultExtensions() {
        return "spec";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReader#read(java.io.InputStream)
     */
    @Override
    public PetriNet read() {
        final PetriNet net = new petruchio.pn.PetriNet();
        this.read(net);
        return net;
    }
        
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReader#read(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public void read(final PetriNet net) {
        try {
            final Scanner s = new Scanner(this.in).useDelimiter("\\s+");
            
            net.setHeaderComment(this.readComment(s));
//            s.useDelimiter("\\s+" + COMMENT + "\\s*");
            
            final Map<String, Place> pl = new HashMap<String, Place>();
            
//            while(!s.next().equals("rules"));
            boolean containsExtended = false;
            s.skip("(?ms)(?:.*?^\\s*rules\\b" + COMMENT + "?)");
            final Pattern pat = Pattern.compile(
                    "(?ms)" + COMMENT + "\\s*"
                  + "([+-])?\\s*" // operator (group 1)
                  + COMMENT + "\\s*"
                  + "(?:(\\d+)\\s*" + COMMENT + "\\*)?" // multiplier (group 2)
                  + COMMENT + "\\s*"
                  + "((?:\\w|\\d)+)\\s*" // place name or number literal (group 3)
                  + COMMENT); // end of input
            int i = 1;
            final Pattern precondition = Pattern.compile(
                      "(?ms)" + COMMENT
                    + "([^#]*?)" // constraint for transition (group 1)
                    + COMMENT
                    + "->" // transition "operator"
                );
            final Pattern postcondition = Pattern.compile(
                      "(?ms)" + COMMENT
                    + "((?:[^#;]|" + COMMENT + ")*?)" // group 1
                    + ";" + COMMENT
                );
            while(!s.hasNext("init")) {
                //s.skip(precondition);
                s.useDelimiter("(?<=->)|\\s*"+COMMENT+"\\s*(,|(?=->))\\s*"+COMMENT+"\\s*");
                //final String[] pre = s.match().group(1).trim().split("\\s*,\\s*");
                final Map<String, Integer> incoming = new HashMap<String, Integer>();
                final Map<String, Integer> inhibitor = new HashMap<String, Integer>();
                final Map<String, Integer> outgoing = new HashMap<String, Integer>();
                while(!s.hasNext("->")) {
                    final String p = s.next().replaceAll(COMMENT, "").trim();
                    if(p.isEmpty()) {
                        continue;
                    }
                    int idx = p.indexOf('>');
                    if(idx < 0) {
                        containsExtended = true;
                        idx = p.indexOf('<');
                        if(idx < 0) {
                            idx = p.indexOf('=');
                            if(idx < 0) {
                                throw new RuntimeException("Unexpected operator in guard: " + p);
                            }
                            final Integer w = Integer.valueOf(p.substring(idx+1).trim());
                            final String place = p.substring(0, idx).trim();
                            inhibitor.put(place, w);
                            if(w.intValue() != 0) {
                                incoming.put(place, w);
                            }
                            continue;
                        } 
                        
                        final Integer w;
                        if(p.charAt(idx+1) != '=') {
                            w = Integer.valueOf(1+Integer.parseInt(p.substring(idx+1).trim()));
                        } else {
                            w = Integer.valueOf(p.substring(idx+2).trim());
                        }
                        final String place = p.substring(0, idx).trim();
                        inhibitor.put(place, w);
                        continue;
                    }
                    
                    final String place = p.substring(0, idx).trim();
                    final Integer w;
                    if(p.charAt(idx+1) != '=') {
                        w = Integer.valueOf(1+Integer.parseInt(p.substring(idx+1).trim()));
                    } else {
                        w = Integer.valueOf(p.substring(idx+2).trim());
                    }
                    incoming.put(place, w);
                    outgoing.put(place, Integer.valueOf(0));
                }
                s.skip("\\s*->\\s*");
//                System.out.println("---");
                s.useDelimiter("(?<=;)|\\s*"+COMMENT+"\\s*(,|(?=;))\\s*"+COMMENT+"\\s*");
//                s.skip(postcondition);
//                final String[] post = s.match().group(1).trim().split("\\s*,\\s*");
                boolean isExtendedTransition = !inhibitor.isEmpty();
                final Map<String, Map<Place, Integer>> arcs = new HashMap<String, Map<Place, Integer>>();
                while(!s.hasNext(";")) {
                    final String arc = s.next().replaceAll(COMMENT, "").trim();
                    if(arc.isEmpty()) {
                        continue;
                    }
                    final String place = arc.substring(0, arc.indexOf('\'')).trim();
                    final String operation = arc.substring(arc.indexOf('=')+1);
                    final Matcher arcMatcher = pat.matcher(operation);
                    // on the right of the assignment "=" there is a linear combination of places
                    // e.g. x0' = x0+1
                    // or x0' = x12 + x3 - 2
                    int constantpart = 0;
                    final Map<Place, Integer> multipliers = new IdentityHashMap<Place, Integer>();
                    while(arcMatcher.find()) {
                        final String operator = arcMatcher.group(1);
                        final String multiplier = arcMatcher.group(2);
                        final String operand = arcMatcher.group(3);
                        final int sign = operator != null && operator.equals("-") ? -1 : 1;
                        final int mult = multiplier == null || multiplier.isEmpty() ? 1 : Integer.parseInt(multiplier);
                        try {
                            final int literal = Integer.parseInt(operand);
                            constantpart += sign*mult*literal;
                        } catch(final NumberFormatException e) {
                            multipliers.put(this.getPlace(net, pl, operand), Integer.valueOf(sign*mult));
                            if(!isExtendedTransition
                                    && (sign*mult != 1 || !operand.equals(place))) {
                                isExtendedTransition = true;
                            }
                        }
                    }
                    if(!isExtendedTransition && multipliers.isEmpty()) {
                        isExtendedTransition = true;
                    }
                    outgoing.put(place, Integer.valueOf(constantpart));
                    arcs.put(place, multipliers);
                }
                s.skip("\\s*;\\s*");
                if(!isExtendedTransition) {
                    final Transition t = net.addTransition("t" + (i++));
                    for(final Map.Entry<String, Integer> out : outgoing.entrySet()) {
                        final Place thePlace = this.getPlace(net, pl, out.getKey());
                        final Integer min = incoming.remove(thePlace.getName());
                        final int w = out.getValue().intValue();
                        final int in = min == null ? 0 : min.intValue();
                        if(w > 0) {
                            if(min != null) {
                                net.addArc(thePlace, t, in);
                                net.addArc(t, thePlace, in-w);
                            } else {
                                net.addArc(t, thePlace, w);
                            }
                        } else {
                            if((min != null) && -w != in) {
                                net.addArc(t, thePlace, in);
                                net.addArc(thePlace, t, -w+in);
                            } else {
                                net.addArc(thePlace, t, -w);
                            }
                        }
                    }

                    for(final Map.Entry<String, Integer> e : incoming.entrySet()) {
                        final String pname = e.getKey();
                        final Place p = this.getPlace(net, pl, pname);
                        net.addArc(p, t, e.getValue().intValue());
                    }
                    
                } else { // an extended transition
                    containsExtended = true;
                    final petruchio.pn.PetriNet pn = (petruchio.pn.PetriNet)net;
                    final ExtendedTransition t = pn.addExtendedTransition("t" + (i++));
                    for(final Map.Entry<String, Integer> out : outgoing.entrySet()) {
                        final Place thePlace = this.getPlace(net, pl, out.getKey());
                        final Map<Place, Integer> multipliers = arcs.get(thePlace.getName());
                        final int constantpart = out.getValue().intValue();
                        final Integer min = incoming.remove(thePlace.getName());
                        final int in = min == null ? 0 : min.intValue();
                        
                        final boolean reset = multipliers != null && !multipliers.containsKey(thePlace);
                        if(reset) {
                            // reset arcs
                            pn.addResetArc(thePlace, t, in);
                        }
                        if(multipliers != null && (multipliers.size() > 1 || (reset && !multipliers.isEmpty()))) {
                            // transfer arc
                            if(in > 0) {
                                net.addArc(thePlace, t, in);
                                net.addArc(t, thePlace, in);
                            }
                            final TransferArc a;
                            if(constantpart > 0) {
                                if(min != null) {
                                    net.addArc(thePlace, t, in);
                                    a = pn.addTransferArc(t, thePlace, in-constantpart);
                                } else {
                                    a = pn.addTransferArc(t, thePlace, constantpart);
                                }
                            } else {
                                if((min != null) && -constantpart != in) {
                                    a = pn.addTransferArc(t, thePlace, in);
                                    net.addArc(thePlace, t, -constantpart+in);
                                } else {
                                    a = pn.addTransferArc(t, thePlace, 0);
                                    net.addArc(thePlace, t, -constantpart);
                                }
                            }
                            
                            a.setMultipliers(multipliers);
                        } else if(reset) {
                            // just a reset arc
                            net.addArc(t, thePlace, constantpart);
                        } else {
                            // normal arc
                            if(constantpart > 0) {
                                if(min != null) {
                                    net.addArc(thePlace, t, in);
                                    net.addArc(t, thePlace, in-constantpart);
                                } else {
                                    net.addArc(t, thePlace, constantpart);
                                }
                            } else {
                                if((min != null) && -constantpart != in) {
                                    net.addArc(t, thePlace, in);
                                    net.addArc(thePlace, t, -constantpart+in);
                                } else {
                                    net.addArc(thePlace, t, -constantpart);
                                }
                            }
                        }
                    }

                    for(final Map.Entry<String, Integer> e : incoming.entrySet()) {
                        final String pname = e.getKey();
                        final Place p = this.getPlace(net, pl, pname);
                        net.addArc(p, t, e.getValue().intValue());
                    }
                    for(final Map.Entry<String, Integer> e : inhibitor.entrySet()) {
                        final String pname = e.getKey();
                        final Place p = this.getPlace(net, pl, pname);
                        pn.addInhibitorArc(p, t).setBound(e.getValue().intValue());
                    }
                }
//                System.out.println("--- " + t + " ---");
//                System.out.println("in:  " + t.getInput());
//                System.out.println("out: " + t.getOutput());
                
                s.useDelimiter("\\s+");
                s.skip(COMMENT);
            }
            s.next();
            s.skip("(?ms)" + COMMENT);
            s.useDelimiter("\\s*[\n,]\\s*"); // linebreaks and commas
            final Pattern p = Pattern.compile("\\s*"
                    + "(\\w+)" // place name (group 1)
                    + "\\s*"
                    + "([<>]?=|[<>])" // operator (group 2)
                    + "\\s*"
                    + "(\\d+)" // natural number (group 3)
                    + "\\s*"
                    + "[,\n]?"); // delimiter
            final Matcher m = p.matcher("");
            final Collection<Transition> intialPost = new IdentityHashSet<Transition>();
            final Collection<Transition> parameterizers = new ArrayList<Transition>();
            boolean hasParameter = false;
            Transition param_hi = null;
            while(!s.hasNext("target") && s.hasNext()) {
                final String init = s.next();
                m.reset(init);
                if(m.find()) {
                    final String place = m.group(1);
                    final String op = m.group(2);
                    final int marking = Integer.parseInt(m.group(3));
                    final Place thePlace = this.getPlace(net, pl, place);
                    thePlace.setMarking(marking);
                    if(op.equals("=")) {
                        // perfect
                    } else if(op.equals("<=") || op.equals("<")) {
                        // option to remove tokens
                        hasParameter = true;
                        if(op.equals("<")) {
                            thePlace.addTokens(-1);    
                        }
                        final Transition t = net.addTransition("tparam_" + place);
                        if(containsExtended || PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) {
                            parameterizers.add(t);
                        }
                        if(containsExtended || PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) {
                            for(final PTArc a : thePlace.getOutput()) {
                                intialPost.add(a.getTarget());
                            }
                        }
                        net.addArc(thePlace, t, 1);
                    } else if(op.equals(">=") || op.equals(">")) {
                        // option to add tokens
                        hasParameter = true;
                        thePlace.setBound(Place.BOUND_UNBOUNDED);
                        if(op.equals(">")) {
                            thePlace.addTokens(1);    
                        }
                        if(USE_PARAMETRIC_PLACES) {
                            thePlace.setParametric(true);
                        } else {
                            final Transition t;
                            if(SINGLE_TRANSITION_FOR_PARAMETERS) {
                                t = param_hi == null ? net.addTransition("tparam") : param_hi;
                            } else {
                                t = net.addTransition("tparam_" + place);
                            }
                            if((containsExtended || PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) && param_hi == null) {
                                parameterizers.add(t);
                                if(SINGLE_TRANSITION_FOR_PARAMETERS) {
                                    param_hi = t;
                                }
                            }
                            if(containsExtended || PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) {
                                for(final PTArc a : thePlace.getOutput()) {
                                    intialPost.add(a.getTarget());
                                }
                            }
                            net.addArc(t, thePlace, 1);
                        }
                    } else {
                        throw new InternalError("unexpected op at initial constraint: " + op);
                    }
                }
            }
            if((containsExtended || PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) && !parameterizers.isEmpty()) {
                int prm = 0;
                String init = "init";
                while(pl.containsKey(init)) {
                    init = "init" + (prm++);
                }
                prm = 0;
                String run = "run";
                while(pl.containsKey(run)) {
                    run = "run" + (prm++);
                }
                final Place initing = net.addPlace(init, 1);
                final Place running = net.addPlace(run, 0);
                final Transition start = net.addTransition("tstart");
                net.addArc(initing, start, 1);
                net.addArc(start, running, 1);
                for(final Transition t : parameterizers) {
                    net.addArc(initing, t, 1);
                    net.addArc(t, initing, 1);
                }
                for(final Transition t : intialPost) {
                    net.addArc(running, t, 1);
                    net.addArc(t, running, 1);
                }
            }
            if(!hasParameter && !containsExtended) {
                while(!s.hasNext("invariants") && s.hasNext()) {
                    s.next();
                }
                s.useDelimiter("\\s*\n\\s*");
                while(s.hasNext()) {
                    final String init = s.next();
                    m.reset(init);
                    final Map<Place, Integer> inv = new IdentityHashMap<Place, Integer>();
                    int sum = 0;
                    while(m.find()) {
                        final String op = m.group(2);
                        if(op.equals("=")) {
                            final int weight = Integer.parseInt(m.group(3));
                            if(weight > 0) {
                                final String place = m.group(1);
                                final Place thePlace = this.getPlace(net, pl, place);
                                inv.put(thePlace, weight);
                                sum += weight*thePlace.getMarking();
                            } else {
                                inv.clear();
                                break;
                            }
                        } else {
                            inv.clear();
                            break;
                        }
                    }
                    if(!inv.isEmpty()) {
//                        System.out.println(inv + " = " + sum);
                        for(final Map.Entry<Place, Integer> e : inv.entrySet()) {
                            e.getKey().setBound(sum / e.getValue());
//                            System.out.println(e.getKey() + " <= " + e.getKey().getBound());
                        }
                    }
                }
            }
            
        } catch(final RuntimeException e) {
            this.reportError(e);
            throw e;
        } finally {
            try {
                this.in.close();
            } catch(final IOException e) {
                // ignore
            }
            this.in = null;
        }
//        System.out.println(net);
    }
    
    
    /**
     * TODO: description
     *
     * @param s
     * @param net
     */
    private String readComment(final Scanner s) {
        return s.skip("(?m)(\\s*#.*$)*").match().group().trim().replaceAll("(?m)^\\s*#\\s", "");
    }
    
    
    /**
     * TODO: description
     *
     * @param pl
     * @param place
     * @return
     */
    private Place getPlace(final PetriNet net, final Map<String, Place> pl, final String place) {
        Place thePlace = pl.get(place);
        if(thePlace == null) {
            thePlace = net.addPlace(place, 0);
            pl.put(place, thePlace);
        }
        return thePlace;
    }
    
    
//    /**
//     * TODO: description
//     *
//     * @param pl
//     * @param place
//     * @return
//     */
//    private Place getPlace(final Map<String, Place> pl, final String place) {
//        Place thePlace = pl.get(place);
//        if(thePlace == null) {
//            thePlace = new petruchio.pn.Place(place, 0);
//            pl.put(place, thePlace);
//        }
//        return thePlace;
//    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReader#setInput(java.io.InputStream)
     */
    @Override
    public void setInput(final InputStream in) {
        this.in = in;
    }

    
    /**
     * TODO: description
     *
     * @param e
     */
    private void reportError(final Exception e) {
        for(final ParserErrorHandler handler : this.errorHandlers) {
            handler.reportError(e.getLocalizedMessage());
        }
        if(this.errorHandlers.isEmpty()) {
            e.printStackTrace();
        }
    }
    

    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        this.errorHandlers.clear();
        if(this.in != null) {
            try {
                this.in.close();
            } catch(final IOException e) {
                // ignore
            }
            this.in = null;
        }
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.LocalConfigurationParser#getLocalConfiguration()
     */
    @Override
    public Properties getLocalConfiguration() {
        try {
            final Scanner s = new Scanner(this.in);
            final String cmt = s.skip("(?m)" + COMMENT).match().group().trim().replaceAll("(?m)^\\s*#\\s", "");
            final LocalConfigurationParser p = new LocalConfigParser();
            p.setInput(new ByteArrayInputStream(cmt.getBytes()));
            final Properties props = p.getLocalConfiguration();
            p.reset();
            return props;
        } catch(final RuntimeException e) {
            this.reportError(e);
            throw e;
        }
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamConverter#convertTo(petruchio.interfaces.petrinet.PetriNetStreamWriter)
     */
    @Override
    public void readTo(final PetriNetStreamWriter writer) {
        try {
            final Scanner s = new Scanner(this.in).useDelimiter("\\s+");
            
            writer.beginNet("noname");
            writer.writeHeaderComment(this.readComment(s));
            
            final Map<String, Integer> incoming = new HashMap<String, Integer>();
            final Collection<String> placeNames = new HashSet<String>();
            final Map<String, Collection<String>> post = new HashMap<String, Collection<String>>();
    
            s.skip("(?ms)(?:.*?^\\s*rules\\b" + COMMENT + "?)");
            int i = 1;
            while(!s.hasNext("init")) {
                final String trans = "t" + (i++);
                writer.writeTransition(new petruchio.pn.Transition(trans));
//                s.skip("(?ms)" + COMMENT
//                        + "([^#]*?)" // constraint for transition (group 1)
//                        + COMMENT
//                        + "->" // transition "operator"
//                        + COMMENT);
                s.useDelimiter("(?<=->)|\\s*"+COMMENT+"\\s*(,|(?=->))\\s*"+COMMENT+"\\s*");
//                final String[] pre = s.match().group(1).trim().split("\\s*,\\s*");
                while(!s.hasNext("->")) {
                    final String p = s.next().replaceAll(COMMENT, "").trim();
                    if(p.isEmpty()) {
                        continue;
                    }
                    final int idx = p.indexOf('>');
                    final String place = p.substring(0, idx).trim();
                    final Integer w;
                    if(p.charAt(idx+1) != '=') {
                        w = Integer.valueOf(p.substring(idx+1).trim())+1;
                    } else {
                        w = Integer.valueOf(p.substring(idx+2).trim());
                    }
                    if(PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) {
                        Collection<String> postCol = post.get(place);
                        if(postCol == null) {
                            postCol = new ArrayList<String>();
                            post.put(place, postCol);
                        }
                        postCol.add(trans);
                    }
                    incoming.put(place, w);
                }
                s.skip("\\s*->\\s*");
                s.useDelimiter("\\s+");
                
                boolean end;
                do {
                    try {
                        s.skip("(?ms)" + COMMENT + "\\s*"
                                + "(?:(\\w+?)'" // primed place name (group 1 without prime) 
                                + COMMENT + "\\s*"
                                + "=" // assignment operator
                                + COMMENT + "\\s*"
                                + "\\w+" // the same place name
                                + COMMENT + "\\s*"
                                + "(\\+|-)" // operator
                                + COMMENT + "\\s*"
                                + "(\\d+))?" // natural number (group 2)
                                + COMMENT + "\\s*"
                                + "(,|;)"); // end of operation
                        final MatchResult m = s.match();
                        end = m.group(4).equals(";");
                        final String place = m.group(1);
                        if(place != null) {
                            final boolean add = m.group(2).equals("+");
                            final Integer w = Integer.valueOf(m.group(3));
                            
                            final Integer min = incoming.remove(place);
                            if(add) {
                                if(min != null) {
                                    writer.writePTArc(place, trans, min);
                                    writer.writeTPArc(trans, place, min-w);
                                } else {
                                    writer.writeTPArc(trans, place, w);
                                }
                            } else {
                                if((min != null) && !w.equals(min)) {
                                    writer.writeTPArc(trans, place, min);
                                    writer.writePTArc(place, trans, w+min);
                                } else {
                                    writer.writePTArc(place, trans, w);
                                }
                            }
                        }
                    } catch(final RuntimeException e) {
                        throw new RuntimeException("Error reading transition:\n" + s.match().group().trim(), e);
                    }
                    
                } while(!end);
                for(final Map.Entry<String, Integer> e : incoming.entrySet()) {
                    final String place = e.getKey();
                    writer.writePTArc(place, trans, e.getValue());
                    writer.writeTPArc(trans, place, e.getValue());
                }
                incoming.clear();
            }
            s.next();
            s.skip("(?ms)" + COMMENT);
            s.useDelimiter("\\s*[\n,]\\s*"); // linebreaks and commas
            final Pattern p = Pattern.compile("\\s*"
                    + "(\\w+)" // place name (group 1)
                    + "\\s*"
                    + "([<>]?=|[<>])" // operator (group 2)
                    + "\\s*"
                    + "(\\d+)" // natural number (group 3)
                    + "\\s*"
                    + "[,\n]?"); // delimiter
            final Matcher m = p.matcher("");
            final Collection<String> initialPost = new HashSet<String>();
            final Collection<String> parameterizers = new ArrayList<String>();
            String param_hi = null;
            while(!s.hasNext("target") && s.hasNext()) {
                final String cur = s.next();
                m.reset(cur);
                if(m.find()) {
                    final String place = m.group(1);
                    if(PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) {
                        initialPost.addAll(post.get(place));
                    }
                    final String op = m.group(2);
                    final int marking = Integer.parseInt(m.group(3));
                    final String t;
                    if(SINGLE_TRANSITION_FOR_PARAMETERS && op.equals(">=")) {
                        t = param_hi == null ? "tparam" : param_hi;
                    } else {
                        t = "tparam_" + place;
                    }
                    if(op.equals("=")) {
                        // perfect
                        writer.writePlace(new petruchio.pn.Place(place, marking));
                    } else if(op.equals("<=")) {
                        // option to remove tokens
                        parameterizers.add(t);
                        writer.writePlace(new petruchio.pn.Place(place, marking));
                        writer.writeTransition(new petruchio.pn.Transition(t));
                        writer.writePTArc(place, t, 1);
                    } else if(op.equals(">=")) {
                        // option to add tokens
                        if(PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS && param_hi == null) {
                            parameterizers.add(t);
                        }
                        writer.writePlace(new petruchio.pn.Place(place, marking));
                        if(param_hi == null) {
                            writer.writeTransition(new petruchio.pn.Transition(t));
                            if(SINGLE_TRANSITION_FOR_PARAMETERS) {
                                param_hi = t;
                            }
                        }
                        writer.writeTPArc(t, place, 1);
                    } else if(op.equals("<")) {
                        // option to remove tokens
                        if(PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS) {
                            parameterizers.add(t);
                        }
                        writer.writePlace(new petruchio.pn.Place(place, marking-1));
                        writer.writeTransition(new petruchio.pn.Transition(t));
                        writer.writePTArc(place, t, 1);
                    } else if(op.equals(">")) {
                        // option to add tokens
                        if(PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS && param_hi == null) {
                            parameterizers.add(t);
                        }
                        writer.writePlace(new petruchio.pn.Place(place, marking+1));
                        if(param_hi == null) {
                            writer.writeTransition(new petruchio.pn.Transition(t));
                            if(SINGLE_TRANSITION_FOR_PARAMETERS) {
                                param_hi = t;
                            }
                        }
                        writer.writeTPArc(t, place, 1);
                    } else {
                        throw new InternalError("unexpected op at initial constraint: " + op);
                    }
                }
            }
            if(PARAMETERS_DONT_INTRODUCE_LIVE_TRANSITIONS && !parameterizers.isEmpty()) {
                int prm = 0;
                String init = "init";
                while(placeNames.contains(init)) {
                    init = "init" + (prm++);
                }
                prm = 0;
                String run = "run";
                while(placeNames.contains(run)) {
                    run = "run" + (prm++);
                }
                final Place initing = new petruchio.pn.Place(init, 1);
                writer.writePlace(initing);
                final Place running = new petruchio.pn.Place(run, 1);
                writer.writePlace(running);
                final Transition start = new petruchio.pn.Transition("tstart");
                writer.writeTransition(start);
                writer.writePTArc(init, start.getName(), 1);
                writer.writeTPArc(start.getName(), run, 0);
                for(final String t : parameterizers) {
                    writer.writePTArc(init, t, 1);
                    writer.writeTPArc(t, init, 1);
                }
                for(final String t : initialPost) {
                    writer.writePTArc(run, t, 1);
                    writer.writeTPArc(t, run, 1);
                }
            }
            writer.endNet();
            
        } catch(final RuntimeException e) {
            this.reportError(e);
            throw e;
            
        } finally {
            try {
                this.in.close();
            } catch(final IOException e) {
                // ignore
            }
            this.in = null;
        }
    }
    
    
    public Collection<Map<Place, Integer>> loadTargetMarkings(final String filename, final PetriNet net) {
        final Collection<Map<Place, Integer>> targets = new ArrayList<Map<Place, Integer>>();
        final Scanner s;
        try {
            s = new Scanner(new FileInputStream(filename)).useDelimiter("\\s+");
        } catch(final FileNotFoundException e) {
            return targets;
        }
        
        s.skip("(?ms).*target" + COMMENT + "\\s*");
        s.useDelimiter("(?ms)^\\s*invariants\\s*" + COMMENT);
        final Pattern p = Pattern.compile("(?ms)\\s*" + COMMENT + "\\s*"
                + "(\\w+)" // place name (group 1)
                + COMMENT + "\\s*"
                + "\\s*"
                + COMMENT + "\\s*"
                + "([<>]?=|[<>])" // operator (group 2)
                + COMMENT + "\\s*"
                + "(\\d+)" // natural number (group 3)
                + COMMENT + "\\s*"
                + "([,;\n]||\\Z)" // delimiter
                + COMMENT + "\\s*");
        final Matcher m = p.matcher(s.next());
        Map<Place, Integer> target = new IdentityHashMap<Place, Integer>();
        while(m.find()) {
            final String place = m.group(1);
            final String op = m.group(2);
            final int marking = Integer.parseInt(m.group(3));
            if(op.equals(">=")) {
                for(final Place pl : net.getPlaces(place)) {
                    target.put(pl, marking);
//                    System.out.println(pl + " >= " + marking);
                }
            } else {
                throw new IllegalArgumentException("unexpected op at target constraint: " + op);
            }
            final String sep = m.group(4);
            if(sep.equals(";") || sep.equals("\n") || sep.isEmpty()) {
                if(!target.isEmpty()) {
                    targets.add(target);
                    target = new IdentityHashMap<Place, Integer>();
                }
//                System.out.println("---");
            }
        }
        if(!target.isEmpty()) {
            targets.add(target);
        }
        
        return targets;
    }
    
    public Collection<Map<Place, Integer>> loadInvariants(final String filename, final PetriNet net) {
        final Collection<Map<Place, Integer>> invs = new ArrayList<Map<Place, Integer>>();
        final Scanner s;
        try {
            s = new Scanner(new FileInputStream(filename)).useDelimiter("\\s+");
        } catch(final FileNotFoundException e) {
            return invs;
        }
        
        s.skip("(?ms).*invariants" + COMMENT + "\\s*");
        s.useDelimiter("\\s*\n\\s*");
        final Pattern p = Pattern.compile("\\s*"
                + "(\\w+)" // place name (group 1)
                + "\\s*"
                + "([<>]?=|[<>])" // operator (group 2)
                + "\\s*"
                + "(\\d+)" // natural number (group 3)
                + "\\s*"
                + "[,\n]?"); // delimiter
        final Matcher m = p.matcher("");
        while(s.hasNext()) {
            final String init = s.next();
            m.reset(init);
            Map<Place, Integer> inv = new IdentityHashMap<Place, Integer>();
            int sum = 0;
            while(m.find()) {
                final String op = m.group(2);
                if(op.equals("=")) {
                    final int weight = Integer.parseInt(m.group(3));
                    final String place = m.group(1);
                    final Collection<Place> places = net.getPlaces(place);
                    if(places.isEmpty()) {
                        inv.clear();
                        break;
                    }
                    for(final Place pl : places) {
                        inv.put(pl, weight);
                        sum += weight*pl.getMarking();
                    }
                } else {
                    inv.clear();
                    break;
                }
            }
            if(!inv.isEmpty()) {
//                System.out.println(inv + " = " + sum);
                invs.add(inv);
            }
        }
        
        
        return invs;
    }

}