/*
 * Project: Petruchio
 * File:    AddExitPlaces.java
 * Author:  Tim Strazny
 * Created: 02.01.2008
 */
package petruchio.pn.reductors;

import java.util.Collection;
import java.util.LinkedList;

import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;


/**
 * TODO: Document
 * 
 * @author Tim Strazny
 */
public class AddExitPlaces implements PetriNetReductor {

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getDescription()
     */
    @Override
    public String getDescription() {
        return "To every transtition t with no output arcs, a place p with label STOP is added in the transition's post set.";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getName()
     */
    @Override
    public String getName() {
        return "add exit places";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public int reduce(PetriNet net) {
        int count = 0;
        
        final LinkedList<Transition> toTreat = new LinkedList<Transition>();
        
        for(final Transition t : net.getTransitions()) {
            if(t.getOutput().isEmpty()) {
                toTreat.add(t);
            }
        }
        
        while(!toTreat.isEmpty()) {
            final Transition t = toTreat.remove();
            final Place p = net.addPlace("STOP", 0);
            p.setX(t.getX()+10);
            p.setY(t.getY()+10);
            p.setMeaning("STOP");
            p.setFinal(true);
            net.addArc(t, p, 1);
            ++count;
        }
        
        return -count;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(PetriNet net, Collection<Place> ignorePlaces,
            Collection<Transition> ignoreTransitions) {
        return this.reduce(net);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // nothing to do
    }

}
