/*
 * Project:  Petruchio
 * Filename: LTLXFalsify.java
 * Created:  05.10.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.reductors;

import java.util.ArrayList;
import java.util.Collection;

import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public class LTLXFalsify implements PetriNetReductor {

    protected PetriNetReductor[] reductors = {
            new DependencyReduction(),
            new PreAgglomeration(),
            new AlternativesReduction(),
            new RedundantPlaceReduction(),
            new OmegaFReduction(),
            new OmegaFReductionBPP(),
            new SimpleDeadElementRemover()            
        };

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getDescription()
     */
    @Override
    public String getDescription() {
        final StringBuilder b = new StringBuilder("A set of reductions that keep LTL-X properties for falsification:");
        int i = 1;
        for(final PetriNetReductor r : this.reductors) {
            b.append("\n  ");
            b.append(i++);
            b.append(". ");
            b.append(r.getName());
            b.append("\n     ");
            b.append(r.getDescription());
        }
        return b.toString();
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getName()
     */
    @Override
    public String getName() {
        return "reductions for LTL-X falsification";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public int reduce(final PetriNet net) {
        return this.reduce(net, new ArrayList<Place>(), new ArrayList<Transition>());
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(final PetriNet net, final Collection<Place> ignorePlaces,
            final Collection<Transition> ignoreTransitions) {
        int removed = 0;
        boolean changed;
        int lastChange = -1;
        do {
            changed = false;
            for(int i = 0; i < this.reductors.length; ++i) {
                if(lastChange != i) {
                    final int reduced = this.reductors[i].reduce(net, ignorePlaces, ignoreTransitions);
                    if(reduced != 0) {
                        changed = true;
                        lastChange = i;
                        removed += reduced;
                    }
                }
            }
        } while(changed);
        return removed;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        for(final PetriNetReductor r : this.reductors) {
            r.reset();
        }
    }

}