/*
 * Project:  Petruchio
 * Filename: PreAgglomeration.java
 * Created:  09.08.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.reductors;

import java.util.ArrayList;
import java.util.Collection;

import petruchio.common.IdentityHashSet;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.petrinet.Transition;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class PreAgglomeration implements PetriNetReductor {
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PNReductor#getDescription()
     */
    @Override
    public String getDescription() {
        return "Pre-Agglomeration as proposed in J. Esparza and C. Schroeter. Net Reductions for LTL Model-Checking. 2001. (Modified for non-safe nets.)";
    }

    
    /* (non-Javadoc)
	 * @see petruchio.interfaces.Resettable#reset()
	 */
	@Override
    public void reset() {
		// nothing
	}
    

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PNReductor#getName()
     */
    @Override
    public String getName() {
        return "pre-agglomeration";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PNReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public int reduce(final PetriNet net) {
    	return this.reduce(net, new ArrayList<Place>(), new ArrayList<Transition>());
    }
    	
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(final PetriNet net, final Collection<Place> ignorePlaces,
    		final Collection<Transition> ignoreTransitions) {
    	final Collection<Place> ignoreP = new IdentityHashSet<Place>(ignorePlaces);
    	final Collection<Transition> ignoreT = new IdentityHashSet<Transition>(ignoreTransitions);
    	
    	final Collection<Place> places = new ArrayList<Place>(net.getPlaces());
        int removed = 0;
        
        next_place:
        for(final Place p : places) {
            if((p.getMarking() != 0) || (p.getInput().size() != 1)
                            || p.getOutput().isEmpty()
                            || ignoreP.contains(p)) {
                continue next_place;
            }
            final TPArc theInArc = p.getInput().iterator().next();
            final Transition t = theInArc.getSource();
            
            if((t.getOutput().size() != 1) || ignoreT.contains(t)) {
                continue next_place;
            }
            for(final PTArc arc : t.getInput()) {
                if(arc.getSource() == p) {
                    continue next_place;
                }
            }
            
            final int weight = theInArc.getWeight();

            for(final PTArc in : t.getInput()) {
                for(final PTArc out : p.getOutput()) {
                    if((out.getWeight() * in.getWeight()) % weight != 0) {
                        continue next_place;
                    }
                }
            }
            
            for(final PTArc in : t.getInput()) {
                for(final PTArc out : p.getOutput()) {
                    net.addArc(in.getSource(), out.getTarget(), (out.getWeight() * in.getWeight()) / weight);
                }
            }
            
            if(net.removePlace(p)) {
            	++removed;
            }
            if(net.removeTransition(t)) {
            	++removed;
            }
        }
        return removed;
    }
}
