/*
 * Project:  Petruchio
 * Filename: RedundantPlaceReduction.java
 * Created:  05.10.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.reductors;

import java.util.ArrayList;
import java.util.Collection;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import petruchio.common.IdentityHashSet;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.petrinet.Transition;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public class RedundantPlaceReduction implements PetriNetReductor {

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getDescription()
     */
    @Override
    public String getDescription() {
        // TODO Auto-generated method stub
        return null;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getName()
     */
    @Override
    public String getName() {
        return "redundant place reduction";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public int reduce(final PetriNet net) {
        return this.reduce(net, new ArrayList<Place>(), null);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(final PetriNet net, final Collection<Place> ignorePlaces,
            final Collection<Transition> ignoreTransitions) {
        final Collection<Place> ignoreP = new IdentityHashSet<Place>(ignorePlaces);
        final Collection<Place> visited = new IdentityHashSet<Place>();
        final LinkedList<Place> candidates = new LinkedList<Place>();
        final LinkedList<Place> rm = new LinkedList<Place>();
        final Map<Transition, Integer> in = new IdentityHashMap<Transition, Integer>();
        final Map<Transition, Integer> out = new IdentityHashMap<Transition, Integer>();

        int removed = 0;
        for(final Transition t : net.getTransitions()) {
            for(final TPArc arc : t.getOutput()) {
                candidates.add(arc.getTarget());
            }
            this.processCandidates(candidates, in, out, visited, rm, ignoreP);
            
            for(final PTArc arc : t.getInput()) {
                candidates.add(arc.getSource());
            }
            this.processCandidates(candidates, in, out, visited, rm, ignoreP);
            
            while(!rm.isEmpty()) {
                if(net.removePlace(rm.remove())) {
                    ++removed;
                }
            }
        }
        return removed;
    }
    
    
    /**
     * TODO: description
     *
     * @param candidates
     * @param in
     * @param out
     * @param visited
     * @param rm
     */
    private void processCandidates(final LinkedList<Place> candidates,
            final Map<Transition, Integer> in, final Map<Transition, Integer> out,
            final Collection<Place> visited, final Collection<Place> rm,
            final Collection<Place> ignoreP) {
        while(candidates.size() > 1) {
            Place p = candidates.remove();
            if(!visited.add(p)) {
                continue;
            }
            for(final PTArc arc : p.getOutput()) {
                out.put(arc.getTarget(), arc.getWeight());
            }
            for(final TPArc arc : p.getInput()) {
                in.put(arc.getSource(), arc.getWeight());
            }
            boolean rename = !ignoreP.contains(p);
            find_match:
            for(final Iterator<Place> it = candidates.iterator(); it.hasNext(); ) {
                Place p2 = it.next();
                if(p2.getInput().size() != in.size() || p2.getOutput().size() != out.size()) {
                    continue find_match;
                }
                for(final PTArc arc : p2.getOutput()) {
                    final Integer w = out.get(arc.getTarget());
                    if(w == null || w != arc.getWeight()) {
                        continue find_match;
                    }
                }
                for(final TPArc arc : p2.getInput()) {
                    final Integer w = in.get(arc.getSource());
                    if(w == null || w != arc.getWeight()) {
                        continue find_match;
                    }
                }
                if(ignoreP.contains(p2)) {
                    if(rename) {
                        final Place buf = p;
                        p = p2;
                        p2 = buf;
                        rename = false;
                    } else {
                        continue;
                    }
                }
                
                if(p2.getMarking() < p.getMarking()) {
                    p.setMarking(p2.getMarking());
                }
                
//                System.out.println(p + " == " + p2 + "\n" + p.getInput() + " == "
//                        + p2.getInput() + "\n" + p.getOutput() + " == " + p2.getOutput());
                
                rm.add(p2);
                it.remove();
                if(rename) {
                    p.setName(p.getName() + " == " + p2.getName());
                    p.setMeaning(p.getMeaning() + " == " + p2.getMeaning());
                }
            }
            in.clear();
            out.clear();
        }
        candidates.clear();
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // no caches
    }

}
