/*
 * Project: Petruchio
 * File:    SafeNetReductor.java
 * Author:  Tim Strazny
 * Created: 19.12.2007
 */
package petruchio.pn.reductors;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;

import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;


/**
 * TODO: Document
 * 
 * @author Tim Strazny
 */
public class SafeNetReductor implements PetriNetReductor {

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getDescription()
     */
    @Override
    public String getDescription() {
        return "Removes transitions that have an input arc with weight larger "
            + "than 1 from the net. Assuming the net is safe!";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#getName()
     */
    @Override
    public String getName() {
        return "safe net dead transition reduction";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public int reduce(final PetriNet net) {
        return this.reduce(net, new ArrayList<Place>(0), new ArrayList<Transition>(0));
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(PetriNet net, Collection<Place> ignorePlaces,
            Collection<Transition> ignoreTransitions) {
        final LinkedList<Transition> rm = new LinkedList<Transition>();
        for(final Transition t : net.getTransitions()) {
            if(!ignoreTransitions.contains(t)) {
                for(final PTArc in : t.getInput()) {
                    if(in.getWeight() > 1) {
                        rm.add(t);
                        break;
                    }
                }
            }
        }
        int count = 0;
        while(!rm.isEmpty()) {
            if(net.removeTransition(rm.remove())) {
                count++;
            }
        }
        return count;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // no caches to clean
    }

}
