/*
 * Project:  Petruchio
 * Filename: SelfLoopReduction.java
 * Created:  09.08.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.reductors;

import java.util.ArrayList;
import java.util.Collection;
import java.util.IdentityHashMap;
import java.util.Map;

import petruchio.common.IdentityHashSet;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class SelfLoopReduction implements PetriNetReductor {
    
    private final Map<Place, Integer> in = new IdentityHashMap<Place, Integer>();
    private final Map<Place, Integer> out = new IdentityHashMap<Place, Integer>();

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PNReductor#getDescription()
     */
    @Override
    public String getDescription() {
        return "A transition is removed iff it has the same output and input arcs";
    }
    
    
    /* (non-Javadoc)
	 * @see petruchio.interfaces.Resettable#reset()
	 */
	@Override
    public void reset() {
		this.in.clear();
		this.out.clear();
	}


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PNReductor#getName()
     */
    @Override
    public String getName() {
        return "self-loop reduction";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PNReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
     */
    @Override
    public synchronized int reduce(final PetriNet net) {
    	return this.reduce(net, null, new ArrayList<Transition>());
    }
    	
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(final PetriNet net, final Collection<Place> ignorePlaces,
    		final Collection<Transition> ignoreTransitions) {
    	final Collection<Transition> ignoreT = new IdentityHashSet<Transition>(ignoreTransitions);
    	
    	final Collection<Transition> trans = new ArrayList<Transition>(net.getTransitions());
        int removed = 0;
        
        for(final Transition t : trans) {
        	if(ignoreT.contains(t)) {
        		continue;
        	}
        	
            this.in.clear();
            for(final PTArc arc : t.getInput()) {
                this.in.put(arc.getSource(), arc.getWeight());
            }
            this.out.clear();
            for(final TPArc arc : t.getOutput()) {
                this.out.put(arc.getTarget(), arc.getWeight());
            }
            if(this.in.equals(this.out)) {
                if(net.removeTransition(t)) {
                	++removed;
                }
            }
            this.in.clear();
            this.out.clear();
        }
        return removed;
    }

}