/*
 * Project:  Petruchio
 * Filename: SimpleElementPartRemover.java
 * Created:  17.09.2007
 * Author:   Tim Strazny
 */

package petruchio.pn.reductors;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import petruchio.common.IdentityHashSet;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetReductor;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;
import petruchio.interfaces.petrinet.TPArc;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public class SimpleDeadElementRemover implements PetriNetReductor {

	/* (non-Javadoc)
	 * @see petruchio.interfaces.petrinet.PetriNetReductor#getDescription()
	 */
	@Override
    public String getDescription() {
		return "Every unmarked place with no input arcs (or test arcs) is "
			+ "removed as well as the transitions in its post set.";
	}
	
	
	/* (non-Javadoc)
	 * @see petruchio.interfaces.Resettable#reset()
	 */
	@Override
    public void reset() {
		// nothing
	}
	

	/* (non-Javadoc)
	 * @see petruchio.interfaces.petrinet.PetriNetReductor#getName()
	 */
	@Override
    public String getName() {
		return "simple dead element remover";
	}

	
	/* (non-Javadoc)
	 * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet)
	 */
	@Override
    public int reduce(final PetriNet net) {
		return this.reduce(net, new ArrayList<Place>(), new ArrayList<Transition>());
    }
    	
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetReductor#reduce(petruchio.interfaces.petrinet.PetriNet, java.util.Collection, java.util.Collection)
     */
    @Override
    public int reduce(final PetriNet net, final Collection<Place> ignorePlaces,
    		final Collection<Transition> ignoreTransitions) {
    	final Collection<Place> ignoreP = new IdentityHashSet<Place>(ignorePlaces);
    	final Collection<Transition> ignoreT = new IdentityHashSet<Transition>(ignoreTransitions);
    	
    	final Collection<Transition> out = new IdentityHashSet<Transition>();
    	
		int removed = 0;
		final LinkedList<Transition> rm = new LinkedList<Transition>();
		final LinkedList<Place> todo = new LinkedList<Place>();
		todo.addAll(net.getPlaces());
		
		while(!todo.isEmpty()) {
			final Place p = todo.remove();
			
			final int m = p.getMarking();
			boolean remove = true;
			if(!p.getInput().isEmpty() || (m != 0)) {
				for(final PTArc arc : p.getOutput()) {
					if(m < arc.getWeight()) {
						out.add(arc.getTarget());
					}
				}
				if(out.size() < p.getOutput().size()) {
					remove = false;
				} else {
					for(final TPArc arc : p.getInput()) {
						if(!out.contains(arc.getSource())) {
							// *p is no subset of p*. thus the place may be markable
							remove = false;
							break;
						}
					}
				}
				out.clear();
			}
			
			if(remove) {
				for(final PTArc arc : p.getOutput()) {
					final Transition t = arc.getTarget();
					if(ignoreT.contains(t)) {
						remove = false;
						continue;
					}
					for(final TPArc arc2 : t.getOutput()) {
						todo.add(arc2.getTarget());
					}
					rm.add(t);
				}
				while(!rm.isEmpty()) {
				    if(net.removeTransition(rm.remove())) {
                        ++removed;
				    }
				}
				if(remove && !ignoreP.contains(p) && net.removePlace(p)) {
					++removed;
				}
			}
		}
		
		return removed;
	}
	
	
	/**
	 * TODO: description
	 *
	 * @param p
	 * @return
	 */
	public static int reduceFromPlace(final PetriNet net, final Place place) {
		if(!place.getInput().isEmpty() || (place.getMarking() != 0)) {
			return 0;
		}
		int removed = 0;
		final Collection<Place> todo = new IdentityHashSet<Place>();
		todo.add(place);
		
		while(!todo.isEmpty()) {
			final Iterator<Place> it = todo.iterator();
			final Place p = it.next();
			it.remove();
			
			if(p.getInput().isEmpty() && (p.getMarking() == 0)) {
				removed += 1 + p.getOutput().size();
				for(final PTArc arc : p.getOutput()) {
					final Transition t = arc.getTarget();
					for(final TPArc arc2 : t.getOutput()) {
						todo.add(arc2.getTarget());
					}
					net.removeTransition(t);
				}
				net.removePlace(p);
			}
		}
		
		return removed;
	}

}