/*
 * Project:  Petruchio
 * Filename: CMWriter.java
 * Created:  04.10.2007
 * Author:   Tim Strazny
 */
package petruchio.pn.writers;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetWriter;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.petrinet.Transition;

/**
 * TODO: description 
 *
 * @author Tim Strazny
 */
public class ARMCWriter implements SelfDescribing, PetriNetWriter {

    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "A Petri net writer for files in the armc input format, "
            + "see http://www.mpi-sws.mpg.de/~rybal/armc/";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#getDefaultExtension()
     */
    @Override
    public String getDefaultExtension() {
        return "armc";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#write(petruchio.interfaces.petrinet.PetriNet, java.io.OutputStream)
     */
    @Override
    public void write(final PetriNet net, final OutputStream output) {
        final PrintWriter out = new PrintWriter(output);
        
        if(net.hasHeaderComment()) {
            final String[] lines = net.getHeaderComment().split("\r\n|[\n\r\u2028\u2029\u0085]");
            for(final String line : lines) {
                out.print("% ");
                out.print(line);
                out.print("\n");
            }
        }
        out.print("% ");
        out.print(net.getPlaces().size());
        out.print(" places, ");
        out.print(net.getTransitions().size());
        out.print(" transitions, ");
        out.print(net.getPTArcCount());
        out.print(" pt-arcs, ");
        out.print(net.getTPArcCount());
        out.print(" tp-arcs\n\n");
        
        out.print(":- multifile r/5, var2names/2, preds/2, cube_size/1, start/1, "
                + "refinement/1, error/1.\nrefinement(inter).\ncube_size(1).\n"
                + "%implicit_updates.\n\n"
                + "start(pc(start)).\nerror(pc(error)).\n\n");
        
        final Map<Transition, String> tr = new IdentityHashMap<Transition, String>();
        final String prefix;
        final List<Place> plOrder = new ArrayList<Place>(net.getPlaces());
        final Map<Place, String> pl = new IdentityHashMap<Place, String>();
        final Map<Place, String> pla = new IdentityHashMap<Place, String>();
        {
            int i = 0;
            final Collection<String> trNames = new HashSet<String>();
            for(final Transition t : net.getTransitions()) {
                final StringBuffer nameHint = new StringBuffer();
                final char ch0 = t.getName().charAt(0);
                int j = 0;
                if(ch0 == '_' || !Character.isJavaIdentifierPart(ch0)) {
                    nameHint.append('t');
                } else {
                    nameHint.append(Character.toLowerCase(ch0));
                    j = 1;
                }
                for(; j < t.getName().length(); ++j) {
                    final char ch = t.getName().charAt(j);
                    if(!Character.isJavaIdentifierPart(ch)) {
                        nameHint.append('_');
                    } else {
                        nameHint.append(ch);
                    }
                }
                String name = nameHint.toString();
                while(!trNames.add(name)) {
                    name = nameHint.toString() + i++;
                }
                tr.put(t, name);
            }
            
            
            final StringBuffer before = new StringBuffer();
            final StringBuffer after = new StringBuffer();
            final StringBuffer def = new StringBuffer();
            final StringBuffer init = new StringBuffer();
            final Collection<String> plNames = new HashSet<String>();
            final Collection<String> plaNames = new HashSet<String>();
            i = 0;
            int j = 0;
            for(final Iterator<Place> it = net.getPlaces().iterator(); it.hasNext(); ) {
                final Place p = it.next();
                final StringBuffer nameHint = new StringBuffer();
                final char ch0 = p.getName().charAt(0);
                int k = 0;
                if(!Character.isJavaIdentifierStart(ch0)) {
                    nameHint.append('P');
                } else {
                    nameHint.append(Character.toUpperCase(ch0));
                    k = 1;
                }
                for(; k < p.getName().length(); ++k) {
                    final char ch = p.getName().charAt(k);
                    if(!Character.isJavaIdentifierPart(ch)) {
                        nameHint.append('_');
                    } else {
                        nameHint.append(ch);
                    }
                }
                String name = nameHint.toString();
                while(!plNames.add(name)) {
                    name = nameHint.toString() + i++;
                }
                pl.put(p, name);
                
                String namea = name + "p";
                while(!plaNames.add(namea) || plNames.contains(namea)) {
                    namea = name + "p" + j++;
                }
                pla.put(p, namea);
                
                if(before.length() != 0) {
                    before.append(", ");
                    after.append(", ");
                    init.append(", ");
                }
                
                before.append(name);
                after.append(namea);
                
                def.append("   (");
                def.append(name);
                def.append(", '");
                def.append(p.getName());
                def.append("')");
                if(it.hasNext()) {
                    def.append(",");
                }
                def.append(" %");
                if(p.getBound() == Place.BOUND_UNBOUNDED) {
                    def.append(" unbounded");
                } else if(p.getBound() != Place.BOUND_UNKNOWN) {
                    def.append(" ");
                    def.append(p.getBound());
                    def.append("-bounded");
                }
                if(p.getNote() != null && p.getNote().length() != 0) {
                    def.append(" note: ");
                    def.append(p.getNote());
                }
                if(p.getMeaning() != null && p.getMeaning().length() != 0) {
                    def.append(" meaning: ");
                    def.append(p.getMeaning());
                }
                def.append("\n");
                
                init.append(namea);
                init.append("=");
                init.append(p.getMarking());
            }
            
            out.print("var2names(p(_, data(");
            out.print(before);
            out.print(")),\n  [\n");
            out.print(def);
            out.print("  ]).\n\n% initial predicates: all values are non-negative (the compiler might have identified some upper bounds, too)\npreds(p(loop, data(");
            out.print(before);
            out.print(")),\n  [");
            for(final Iterator<Place> it = plOrder.iterator(); it.hasNext();) {
                final Place p = it.next();
                final String n = pl.get(p);
                out.print(n);
                out.print(">=0");
                if(p.boundKnown()) {
                    out.print(", ");
                    out.print(n);
                    out.print("=<");
                    out.print(p.getBound());
                }
                if(it.hasNext()) {
                    out.print(", ");
                }
            }
            out.print("]).\n\n% create initial marking\nr(p(pc(start), data(");
            for(int k = net.getPlaces().size(); k > 0; --k) {
                out.print("_");
                if(k > 1) {
                    out.print(", ");
                }
            }
            out.print(")),\n  p(pc(loop), data(");
            out.print(after);
            out.print(")),\n  [");
            out.print(init);
            out.print("],\n  [],\n  init).\n\n");
            
            prefix = "r(p(pc(loop), data(" + before + ")),\n  p(pc(loop), data(";
        }
        
        out.print("% PROPERTY DUMMY. YOUR ERROR PROPERTY GOES HERE!\n%r(p(pc(loop), data(");
        for(final Iterator<Place> it = plOrder.iterator(); it.hasNext();) {
            final Place p = it.next();
            out.print(pl.get(p));                    
            if(it.hasNext()) {
                out.print(", ");
            }
        }
        out.print(")),\n%  p(pc(error), data(");
        for(final Iterator<Place> it = plOrder.iterator(); it.hasNext();) {
            final Place p = it.next();
            out.print(pl.get(p));                    
            if(it.hasNext()) {
                out.print(", ");
            }
        }
        out.print(")),\n%  [], % The error property goes here! \n%  [],\n%  ");
        out.print("error");
        out.print(").\n\n");
        
        final Map<Place, Integer> change = new IdentityHashMap<Place, Integer>();
        for(final Transition t : net.getTransitions()) {
            out.print("% ");
            out.print(t.getName());
            out.print("\n");
            out.print(prefix);
            final StringBuffer b = new StringBuffer();
            for(final Iterator<PTArc> it = t.getInput().iterator(); it.hasNext(); ) {
                final PTArc arc = it.next();
                b.append(pl.get(arc.getSource()));
                b.append(">=");
                b.append(arc.getWeight());
                if(it.hasNext()) {
                    b.append(", ");
                }
                
                final Integer old = change.put(arc.getSource(), -arc.getWeight());
                if(old != null) {
                    change.put(arc.getSource(), old-arc.getWeight());
                }
            }
            for(final TPArc arc : t.getOutput()) {
                final Integer old = change.put(arc.getTarget(), arc.getWeight());
                if(old != null) {
                    change.put(arc.getTarget(), old+arc.getWeight());
                }
            }
            for(final Iterator<Place> it = plOrder.iterator(); it.hasNext();) {
                final Place p = it.next();
                final Integer ch = change.get(p);
                if(ch != null && ch != 0) {
                    out.print(pla.get(p));
                } else {
                    out.print(pl.get(p));                    
                }
                if(it.hasNext()) {
                    out.print(", ");
                }
            }
            out.print(")),\n  [");
            out.print(b);
            out.print("],\n  [");
            
            boolean first = true;
            for(final Iterator<Map.Entry<Place, Integer>> it = change.entrySet().iterator(); it.hasNext(); ) {
                final Map.Entry<Place, Integer> e = it.next();
                final Place p = e.getKey();
                final Integer c = e.getValue();
                if((c == null) || (c == 0)) {
                    continue;
                }
                if(!first) {
                    out.print(", ");
                } else {
                    first = false;
                }
                out.print(pla.get(p));
                out.print("=");
                out.print(pl.get(p));
                if(c > 0) {
                    out.print('+');
                } 
                out.print(c);
            }
            out.print("],\n  ");
            out.print(tr.get(t));
            out.print(").\n\n");
            change.clear();
        }
        out.close();
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // no caches
    }

}