/*
 * Project:  Petruchio
 * Filename: BPPWriter.java
 * Created:  05.03.2008
 * Author:   Tim Strazny
 */
package petruchio.pn.writers;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Iterator;

import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetWriter;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.TPArc;
import petruchio.interfaces.petrinet.Transition;

/**
 * @author Tim Strazny
 */
public class BPPWriter implements PetriNetWriter {

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#getDefaultExtension()
     */
    @Override
    public String getDefaultExtension() {
        return "bpp";
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#write(petruchio.interfaces.petrinet.PetriNet, java.io.OutputStream)
     */
    @Override
    public void write(final PetriNet net, final OutputStream output) {
        final PrintWriter out = new PrintWriter(output);
        
        if(net.hasHeaderComment()) {
            out.println("/*");
            final String[] lines = net.getHeaderComment().split("\r\n|[\n\r\u2028\u2029\u0085]");
            for(final String line : lines) {
                out.print(" * ");
                out.print(line);
                out.print("\n");
            }
            out.println(" */");
        }
        
        for(final Place p : net.getPlaces()) {
            out.print(p.getName());
            out.print(" := \"");
            out.print(p.getMeaning());
            out.println("\";");
        }
        
        main:
        for(final Transition t: net.getTransitions()) {
            if(t.getInput().size() != 1) {
                out.println("// THE NET CANNOT BE TRANSLATED TO A BASIC PARALLEL PROCESS!");
                break main;
            }
            for(final PTArc a : t.getInput()) {
                out.print(a.getSource().getName());
                if(a.getWeight() != 1) {
                    out.println(";");
                    out.println("// THE NET CANNOT BE TRANSLATED TO A BASIC PARALLEL PROCESS!");
                    break main;
                }
            }
            out.print(" -> ");
            for(final Iterator<TPArc> it = t.getOutput().iterator(); it.hasNext();) {
                final TPArc a = it.next();
                for(int i = a.getWeight(); i > 0; --i) {
                    out.print(a.getTarget().getName());
                    if(i > 1) {
                        out.print(" | ");
                    }
                }
                if(it.hasNext()) {
                    out.print(" | ");
                }
            }
            out.println(";");
        }
        out.close();
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // nothing
    }

}
