/**
 * 
 */
package petruchio.pn.writers;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.LinkedList;

import petruchio.cov.Coverability;

import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetCovWriter;
import petruchio.interfaces.petrinet.Place;

/**
 * @author Tim_2
 *
 */
public class CovDOTWriter implements PetriNetCovWriter, SelfDescribing {

    private Coverability cov = null;
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#getDefaultExtension()
     */
    @Override
    public String getDefaultExtension() {
        return "cov.dot";
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#write(petruchio.interfaces.petrinet.PetriNet, java.io.OutputStream)
     */
    @Override
    public void write(final PetriNet net, final OutputStream output) {
        if(net.hasHeaderComment()) {
            try {
                output.write("/*\n".getBytes());
                final String[] lines = net.getHeaderComment().split("\r\n|[\n\r\u2028\u2029\u0085]");
                for(final String line : lines) {
                    output.write(" * ".getBytes());
                    output.write(line.getBytes());
                    output.write("\n".getBytes());
                }
                output.write(" */\n".getBytes());
            } catch(final IOException e) {
                // ignore
            }
        }
        
        if(this.cov == null) {
            this.cov = new petruchio.cov.Coverability();
            final Collection<Place> init = new LinkedList<Place>();
            for(final Place p : net.getPlaces()) {
                for(int i = p.getMarking(); i > 0; --i) {
                    init.add(p);
                }
            }
            this.cov.setCheckMarkable(false);
            this.cov.compute(init, net.getTransitions());
        }
        this.cov.printTo(output);
        this.cov = null;
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetCovWriter#setCoverability(petruchio.interfaces.algorithms.Coverability)
     */
    @Override
    public void setCoverability(final petruchio.interfaces.algorithms.Coverability cov) {
        this.cov = cov instanceof Coverability ? (Coverability)cov : null;
    }

    
    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        this.cov = null;
    }

    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "Calculates and prints the coverability graph to the given Petri net in Graphviz's DOT language.";
    }

}
