/*
 * Project: Petruchio
 * File:    LoLAWriter.java
 * Author:  Tim Strazny
 * Created: 12.10.2007
 */
package petruchio.pn.writers;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.petrinet.PTArc;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetWriter;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;
import petruchio.interfaces.petrinet.TPArc;


/**
 * TODO: Document
 * 
 * @author Tim Strazny
 */
public class LoLAWriter implements SelfDescribing, PetriNetWriter {
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "A Petri net writer which outputs in the format accepted by the LoLA tool."; 
    }
    

    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#getDefaultExtension()
     */
    @Override
    public String getDefaultExtension() {
        return "lola";
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#write(petruchio.interfaces.petrinet.PetriNet, java.io.OutputStream)
     */
    @Override
    public void write(final PetriNet net, final OutputStream output) {
        final PrintWriter out = new PrintWriter(output);
        
        if(net.hasHeaderComment()) {
            final String[] lines = net.getHeaderComment().split("\r\n|[\n\r\u2028\u2029\u0085]");
            for(final String line : lines) {
                out.print("{ ");
                out.print(line.replaceAll("\\{", "((").replaceAll("\\}", "))"));
                out.print(" }\n");
            }
        }
        out.print("{ ");
        out.print(net.getPlaces().size());
        out.print(" places, ");
        out.print(net.getTransitions().size());
        out.print(" transitions, ");
        out.print(net.getPTArcCount());
        out.print(" pt-arcs, ");
        out.print(net.getTPArcCount());
        out.print(" tp-arcs }\n");
        out.print("{ Meanings of places: }\n");
        for(final Place p : net.getPlaces()) {
            if(p.getMeaning() == null || p.getMeaning().length() == 0) {
                continue;
            }
            out.print("{ ");
            out.print(p.getName());
            out.print(" := ");
            out.print(p.getMeaning());
            out.print(" }\n");
        }
        out.print("{ Meanings of transitions: }\n");
        for(final Transition t : net.getTransitions()) {
            if(t.getMeaning() == null || t.getMeaning().length() == 0) {
                continue;
            }
            out.print("{ ");
            out.print(t.getName());
            out.print(" := ");
            out.print(t.getMeaning());
            out.print(" }\n");
        }
        out.print("\n");
        
        final Collection<Place> marked = new ArrayList<Place>();
        
        out.print("PLACE ");
        for(final Iterator<Place> it = net.getPlaces().iterator(); it.hasNext(); ) {
            final Place p = it.next();
            if(p.getMarking() != 0) {
                marked.add(p);
            }
            
            out.print(p.getName());
            if(it.hasNext()) {
                out.print(", ");
            }
        }
        
        out.print(";\n\nMARKING ");
        for(final Iterator<Place> it = marked.iterator(); it.hasNext(); ) {
            final Place p = it.next();
            out.print(p.getName());
            out.print(": ");
            out.print(p.getMarking());
            if(it.hasNext()) {
                out.print(", ");
            }
        }
        out.print(";\n");
        marked.clear();
        
        for(final Transition t : net.getTransitions()) {
            out.print("\nTRANSITION ");
            out.print(t.getName());
            out.print("\n");
            if(!t.getInput().isEmpty()) {
                out.print("CONSUME ");
                for(final Iterator<PTArc> it = t.getInput().iterator(); it.hasNext(); ) {
                    final PTArc arc = it.next();
                    out.print(arc.getSource().getName());
                    out.print(": ");
                    out.print(arc.getWeight());
                    if(it.hasNext()) {
                        out.print(", ");
                    }
                }
                out.print(";\n");
            }
            if(!t.getOutput().isEmpty()) {
                out.print("PRODUCE ");
                for(final Iterator<TPArc> it = t.getOutput().iterator(); it.hasNext(); ) {
                    final TPArc arc = it.next();
                    out.print(arc.getTarget().getName());
                    out.print(": ");
                    out.print(arc.getWeight());
                    if(it.hasNext()) {
                        out.print(", ");
                    }
                }
                out.print(";\n");
            }
        }
        
        
        out.close();
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        // nothing to do
    }

}