/*
 * Project: Petruchio
 * File:    ReachFragWriter.java
 * Author:  Tim Strazny
 * Created: 25.02.2008
 */
package petruchio.pn.writers;

import java.io.IOException;
import java.io.OutputStream;

import petruchio.interfaces.Retriever;
import petruchio.interfaces.SelfDescribing;
import petruchio.interfaces.petrinet.PetriNet;
import petruchio.interfaces.petrinet.PetriNetWriter;
import petruchio.interfaces.petrinet.PetriNetStreamWriter;
import petruchio.interfaces.petrinet.Place;
import petruchio.interfaces.petrinet.Transition;


/**
 * 
 * @author Tim Strazny
 */
public class ReachFragWriter implements SelfDescribing, PetriNetWriter, PetriNetStreamWriter {

    private OutputStream theout;
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.SelfDescribing#getDescription()
     */
    @Override
    public String getDescription() {
        return "A writer that outputs the all the place's meanings.";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.pn.PetriNetWriter#getDefaultExtension()
     */
    @Override
    public String getDefaultExtension() {
        return "frags";
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetWriter#write(petruchio.interfaces.petrinet.PetriNet, java.io.OutputStream)
     */
    @Override
    public synchronized void write(final PetriNet net, final OutputStream out) {
        try {
            if(net.hasHeaderComment()) {
                this.writeHeaderComment(net.getHeaderComment(), out);
            }
            for(final Place p : net.getPlaces()) {
                this.writePlace(p, out);
            }
        } finally {
            try {
                out.close();
            } catch(final IOException e) {
                // ignore
            }
        }
    }
    
    
    /**
     * TODO: document
     *
     * @param theout
     */
    private void writeHeaderComment(final String comment, final OutputStream out) {
        final String[] lines = comment.split("\r\n|[\n\r\u2028\u2029\u0085]");
        try {
            for(final String line : lines) {
                out.write("% ".getBytes());
                out.write(line.getBytes());
                out.write('\n');
            }
        } catch(final IOException e) {
            // ignore
        }
    }
    
    
    /* (non-Javadoc)
     * @see petruchio.interfaces.Resettable#reset()
     */
    @Override
    public void reset() {
        if(this.theout != null) {
            try {
                this.theout.close();
            } catch(final IOException e) {
                // ignore
            }
            this.theout = null;
        }
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#beginNet(java.lang.String)
     */
    @Override
    public void beginNet(final String name) {
        // nothing to do
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#endNet()
     */
    @Override
    public void endNet() {
        try {
            this.theout.close();
        } catch (final IOException e) {
            // ignore
        }
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#setOutput(java.io.OutputStream)
     */
    @Override
    public void setOutput(final OutputStream output) {
        this.theout = output;
    }
    
    
    public void writePlace(final Place p, final OutputStream out) {
        try {
            out.write(p.getName().getBytes());
            out.write(" := ".getBytes());
            final int start = p.getMeaning().indexOf(Retriever.MEANING_SEP);
            out.write(p.getMeaning().substring(start < 0 ? 0 : start + Retriever.MEANING_SEP.length()).getBytes());
            out.write("\n".getBytes());
        } catch(final IOException e) {
            throw new RuntimeException(e);
        }
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#writeHeaderComment(java.lang.String)
     */
    @Override
    public void writeHeaderComment(final String comment) {
        this.writeHeaderComment(comment, this.theout);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#writePTArc(java.lang.String, java.lang.String, int)
     */
    @Override
    public void writePTArc(final String name, final String name2, final int weight) {
        // no output;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#writePlace(petruchio.interfaces.petrinet.Place)
     */
    @Override
    public void writePlace(final Place p) {
        this.writePlace(p, this.theout);
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#writeTPArc(java.lang.String, java.lang.String, int)
     */
    @Override
    public void writeTPArc(final String name, final String name2, final int weight) {
        // no output;
    }


    /* (non-Javadoc)
     * @see petruchio.interfaces.petrinet.PetriNetStreamWriter#writeTransition(petruchio.interfaces.petrinet.Transition)
     */
    @Override
    public void writeTransition(final Transition t) {
        // no output;
    }

}