/*
 * Project:  Petruchio
 * Filename: Pstl.flex
 * Created:  29.05.2011
 * Author:   Sören Jeserich
 * Remarks:  A JFlex (jflex.de) lexer for PSTL formulae
 */

package petruchio.pstl.readers.cup;

import petruchio.pi.readers.cup.PiSymbol;


%%

%public
%class PstlLexer
%implements petruchio.pstl.readers.cup.PstlSymbols, petruchio.pstl.readers.cup.CUPLexer

%unicode

%switch

%line
%column

%cup

%{
  private PiSymbol symbol(int type) {
    return new PiSymbol(type, yyline+1, yycolumn+1, yytext());
  }
%}

LineTerminator = \r | \n | \r\n
InputCharacter = [^\r\n]

WhiteSpace = {LineTerminator} | [ \t\f]

Comment = {MLComment} | {EOLComment}
MLComment = "/*" ~"*/"
EOLComment = "//" {InputCharacter}* {LineTerminator}?

Cha = [0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz*_'~$@%&-]
StringCha = [^\"]

%%

<YYINITIAL> {

/* temporal modal operators */
  "F"			{ return symbol(F); }
  "G"			{ return symbol(G); }
  "X"			{ return symbol(X); }

/* logical operators */
  "and"			{ return symbol(AND); }
  "or"			{ return symbol(OR); }
  "not"			{ return symbol(NOT); }

/* atomic terms */
  "T"			{ return symbol(TRUE); }
  "true"		{ return symbol(TRUE); }
  "*"			{ return symbol(TRUE); }  
  "free"		{ return symbol(FREE); }
  "!free"		{ return symbol(NOTFREE); }
  "notfree"		{ return symbol(NOTFREE); }

/* restriction operator */  
  "res"			{ return symbol(RES); }
  
  "{"			{ return symbol(LBRACE); }
  "}"			{ return symbol(RBRACE); }
  "("			{ return symbol(LPAREN); }
  ")"			{ return symbol(RPAREN); }
  "<"			{ return symbol(LANGLE); }
  ">"			{ return symbol(RANGLE); }
  "["			{ return symbol(LBRACK); }
  "]"			{ return symbol(RBRACK); }
  ";"			{ return symbol(SEMICOLON); }
  "."			{ return symbol(DOT); }
  ","			{ return symbol(COMMA); }
  "="			{ return symbol(EQUAL); }
  "|"			{ return symbol(VERTICAL); }
  "+"			{ return symbol(PLUS); }
  "?"           { return symbol(QUESTION); }
  "!"           { return symbol(EXCLAMATION); }
  "0"			{ return symbol(ZERO); }
  "nil"         { return symbol(ZERO); }
  "tau"			{ return symbol(TAU); }
  "in"			{ return symbol(IN); }
  "out"			{ return symbol(OUT); }
  
  {Comment}		{ /* ignore */ }
  {WhiteSpace}	{ /* ignore */ }
  
  \" {StringCha}* \"	{ return symbol(NAME); }
  {Cha}+		{ return symbol(NAME); }
}

/* error fallback */
.|\n			{ throw new RuntimeException("Illegal character \"" + yytext() + "\" at line " + yyline + ", column " + yycolumn); } 

<<EOF>>	{return symbol(EOF); }