/*
 * Project:  pg
 * Filename: NaturalRange.java
 * Created:  Jul 20, 2005
 * Author:   Tim Strazny
 */
package petruchio.sim.petrinettool.type;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import petruchio.sim.petrinettool.IPlaceType;

/**
 * 
 *
 * @author Tim Strazny
 */
public final class Boolean implements IPlaceType {

    public static final Boolean BOOLEAN = new Boolean();
    
    public static final Collection<String> VALUES;
    
    static {
        final Collection<String> temp = new ArrayList<String>();
        temp.add("tt");
        temp.add("ff");
        VALUES = Collections.unmodifiableCollection(temp);
    }
    
    private Boolean() { /* empty but private */ }
    
    public Boolean getInstance() {
        return Boolean.BOOLEAN;
    }
    
    /**
     * Returns the name of the natural range type. It returns &quot;Boolean&quot;
     * 
     * @return A String representing the type.
     */
    public String getName() {
        return "Boolean";
    }


    /**
     * Returns this type's PNML representation which is <code>&lt;boolean/&gt;</code>.
     * 
     * @return The PNML representation of this type.
     */
    public String getPNML() {
        return "<boolean />\n";
    }


    /**
     * Returns this type's PEP representation which is
     * <code>tt,ff</code>.
     * 
     * @return The PEP representation of this type.
     */
    public String getPEP() {
        return "tt,ff";
    }

    
    /**
     * See {@link #getName()}.
     * @return A String representing this object.
     * @see #getName()
     */
    public String toString() {
        return this.getName();
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(final Object o) {
        return (o != null) && (o instanceof Boolean);
    }
    
    
    public boolean isValue(final String value) {
        return value.equals("tt") || value.equals("ff");
    }
    
    
    public String getRandomValue() {
        if(Math.random() < 0.5) {
            return "ff";
        } else {
            return "tt";
        }
    }

    public Collection<String> getValues() {
        return Boolean.VALUES;
    }
}
