/*
 * Project:  pg
 * Filename: SetOfTypes.java
 * Created:  Jun 9, 2005
 * Author:   Tim Strazny
 */
package petruchio.sim.petrinettool.type;

import petruchio.sim.petrinettool.IPlaceType;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;


/**
 * TODO: document
 *
 * @author Tim Strazny
 */
public class Set implements IPlaceType {

    private Collection<IPlaceType> types;
    
    private Collection<String> values;
    
    public Set() {
        this.types = new ArrayList<IPlaceType>();
        this.values = new ArrayList<String>();
    }
    
    
    public boolean isEmpty() {
        return this.types.isEmpty();
    }
    
    
    public Collection<String> getValues() {
        return Collections.unmodifiableCollection(this.values);
    }
    
    public Collection<IPlaceType> getTypes() {
        return Collections.unmodifiableCollection(this.types);
    }
    
    
    public void addType(final IPlaceType type) {
        if(type == null) {
            throw new IllegalArgumentException("Type may not be null");
        } else if(type instanceof Set) {
            this.addTypes((Set)type);
        } else {
            if(!this.types.contains(type)) {
                this.types.add(type);
                for(final String v : type.getValues()) {
                    if((v != null) && !this.values.contains(v)) {
                        this.values.add(v);
                    }
                }
            }
        }
    }
    
    
    public void addTypes(final Set theTypes) {
        if(theTypes == null) {
            throw new IllegalArgumentException("Types may not be null");
        }
        for(final IPlaceType type : theTypes.types) {
            this.addType(type);
        }
    }
    
    
    public void clear() {
        this.types.clear();
        this.values.clear();
    }
    
    
    /**
     * @return
     */
    public String getName() {
        return "Set" + this.types + "";
    }


    /**
     * @return
     */
    public String getPNML() {
        final StringBuffer result = new StringBuffer();
        final Collection<IPlaceType> sets = new ArrayList<IPlaceType>();
        final Collection<IPlaceType> values = new ArrayList<IPlaceType>();
        for(final IPlaceType o : this.types) {
            if((o instanceof SingleValue) || (o instanceof BlackToken)) {
                values.add(o);
            } else {
                sets.add(o);
            }
        }
        
        final String valu;
        if(!values.isEmpty()) {
            final StringBuffer v = new StringBuffer(); 
            v.append("<set>\n");
            for(final IPlaceType type : values) {
                v.append(type.getPNML());
            }
            v.append("</set>\n");
            valu = v.toString();
        } else {
            valu = "";
        }
        
        if((sets.size() > 1) || (sets.size()*values.size() > 0)) {
            result.append("<apply>\n");
            result.append("<union />\n");
        }
        
        result.append(valu);
        for(final IPlaceType type : sets) {
            result.append(type.getPNML());
        }
        
        if((sets.size() > 1) || (sets.size()*values.size() > 0)) {
            result.append("</apply>\n");
        }
        
        return result.toString();
    }


    /**
     * @return
     */
    public String getPEP() {
        final StringBuffer result = new StringBuffer();
        
        result.append("{");
        for(final Iterator<IPlaceType> it = this.types.iterator(); it.hasNext(); ) {
            result.append(it.next().getPEP());
            if(it.hasNext()) {
                result.append(",");
            }
        }
        result.append("}");
        
        return result.toString();
    }
    
    
    public String toString() {
        return this.getName();
    }

    
    public boolean equals(final Object o) {
        return (o instanceof Set) && this.equals((Set)o);
    }
    
    
    public boolean equals(Set set) {
        if(set == this) {
            return true;
        } else if((set == null) || (set.types.size() != this.types.size())) {
            return false;
        }
        
        final Collection<IPlaceType> list = new ArrayList<IPlaceType>(set.types);
        for(final IPlaceType t : this.types) {
            if(!list.remove(t)) {
                return false;
            }
        }
        return true;
    }
    
    
    public boolean isValue(final String value) {
        if(value == null) {
            return false;
        }
        for(final IPlaceType type : this.types) {
            if(type.isValue(value)) {
                return true;
            }
        }
        
        return false;
    }
    
    
    public String getRandomValue() {
        final int idx = (int)(Math.random()*this.types.size());
        final IPlaceType type = new ArrayList<IPlaceType>(this.types).get(idx);
        return type.getRandomValue();
    }
}
