/*
 * Project:  pg
 * Filename: TupleType.java
 * Created:  Jun 9, 2005
 * Author:   Tim Strazny
 */
package petruchio.sim.petrinettool.type;

import petruchio.sim.petrinettool.IPlaceType;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Iterator;

/**
 * TODO: document
 *
 * @author Tim Strazny
 */
public class Tuple implements IPlaceType {

    private final List<IPlaceType> types;
    
    private final Collection<String> values;
    
    public Tuple() {
        this.types = new ArrayList<IPlaceType>();
        this.values = new ArrayList<String>();
    }
    
    
    public Collection<String> getValues() {
        return Collections.unmodifiableCollection(this.values);
    }
    
    
    public IPlaceType getType(final int index) {
        return this.types.get(index);
    }
    
    
    public int size() {
        return this.types.size();
    }
    
    
    /**
     * @return
     */
    public String getName() {
        return "Tuple" + this.types;
    }
    
    
    public boolean isEmpty() {
        return this.types.isEmpty();
    }
    
    
    public void addType(IPlaceType type) {
        if(type == null) {
            throw new IllegalArgumentException("Type may not be null");
        }
        if(!(type instanceof Set)) {
            final Set t = new Set();
            t.addType(type);
            type = t;
        }
        this.types.add(type);
        if(this.types.size() == 1) {
            for(final String value : type.getValues()) {
                this.values.add("(" + value + ")");
            }
        } else {
            final Collection<String> newValues = new ArrayList<String>();
            for(String value : this.values) {
                value = value.substring(0, value.length()-1) + ",";
                for(final Iterator<String> it2 = type.getValues().iterator(); it2.hasNext(); ) {
                    final String sub = it2.next();
                    newValues.add(value + sub + ")");
                }
            }
            this.values.clear();
            this.values.addAll(newValues);
        }
    }


    /**
     * @return
     */
    public String getPNML() {
        final StringBuffer result = new StringBuffer();
        
        result.append("<apply>\n");
        result.append("<cartesianproduct />\n");
        for(final IPlaceType t : this.types) {
            final String type = t.getPNML();
            if((t instanceof SingleValue) || (t instanceof BlackToken)) {
                result.append("<set>\n");
                result.append(type);
                result.append("</set>\n"); 
            } else {
                result.append(type);
            }
        }
        result.append("</apply>\n");
        
        return result.toString();
    }


    /**
     * @return
     */
    public String getPEP() {
        final StringBuffer result = new StringBuffer();
        
        for(final Iterator<IPlaceType> it = this.types.iterator(); it.hasNext(); ) {
            result.append(it.next().getPEP());
            if(it.hasNext()) {
                result.append("X");
            }
        }
        
        return result.toString();
    }

    
    public String toString() {
        return this.getName();
    }
    
    
    public boolean equals(final Object o) {
        return (o instanceof Tuple) && this.equals((Tuple)o);
    }
    
    
    public boolean equals(final Tuple tuple) {
        if(tuple == this) {
            return true;
        } else if((tuple == null) || (tuple.types.size() != this.types.size())) {
            return false;
        }
        
        for(final Iterator<IPlaceType> it = this.types.iterator(), it2 = tuple.types.iterator();
                it.hasNext(); ) {
            if(!it.next().equals(it2.next())) {
                return false;
            }
        }
        return true;
    }
    
    
    public boolean isValue(String value) {
        try {
            value = value.substring(1, value.length()-1);
            final String[] theValues = value.split(",");
            if(theValues.length != this.types.size()) {
                return false;
            }
            int i = 0;
            for(final Iterator<IPlaceType> it = this.types.iterator(); it.hasNext(); i++) {
                final String val = theValues[i].trim();
                final IPlaceType type = it.next();
                if(!type.isValue(val)) {
                    return false;
                }
            }
            
            return true;
        } catch(final Exception e) {
            return false;
        }
    }
    
    
    public String getRandomValue() {
        final StringBuffer buf = new StringBuffer();
        buf.append("(");
        for(final Iterator<IPlaceType> it = this.types.iterator(); it.hasNext(); ) {
            final IPlaceType type = it.next();
            buf.append(type.getRandomValue());
            if(it.hasNext()) {
                buf.append(",");
            }
        }
        buf.append(")");
        return buf.toString();
    }
}
