/*
 * Project:  UML
 * Filename: Action.java
 * Created:  24.04.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.net;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class Action implements Cloneable {
    
    public static final Tuple DEFAULT_LABEL = new Tuple();

    public static enum Type { ACTION, TIE };

    private Type type;

    private String symbol;

    private Tuple label;

    private boolean isSending;


    public Action(final Type type, final String symbol, final String label,
                    final boolean isSending) {
        this.type = type;
        this.symbol = symbol;
        this.setLabel(label);
        this.isSending = isSending;
    }
    
    public Action(final Type type, final String symbol, final Tuple label,
                    final boolean isSending) {
        this.type = type;
        this.symbol = symbol;
        this.label = new Tuple(label);
        this.isSending = isSending;
    }
    
    
    public void setLabel(String label) {
        if((label == null) || label.equals("")) {
            this.label = Action.DEFAULT_LABEL;
        } else {
            if(!label.startsWith("(")) {
                label = "(" + label + ")";
            }
            this.label = new Tuple(label);
        }
    }


    public String getSymbol() {
        return this.symbol;
    }


    public Tuple getLabel() {
        return this.label;
    }


    public boolean isSending() {
        return this.isSending;
    }


    public Type getType() {
        return this.type;
    }


    public Action clone() {
        Action clone = null;
        try {
            clone = (Action)super.clone();
            clone.symbol = this.symbol;
            clone.isSending = this.isSending;
            clone.label = this.label.clone();
            clone.type = this.type;
        } catch(final CloneNotSupportedException e) {
            throw new InternalError();
        }
        return clone;
    }
    
    
    public void clear() {
        // nothing
    }

    
    public boolean equalSymbol(final Action a) {
        return (this == a) || ((a != null)
                && (this.isSending == this.isSending())
                && (this.type == a.getType())
                && this.symbol.equals(a.getSymbol()));
    }
    

    @Override
    public boolean equals(final Object o) {
        return (o instanceof Action) && this.equals((Action) o);
    }


    public boolean equals(final Action a) {
        return (this == a)
                || (this.equalSymbol(a) && this.label.equals(a.getLabel()));
    }

    
    @Override
    public int hashCode() {
        return this.type.hashCode() ^ this.getSymbol().hashCode() ^ this.label.hashCode();
    }

    
    @Override
    public String toString() {
        final String t;
        if(this.type == Action.Type.TIE) {
            t = "tie";
        } else if(this.type == Action.Type.ACTION) {
            t = "action";
        } else {
            t = "UNKNOWN";
        }
        
        return t + "(" + this.symbol + (this.isSending ? '!' : '?')
                + this.label.toString() + ")";
    }
}