/*
 * Project:  UML
 * Filename: Label.java
 * Created:  27.04.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.net;


import java.util.List;
import java.util.Iterator;

import petruchio.sim.pnmodel.PNModel;
import petruchio.sim.pnmodel.util.Pool;

/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class Label implements Cloneable {

    private ValueList values;

    private ValueList variables;

    private ValueList tuples;
    
    private List<String> expressions;
    
    private ValueList all;

    
    public Label(String text) {
        if(text != null) {
            text = PNModel.DOT.matcher(text).replaceAll(PNModel.DOT_REPL);
            text = PNModel.cleanup(text);
        }
        this.values = Pool.getPool().getValueList();
        this.variables = Pool.getPool().getValueList();
        this.tuples = Pool.getPool().getValueList();
        this.expressions = Pool.getPool().getList();
        this.all = null;
        if(text != null) {
            PNModel.parse(text, this.values, this.variables, this.tuples,
                            this.expressions);
        }
    }
    
    
    public Label(final Value value) {
        this.values = Pool.getPool().getValueList();
        this.variables = Pool.getPool().getValueList();
        this.tuples = Pool.getPool().getValueList();
        this.expressions = Pool.getPool().getList();
        this.all = null;
        this.values.add(value);
    }
    
    
    public Label(final Label label) {
        this.values = Pool.getPool().getValueList();
        this.variables = Pool.getPool().getValueList();
        this.tuples = Pool.getPool().getValueList();
        this.expressions = Pool.getPool().getList();
        this.all = null;
        this.add(label);
    }
    
    
    public Label(final Tuple tuple) {
        this.values = Pool.getPool().getValueList();
        this.variables = Pool.getPool().getValueList();
        this.tuples = Pool.getPool().getValueList();
        this.expressions = Pool.getPool().getList();
        this.all = null;
        this.tuples.add(Value.getValue(tuple));
    }


    public void add(final Label label) {
        this.values.addAll(label.getValues());
        this.variables.addAll(label.getVariables());
        this.tuples.addAll(label.getTuples());
        this.expressions.addAll(label.getExpressions());
    }
    
    
    public boolean isEmpty() {
        return this.values.isEmpty() && this.variables.isEmpty()
                && this.tuples.isEmpty() && this.expressions.isEmpty();
    }
    
    
    public int size() {
        return this.values.size() + this.variables.size() + this.tuples.size()
                + this.expressions.size(); 
    }

    
    /**
     * @return Returns the tuples.
     */
    public ValueList getTuples() {
        return this.tuples;
    }


    /**
     * @return Returns the values.
     */
    public ValueList getValues() {
        return this.values;
    }


    /**
     * @return Returns the variables.
     */
    public ValueList getVariables() {
        return this.variables;
    }
    
    
    public List<String> getExpressions() {
        return this.expressions;
    }

    
    @Override
    public Label clone() {
        Label clone = null;
        try {
            clone = (Label)super.clone();
            clone.all = Pool.getPool().getValueList();
            clone.tuples = Pool.getPool().getValueList();
            clone.variables = Pool.getPool().getValueList();
            clone.expressions = Pool.getPool().getList();
            clone.values = Pool.getPool().getValueList();
            clone.add(this);
        } catch(final CloneNotSupportedException e) {
            throw new InternalError();
        }
        
        return clone;
    }
    
    
    @Override
    public int hashCode() {
        return this.expressions.hashCode() ^ this.tuples.hashCode()
                ^ this.values.hashCode() ^ this.variables.hashCode();
    }
    
    
    @Override
    public boolean equals(final Object o) {
        return (o instanceof Label) && this.equals((Label)o);
    }
    
    
    public boolean equals(final Label label) {
        if(this == label) {
            return true;
        } else if(label == null) {
            return false;
        } else if((this.values.size() + this.variables.size() != 
                    label.getValues().size() + label.getVariables().size())
                        ||(this.tuples.size() != label.getTuples().size())) {
            return false;
        } else {
            return PNModel.setEquals(label.getValues(), this.values)
                 && PNModel.setEquals(this.tuples, label.getTuples())
                 && PNModel.setEquals(this.expressions, label.getExpressions());
        }
    }
    
    
    public ValueList getAll() {
        if(this.all == null) {
            this.all = Pool.getPool().getValueList();
            this.all.addAll(this.values);
            this.all.addAll(this.variables);
            this.all.addAll(this.tuples);
        }
        return this.all;
    }
    

    @Override
    public String toString() {
        final List<Object> all2 = Pool.getPool().getList();
        all2.addAll(this.getAll());
        all2.addAll(this.expressions);

        final StringBuffer text = new StringBuffer();

        for(final Iterator<Object> it = all2.iterator(); it.hasNext(); ) {
            final Object o = it.next();
            if(o instanceof String) {
                text.append((String)o);
            } else {
                Value value = (Value)o;
                text.append(value.getNetValue());
            }
            if(it.hasNext()) {
                text.append(",");
            }
        }
        
        return text.toString();
    }
}
