/*
 * Project:  UML
 * Filename: MultiSet.java
 * Created:  24.04.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.net;


import java.util.Collection;
import java.util.List;
import java.util.Map;

import petruchio.sim.pnmodel.PNModel;
import petruchio.sim.pnmodel.util.Pool;

/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class MultiSet implements petruchio.sim.petrinettool.IMultiSet, Cloneable {

    private Map<String, List<Action>> actionSymbols;
    private Map<String, List<Action>> tieSymbols;
    private List<Action> actionSymbolList;
    private List<Action> tieSymbolList;

    public MultiSet() {
        this.actionSymbols = Pool.getPool().getMap();
        this.tieSymbols = Pool.getPool().getMap();
        this.actionSymbolList = Pool.getPool().getList();
        this.tieSymbolList = Pool.getPool().getList();
    }


    /*
     * (non-Javadoc)
     * 
     * @see petruchio.sim.petrinettool.MultiSet#addSendTie(java.lang.String, java.lang.String)
     */
    public void addSendTie(final String tie, final String label) {
        this.addAction(new Action(Action.Type.TIE, tie, label, true));
    }


    /*
     * (non-Javadoc)
     * 
     * @see petruchio.sim.petrinettool.MultiSet#addReceiveTie(java.lang.String,
     *      java.lang.String)
     */
    public void addReceiveTie(final String tie, final String label) {
        this.addAction(new Action(Action.Type.TIE, tie, label, false));
    }


    /*
     * (non-Javadoc)
     * 
     * @see petruchio.sim.petrinettool.MultiSet#addActionSymbol(String, String)
     */
    public void addActionSymbol(final String action, final String label) {
        this.addAction(new Action(Action.Type.ACTION, action, label, false));
    }


    /*
     * (non-Javadoc)
     * 
     * @see petruchio.sim.petrinettool.MultiSet#addConjugatedActionSymbol(String, String)
     */
    public void addConjugatedActionSymbol(final String action, final String label) {
        this.addAction(new Action(Action.Type.ACTION, action, label, true));
    }
    
    
    public void addSet(petruchio.sim.petrinettool.IMultiSet set) {
        this.addMultiSet((MultiSet)set);
    }
    
    
    public void addMultiSet(final MultiSet set) {
        if(set == null) {
            throw new RuntimeException("Cannot add multiset NULL");
        }
        for(final Action a : set.getActionSymbols()) {
            this.addAction(a.clone());
        }
        for(final Action a : set.getTieSymbols()) {
            this.addAction(a.clone());
        }
    }

    
//    private boolean addUnique(ArrayList list, Action action) {
//        for(int i = 0; i < list.size(); i++) {
//            Action a = (Action)list.get(i);
//            if(a.getSymbol().equals(action.getSymbol())
//                            && a.getLabel().toString().equals(
//                                            action.getLabel().toString())) {
//                return false; 
//            }
//        }
//        
//        list.add(action);
//        return true;
//    }
    
    
//    private void addUniqueTie(Action tie) {
//        if(!this.addUnique(this.tieSymbols, tie)) {
//            throw new RuntimeException("Error: Tried to add dublicate tie-"
//                            + "symbol to a multi set.\nNotice that you may "
//                            + "not scope when there are tie-symbols "
//                            + "present.\nYou may scope after performing a "
//                            + "tieAll() operation.\nAction: " + tie); 
//        }
//    }
    
    
    public Action getAction(final Action action) {
        final List<Action> list;
        if(action.getType() == Action.Type.TIE) {
            list = this.tieSymbolList;
        } else {
            list = this.actionSymbolList;
        }
        for(final Action a : list) {
            if((a.isSending() == action.isSending())
                            && a.getLabel().equals(action.getLabel())
                            && a.getSymbol().equals(action.getSymbol())) {
                return a;
            }
        }
        
        return null;
    }


    public boolean isEmpty() {
        return this.actionSymbols.isEmpty() && this.tieSymbols.isEmpty();
    }
    
    
    public int size() {
        return this.actionSymbols.size() + this.tieSymbols.size();
    }
    
    
    public Action getAction(final String symbol, final Action.Type type,
                    final boolean sending) {
        final List<Action> list;
        if(type == Action.Type.TIE) {
            list = this.tieSymbols.get(symbol);
        } else if(type == Action.Type.ACTION) {
            list = this.actionSymbols.get(symbol);
        } else {
            list = null;
        }
        if(list == null) {
            return null;
        }
        
        for(final Action a : list) {
            if(a.isSending() == sending) {
                return a;
            }
        }
        
        return null;
    }
    
    
    public Action getAction(final String symbol, final Action.Type type) {
        final List<Action> list;
        if(type == Action.Type.TIE) {
            list = this.tieSymbols.get(symbol);
        } else if(type == Action.Type.ACTION) {
            list = this.actionSymbols.get(symbol);
        } else {
            list = null;
        }
        if(list == null) {
            return null;
        } else {
            return list.get(0);
        }
    }


    public void addAction(final Action action) {
        if(action.getSymbol() == null) {
            throw new RuntimeException("Cannot add action without a symbol.");
        } else if(action.getSymbol().length() == 0) {
            throw new RuntimeException("Cannot add action with an empty symbol.");
        }
        
        if(action.getType() == Action.Type.TIE) {
            //this.addUniqueTie(action);
            List<Action> list = this.tieSymbols.get(action.getSymbol());
            if(list == null) {
                list = Pool.getPool().getList();
                this.tieSymbols.put(action.getSymbol(), list);
            }
            list.add(action);
            this.tieSymbolList.add(action);
        } else if(action.getType() == Action.Type.ACTION) {
            List<Action> list = this.actionSymbols.get(action.getSymbol());
            if(list == null) {
                list = Pool.getPool().getList();
                this.actionSymbols.put(action.getSymbol(), list);
            }
            list.add(action);
            this.actionSymbolList.add(action);
        }
    }
    
    
    public void removeAction(final Action action) {
        if(action == null) {
            return;
        } else if(action.getType() == Action.Type.ACTION) {
            final List<Action> list = this.actionSymbols.get(action.getSymbol());
            PNModel.removeThisObject(this.actionSymbolList, action);
            PNModel.removeThisObject(list, action);
        } else if(action.getType() == Action.Type.TIE) {
            final List<Action> list = this.tieSymbols.get(action.getSymbol());
            PNModel.removeThisObject(this.tieSymbolList, action);
            PNModel.removeThisObject(list, action);
        }
    }


    public Action[] getTie(final String symbol) {
        final List<Action> list = this.tieSymbols.get(symbol);
        if(list == null) {
            return new Action[0];
        }
        
        final Action[] actions = new Action[list.size()];
        list.toArray(actions);
        return actions;
    }
    
    
    public Action[] getAction(final String symbol) {
        final List<Action> list = this.actionSymbols.get(symbol);
        if(list == null) {
            return new Action[0];
        }
        
        final Action[] actions = new Action[list.size()];
        list.toArray(actions);
        return actions;
    }   

    
    public List<Action> getActionSymbols() {
        return this.actionSymbolList;
    }
    
    
    public List<Action> getTieSymbols() {
        return this.tieSymbolList;
    }


    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#clone()
     */
    @Override
    public MultiSet clone() {
        MultiSet set = null;
        try{
            set = (MultiSet)super.clone();
            set.actionSymbolList = Pool.getPool().getList();
            set.actionSymbols = Pool.getPool().getMap();
            set.tieSymbolList = Pool.getPool().getList();
            set.tieSymbols = Pool.getPool().getMap();
        } catch(final CloneNotSupportedException e) {
            throw new InternalError();
        }
        set.addMultiSet(this);
        return set;
    }


    /*
     * (non-Javadoc)
     * 
     * @see petruchio.sim.petrinettool.MultiSet#clear()
     */
    public void clear() {
        this.actionSymbols.clear();
        this.tieSymbols.clear();
        this.actionSymbolList.clear();
        this.tieSymbolList.clear();
    }

    
    public static boolean actionSetsAreEqual(final List<Action> set1, final List<Action> set2) {
        if(set1.size() != set2.size()) {
            return false;
        }
        
        boolean[] used = new boolean[set2.size()];
        for(int i = 0; i < set1.size(); i++) {
            final Action a = set1.get(i);
            boolean unmatched = true;
            for(int j = 0; unmatched && (j < used.length); j++) {
                if(!used[j] && a.equals(set2.get(j))) {
                    unmatched = false;
                    used[j] = true;
                }
            }
            if(unmatched) {
                return false;
            }
        }
        
        return true;
    }
    
    
    @Override
    public boolean equals(final Object o) {
        return (o instanceof MultiSet) && this.equals((MultiSet)o);
    }
    
    
    public boolean equals(final MultiSet set) {
        return (set == this) || ((set != null)
                && MultiSet.actionSetsAreEqual(this.tieSymbolList, set.getTieSymbols())
                && MultiSet.actionSetsAreEqual(this.actionSymbolList, set.getActionSymbols()));
    }
    
    
    @Override
    public int hashCode() {
        return this.actionSymbols.hashCode() ^ this.tieSymbols.hashCode();
    }

    
    public static boolean containsAction(final Collection<Action> actions, final Action action2) {
        return actions.contains(action2);
    }


    @Override
    public String toString() {
        final StringBuffer result = new StringBuffer();
        for(int i = 0; i < this.actionSymbolList.size(); i++) {
            result.append(this.actionSymbolList.get(i));
            if (i < this.actionSymbolList.size() - 1) {
                result.append(", ");
            }
        }
        if((this.tieSymbolList.size() > 0) && (this.actionSymbolList.size() > 0)) {
            result.append(", ");
        }
        for(int i = 0; i < this.tieSymbolList.size(); i++) {
            result.append(this.tieSymbolList.get(i));
            if (i < this.tieSymbolList.size() - 1) {
                result.append(", ");
            }
        }
        return result.toString();
    }

}