/*
 * Project:  UML
 * Filename: Tuple.java
 * Created:  27.04.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.net;


import java.util.List;
import java.util.Iterator;

import petruchio.sim.pnmodel.PNModel;
import petruchio.sim.pnmodel.util.Pool;


/**
 * <Description>
 * 
 * @author Tim Strazny
 */
public class Tuple implements Cloneable {
    private List<Value> entries;
    private ValueList variables;
    
    public Tuple() {
        this.entries = Pool.getPool().getList();
        this.variables = Pool.getPool().getValueList();
    }
    
    public Tuple(final String text) {
        this(text, false);
    }
    
    public Tuple(String text, final boolean strings) {
        this();
        // remove braces
        text = text.substring(1, text.length() - 1);
        text = PNModel.DOT.matcher(text).replaceAll(PNModel.DOT_REPL);
        text = PNModel.cleanup(text);
        PNModel.parse(text, this.entries, this.entries, this.entries, null, strings);
        
        for(final Value o : this.entries) {
            if(o.isVariable()) {
                if(!this.variables.contains(o)) {
                    this.variables.add(o);
                }
            } else if(o.isTuple()) {
                for(final Value var : o.getTuple().getVariables()) {
                    if(!this.variables.contains(var)) {
                        this.variables.add(var);
                    }
                }
            }
        }
    }
    
    
    public Tuple(final Tuple tuple) {
        this.entries = Pool.getPool().getList();
        this.variables = Pool.getPool().getValueList();
        for(final Value v : tuple.getEntries()) {
            this.addEntry(v.clone());
        }
    }
   
    
    public ValueList getVariables() {
        return this.variables;
    }
    
    
    public boolean contains(final Value value) {
        for(int i = 0; i < this.entries.size(); i++) {
            final Value v = this.entries.get(i);
            if(v.equals(value) || (v.isTuple() && v.getTuple().contains(value))) {
                return true;
            }
        }
        return false;
    }
    
    
    public boolean containsVariable() {
        return !this.variables.isEmpty();
    }
    
    
    public List<Value> getEntries() {
        return this.entries;
    }
    
    
    public void addEntry(final Value o) {
        if(o.isVariable()) {
            if(!this.variables.contains(o)) {
                this.variables.add(o);
            }
        } else if(o.isTuple()) {
            for(final Value var : o.getTuple().getVariables()) {
                if(!this.variables.contains(var)) {
                    this.variables.add(var);
                }
            }
        }
        this.entries.add(o);
    }
    
    
    public int size() {
        return this.entries.size();
    }
    
    
    public String get(final int i) {
        return this.entries.get(i).toString();
    }
    
    public Value getValue(final int i) {
        return this.entries.get(i);
    }
    
    
    @Override
    public Tuple clone() {
        Tuple t = null;
        try {
            t = (Tuple)super.clone();
            t.entries = Pool.getPool().getList();
            t.variables = Pool.getPool().getValueList();
            for(final Value v : this.entries) {
                t.addEntry(v.clone());
            }
        } catch(final CloneNotSupportedException e) {
            throw new InternalError();
        }
        return t;
    }
    
    
    @Override
    public int hashCode() {
        return this.entries.hashCode();
    }
    
    
    @Override
    public boolean equals(final Object o) {
        return (o instanceof Tuple) && this.equals((Tuple)o);
    }
    
    
    public boolean equals(final Tuple tuple) {
        if(tuple == this) {
            return true;
        } else if(tuple == null) {
            return false;
        }
        if(this.size() != tuple.size()) {
            return false;
        }
        final List<Value> foreignEntries = tuple.getEntries();
        for(final Iterator<Value> itA = this.entries.iterator(),
                     itB = foreignEntries.iterator(); itA.hasNext(); ) {
            final Value oA = itA.next();
            final Value oB = itB.next();
            
            if(!oA.equals(oB)) {
                return false;
            }
        }
        
        return true;
    }
    
    
    @Override
    public String toString() {
        final StringBuffer text = new StringBuffer();
        text.append("(");

        for(final Iterator<Value> it = this.entries.iterator(); it.hasNext(); ) {
            final Value value = it.next();
            value.setNetValue();
            text.append(value.getNetValue());
            if(it.hasNext()) {
                text.append(",");
            }
        }
        text.append(")");

        return text.toString();
    }
}
