/*
 * Project:  P-UMLaut
 * Filename: Functions.java
 * Created:  09.08.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import petruchio.sim.pnmodel.PNModel;
import petruchio.sim.pnmodel.net.Tuple;
import petruchio.sim.pnmodel.net.Value;

public class Functions {
    private static final InputStream IN = System.in;
    private static final PrintStream OUT = System.out;
    private static final BufferedReader SCANNER = new BufferedReader(new InputStreamReader(Functions.IN));

    public static boolean allowAssign(final String func) {
        return (func == null) || (!func.equals("forall") && !func.equals("exists") 
                && !func.equals("ifthen") && !func.equals("exists1")
                && !func.equals("countTrue") && !func.equals("evaluate"));
    }
    
    public static boolean canBeEvaluatedStatically(final String func) {
        return (func == null) || (!func.equals("rand") && !func.equals("property")
                        && !func.equals("write") && !func.equals("read")
                        && !func.equals("evaluate"));
    }
    
    public static String inverse(final String func) {
        if(func.equals("exp")) {
            return "logn";
        } else if(func.equals("logn")) {
            return "exp";
//        } else if(func.equals("sqrt")) {
//            return "sqr";
        } else if(func.equals("cbrt")) {
            return "cb";
        } else if(func.equals("cb")) {
            return "cbrt";
        } else if(func.equals("deg")) {
            return "rad";
        } else if(func.equals("rev")) {
            return "rev";
        } else if(func.equals("rad")) {
            return "deg";
        } else if(func.equals("pow")) {
            return "log";
        }
        
        return null;
    }
    
    
//    private static Value getParameter(final Value o,
//                    final Map<String, Function> functions, final Binding bind) {
//        return o;
//    }
    
    
    private static Value getParameter(final Node o,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        final Value v = TreeComputer.evaluate(o, functions, bind, properties);
//            if(v == null) {
//                System.err.println("--> " + o + "\n" + bind);
//            }
//        if(v == null) {
//            throw new RuntimeException("Evaluated to 'null': " + o);
//        }
        return v;
    }
    
    
    public static Value evaluate(final String func, final List<Node> operands,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        final List<Value> params;
        // there are functions that evaluate the parameters by themselves
        if(!(func.equals("ifthen") || func.equals("forall")
                        || func.equals("exists") || func.equals("create")
                        || func.equals("evaluate"))) {
            params = Pool.getPool().getList();
            for(int i = 0; i < operands.size(); i++) {
                try {
                    params.add(Functions.getParameter(operands.get(i),
                                    functions, bind, properties));
                } catch(Exception e) {
                    throw new RuntimeException(func + operands + " " + i, e);
                }
            }
        } else {
            params = null;
        }
        
        final Value value;
        // math functions
        if(func.equals("sin")) {
            value = Functions.sin(params);
        } else if(func.equals("sinh")) {
            value = Functions.sinh(params);
        } else if(func.equals("asin")) {
            value = Functions.asin(params);
        } else if(func.equals("cos")) {
            value = Functions.cos(params);
        } else if(func.equals("cosh")) {
            value = Functions.cosh(params);
        } else if(func.equals("acos")) {
            value = Functions.acos(params);
        } else if(func.equals("tan")) {
            value = Functions.tan(params);
        } else if(func.equals("cot")) {
            value = Functions.cot(params);
        } else if(func.equals("acot")) {
            value = Functions.acot(params);
        } else if(func.equals("coth")) {
            value = Functions.coth(params);
        } else if(func.equals("tanh")) {
            value = Functions.tanh(params);
        } else if(func.equals("atan")) {
            value = Functions.atan(params);
        } else if(func.equals("abs")) {
            value = Functions.abs(params);
        } else if(func.equals("max")) {
            value = Functions.max(params);
        } else if(func.equals("min")) {
            value = Functions.min(params);
        } else if(func.equals("ceil")) {
            value = Functions.ceil(params);
        } else if(func.equals("floor")) {
            value = Functions.floor(params);
        } else if(func.equals("rand")) {
            value = Functions.rand(params);
        } else if(func.equals("e")) {
            value = Functions.e(params);
        } else if(func.equals("pi")) {
            value = Functions.pi(params);
        } else if(func.equals("exp")) {
            value = Functions.exp(params);
        } else if(func.equals("log")) {
            value = Functions.log(params);
        } else if(func.equals("logn")) {
            value = Functions.logn(params);
        } else if(func.equals("log10")) {
            value = Functions.log10(params);
        } else if(func.equals("round")) {
            value = Functions.round(params);
        } else if(func.equals("sig")) {
            value = Functions.sig(params);
        } else if(func.equals("sqrt")) {
            value = Functions.sqrt(params);
        } else if(func.equals("root")) {
            value = Functions.root(params);
        } else if(func.equals("cbrt")) {
            value = Functions.cbrt(params);
        } else if(func.equals("cb")) {
            value = Functions.cb(params);
        } else if(func.equals("sqr")) {
            value = Functions.sqr(params);
        } else if(func.equals("deg")) {
            value = Functions.deg(params);
        } else if(func.equals("rad")) {
            value = Functions.rad(params);
        } else if(func.equals("pow")) {
            value = Functions.pow(params);
        } else if(func.equals("factorial")) {
            value = Functions.factorial(params);
            
        } else if(func.equals("property")) {
            value = Functions.property(params, properties); // special
            
        // functions on tuples
            
        } else if(func.equals("proj")) {
            value = Functions.proj(params);
        } else if(func.equals("sum")) {
            value = Functions.sum(params);
        } else if(func.equals("prod")) {
            value = Functions.prod(params);
        } else if(func.equals("map")) {
            value = Functions.map(params, functions, bind, properties);
        } else if(func.equals("dim")) {
            value = Functions.dim(params);
        } else if(func.equals("reverse")) {
            value = Functions.reverse(params);
        } else if(func.equals("append")) {
            value = Functions.append(params);
        } else if(func.equals("concatenate")) {
            value = Functions.concatenate(params);
        } else if(func.equals("replace")) {
            value = Functions.replace(params);
        } else if(func.equals("remove")) {
            value = Functions.remove(params);
        } else if(func.equals("create")) {
            value = Functions.create(operands, functions, bind, properties);
        } else if(func.equals("asTuple")) {
            value = Functions.asTuple(params);
            
        } else if(func.equals("isTuple")) {
            value = Functions.isTuple(params);
        } else if(func.equals("isNumber")) {
            value = Functions.isNumber(params);
        } else if(func.equals("isBoolean")) {
            value = Functions.isBoolean(params);
        } else if(func.equals("isString")) {
            value = Functions.isString(params);
        } else if(func.equals("isDot")) {
            value = Functions.isDot(params);
            
        } else if(func.equals("read")) {
            value = Functions.read(params);
        } else if(func.equals("write")) {
            value = Functions.write(params);
            
            
        // programming functions
        } else if(func.equals("ifthen")) {
            value = Functions.ifthen(operands, functions, bind, properties);
        } else if(func.equals("forall")) {
            value = Functions.forall(operands, functions, bind, properties);
        } else if(func.equals("exists")) {
            value = Functions.exists(operands, functions, bind, properties);
        } else if(func.equals("exists1")) {
            value = Functions.exists1(operands, functions, bind, properties);
        } else if(func.equals("countTrue")) {
            value = Functions.countTrue(operands, functions, bind, properties);
        } else if(func.equals("evaluate")) {
            value = Functions.evaluate(operands, functions, bind, properties);
            
        } else if((functions != null) && functions.containsKey(Function.getFunctionName(func, params.size()))) {
            final Function function = functions.get(Function.getFunctionName(func, params.size()));
            return function.evaluate(operands, functions, bind, properties);
        } else {
            throw new RuntimeException("Unknown function in call: " + func + "("
                            + params + ")" + " Binding: " + bind + "\nKnown "
                            + "functions: " + (functions != null ? functions.keySet() : null));
        }
        
        return value;
    }
    
    
    public static Value property(final List<Value> params,
                    final Map<Value, Value> properties) {
        if(properties == null) {
            throw new RuntimeException("The property function is not available "
                            + "in this context.");
        }
        if(params.size() != 1) {
            throw new RuntimeException("The property function expects one parameter.");
        }
        final Value k = params.get(0);
        final Value v = properties.get(k);
        if(v == null) {
            throw new RuntimeException("The property " + k.getNetValue()
                            + " is not available in this context.");
        }
        return v;
    }
    
    
    public static Value read(final List<Value> params) {
        if(Functions.write(params) == null) {
            return null;
        }
        try {
            final String in = PNModel.escape(Functions.SCANNER.readLine());
            return Value.getValueNoVars(in);
        } catch(IOException e) {
            return null;
        }
    }
    
    
    public static Value write(final List<Value> params) {
//        System.out.println(params);
        if(params.contains(null)) {
            return null;
        }
        for(int i = 0, n = params.size(); i < n; ++i) {
            final Value val = params.get(i);
            final String s = PNModel.unescape(val.isString() ? val.getString() : val.getNetValue());
            Functions.OUT.print(s);
        }
        Functions.OUT.flush();
        return Value.getValue(true);
    }
    
    
    public static Value asTuple(final List<Value> params) {
        final Tuple t = new Tuple();
        for(int i = 0, n = params.size(); i < n; ++i) {
            final Value val = params.get(i);
            t.addEntry(val);
        }
        return Value.getValue(t);
    }
    
    
    public static Value isTuple(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The isTuple function expects one parameter.");
        }
        final Value v = params.get(0);
        return Value.getValue((v != null) && v.isTuple());
    }
    
    
    public static Value isBoolean(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The isBoolean function expects one parameter.");
        }
        final Value v = params.get(0);
        return Value.getValue((v != null) && v.isBoolean());
    }
    
    
    public static Value isNumber(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The isNumber function expects one parameter.");
        }
        final Value v = params.get(0);
        return Value.getValue((v != null) && v.isNumber());
    }
    
    
    public static Value isString(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The isString function expects one parameter.");
        }
        final Value v = params.get(0);
        return Value.getValue((v != null) && v.isString());
    }
    
    
    public static Value isDot(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The isDot function expects one parameter.");
        }
        final Value v = params.get(0);
        return Value.getValue((v != null) && v.isDot());
    }
    
    
    public static Value evaluate(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2) {
            throw new RuntimeException("The evaluate function expects two parameters: " + params
                            + "\nUsage: evaluate(<func>, (<param0>, ..., <paramN>)) = <func>(<param0>, ..., <paramN>)");
        }
        try {
            final String func = Functions.getParameter(params.get(0), functions,
                            bind, properties).getString();
            Node n = params.get(1);
            final List<Node> arguments;
            if(n.getToken().isValue() && n.getToken().getValue().isVariable()
                            && bind.hasBinding(n.getToken().getValue().getVariable())) {
                n = new Node(null, bind.getBinding(n.getToken().getValue().getVariable()));
            }
            if(n.getToken().getOperator() == Token.Operator.SEPARATOR && n.hasChildren()) {
                arguments = n.getChildren();
            } else if(!n.getToken().isValue()) {
                final Value v = Functions.getParameter(n, functions, bind, properties);
                if(v.isTuple()) {
                    arguments = Pool.getPool().getList();
                    final List<Value> vals = v.getTuple().getEntries();
                    for(int i = 0, in = vals.size(); i < in; ++i) {
                        final Value e = vals.get(i);
                        arguments.add(new Node(null, e));
                    }
                } else {
                    arguments = Pool.getPool().getList();
                    arguments.add(new Node(null, v));
                }
            } else if(n.getToken().isValue() && n.getToken().getValue().isTuple()) {
                arguments = Pool.getPool().getList();
                final List<Value> vals = n.getToken().getValue().getTuple().getEntries();
                for(int i = 0, in = vals.size(); i < in; ++i) {
                    final Value v = vals.get(i);
                    arguments.add(new Node(null, v));
                }
            } else {
                arguments = Pool.getPool().getList();
                arguments.add(new Node(null, n.getToken()));
            }
            final Value result = Functions.evaluate(func, arguments, functions, bind, properties);
            return result;
        } catch(final RuntimeException e) {
//            e.printStackTrace();
            return null;
        }
    }
    
    
    public static Value create(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2 && params.size() != 3 && params.size() != 5) {
            throw new RuntimeException("The create function expects two, three or five parameters: " + params
                            + "\nUsage: create(<var>, <expression>) the expression will be evaluated with value the variable could hold"
                            + "\n    note that the set of values depends on the simulator's optimizations etc. In expressions such as"
                            + "\n    \"X{0..5}:forall(X, <expression>)\" where X only appears in that context no problem appears"
                            + "\nor forall(<var>, (t1, ..., tn), <expression>) the expression will be evaluated with element of the tuple"
                            + "\nor forall(<var>, <from>, <to>, <increment>, <expression>) the expression will be evaluated with each number "
                            + "\n    starting at <from> and incrementing by <increment> undtil <to> is reached"
                            + "\nResult: a tuple (t1, ..., tn) where ti = <expression>");
        }
        Value var = params.get(0).getToken().getValue();
//        if(!var.isVariable()) {
//            throw new RuntimeException("Function forall expects its first "
//                            + "parameter to be an unbound/free variable. "
//                            + "Found: " + var);
//        }
        final Value oldValue = var.copy();
        final Tuple result = new Tuple();
        boolean hasResult = true;
        if(params.size() == 5) {
            try {
                double cur = Functions.getParameter(params.get(1), functions,
                                bind, properties).getNumber();
                final double to = Functions.getParameter(params.get(2),
                                functions, bind, properties).getNumber();
                final double inc = Functions.getParameter(params.get(3),
                                functions, bind, properties).getNumber();
                for(; cur <= to; cur += inc) {
                    var.changeValue(Value.getValue(cur));
//                    System.out.println(params.get(4) + " -> " + var);
                    final Value entry = Functions.getParameter(params.get(4),
                                    functions, bind, properties);
                    if(entry != null) {
                        result.addEntry(var == entry ? entry.copy() : entry);
                    } else {
                        hasResult = false;
                        break;
                    }
                }
            } catch(Exception e) {
                hasResult = false;
            }
            
        } else if(params.size() == 3){
            try {
                final Value v = Functions.getParameter(params.get(1), functions,
                                bind, properties);
                if(v.isTuple()) {
                    final Tuple tuple = v.getTuple();
                    for(final Iterator<Value> it = tuple.getEntries().iterator(); it.hasNext();) {
                        var.changeValue(it.next());
    //                    System.out.println(params.get(2));
                        Value entry = Functions.getParameter(params.get(2),
                                        functions, bind, properties);
                        if(entry != null) {
                            result.addEntry(var == entry ? entry.copy() : entry);
                        } else {
                            hasResult = false;
                            break;
                        }
                    }
                } else {
                    var.changeValue(v);
                    final Value entry = Functions.getParameter(params.get(2),
                                    functions, bind, properties);
                    if(entry != null) {
                        result.addEntry(var == entry ? entry.copy() : entry);
                    } else {
                        hasResult = false;
                    }
                }
            } catch(Exception e) {
                hasResult = false;
            }
        } else {
            try {
                for(final Iterator<Value> it = bind.getConstraint(var.getVariable()).iterator(); it.hasNext();) {
                    var.changeValue(it.next());
//                    System.out.println(params.get(1));
                    Value entry = Functions.getParameter(params.get(1),
                                    functions, bind, properties);
                    if(entry != null) {
                        result.addEntry(var == entry ? entry.copy() : entry);
                    } else {
                        hasResult = false;
                        break;
                    }
                }
            } catch(Exception e) {
                hasResult = false;
            }
        }
        
        var.changeValue(oldValue);
        return hasResult ? Value.getValue(result) : null;
    }
    
    
    public static Value forall(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2 && params.size() != 3 && params.size() != 5) {
            throw new RuntimeException("The forall function expects two, three or five parameters: " + params
                            + "\nUsage: forall(<var>, <condition>) the condition will be tested against each value the variable could hold"
                            + "\n    note that the set of values depends on the simulator's optimizations etc. In expressions such as"
                            + "\n    \"X{0..5}:forall(X, <condition>)\" where X only appears in that context no problem appears"
                            + "\nor forall(<var>, (t1, ..., tn), <condition>) the condition will be tested against each element of the tuple"
                            + "\nor forall(<var>, <from>, <to>, <increment>, <condition>) the condition will be tested against each number "
                            + "\n    starting at <from> and incrementing by <increment> undtil <to> is reached");
        }
        Value var = params.get(0).getToken().getValue();
//        if(!var.isVariable()) {
//            throw new RuntimeException("Function forall expects its first "
//                            + "parameter to be an unbound/free variable. "
//                            + "Found: " + var);
//        }
        final Value oldValue = var.copy();
        boolean result = true;
        boolean hasResult = false;
        if(params.size() == 5) {
            try {
                var.changeValue(Value.V_NONE);
                result = Functions.getParameter(params.get(4), functions, bind,
                                properties).getBoolean();
                hasResult = true;
            } catch(Exception e) {
                hasResult = false;
            }
            if(!hasResult) {
                hasResult = true;
                double cur = Functions.getParameter(params.get(1), functions,
                                bind, properties).getNumber();
                final double to = Functions.getParameter(params.get(2),
                                functions, bind, properties).getNumber();
                final double inc = Functions.getParameter(params.get(3),
                                functions, bind, properties).getNumber();
                try {
                    for(; result && (cur <= to); cur += inc) {
                        var.changeValue(Value.getValue(cur));
    //                    System.out.println(params.get(4));
                        result = result && Functions.getParameter(params.get(4),
                                        functions, bind, properties).getBoolean();
                    }
                } catch(Exception e) {
                    hasResult = false;
                }
            }
            
        } else if(params.size() == 3){
            try {
                var.changeValue(Value.V_NONE);
                result = Functions.getParameter(params.get(2), functions, bind,
                                properties).getBoolean();
                hasResult = true;
            } catch(Exception e) {
                hasResult = false;
            }
            if(!hasResult) {
                hasResult = true;
                try {
                    final Value v = Functions.getParameter(params.get(1),
                                    functions, bind, properties);
                    if(v.isTuple()) {
                        final Tuple tuple = v.getTuple();
                        for(final Iterator<Value> it = tuple.getEntries().iterator(); result && it.hasNext();) {
                            var.changeValue(it.next());
        //                    System.out.println(params.get(2));
                            result = result && Functions.getParameter(
                                            params.get(2), functions, bind,
                                            properties).getBoolean();
                        }
                    } else {
                        var.changeValue(v);
                        result = Functions.getParameter(params.get(2),
                                        functions, bind, properties).getBoolean();
                    }
                } catch(Exception e) {
                    hasResult = false;
                }
            }
        } else {
            try {
                var.changeValue(Value.V_NONE);
                result = Functions.getParameter(params.get(1), functions, bind,
                                properties).getBoolean();
                hasResult = true;
            } catch(Exception e) {
                hasResult = false;
            }
            if(!hasResult) {
                hasResult = true;
                try {
                    for(final Iterator<Value> it = bind.getConstraint(var.getVariable()).iterator(); result && it.hasNext();) {
                        var.changeValue(it.next());
    //                    System.out.println(params.get(1));
                        result = result && Functions.getParameter(params.get(1),
                                        functions, bind, properties).getBoolean();
                    }
                } catch(Exception e) {
                    hasResult = false;
                }
            }
        }
        
        var.changeValue(oldValue);
        return hasResult ? Value.getValue(result) : null;
    }
    
    
    public static Value exists(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2 && params.size() != 3 && params.size() != 5) {
            throw new RuntimeException("The exists function expects two, three or five parameters: " + params
                            + "\nUsage: exists(<var>, <condition>) the condition will be tested against each value the variable could be"
                            + "\n    note that the set of values depends on the simulator's optimizations etc. In expressions such as"
                            + "\n    \"X{0..5}:exists(X, <condition>)\" where X only appears in that context no problem appears"
                            + "\nor exists(<var>, (t1, ..., tn), <condition>) the condition will be tested against each element of the tuple"
                            + "\nor exists(<var>, <from>, <to>, <increment>, <condition>) the condition will be tested against each number "
                            + "    starting at <from> and incrementing by <increment> undtil <to> is reached");
        }
        Value var = params.get(0).getToken().getValue();
//        if(!var.isVariable()) {
//            throw new RuntimeException("Function exists expects its first "
//                            + "parameter to be an unbound/free variable. "
//                            + "Found: " + var);
//        }
        final Value oldValue = var.copy();
        boolean result = false;
        boolean hasResult = true;
        if(params.size() == 5) {
            try {
                var.changeValue(Value.V_NONE);
                result = Functions.getParameter(params.get(4), functions, bind,
                                properties).getBoolean();
                hasResult = true;
            } catch(Exception e) {
                hasResult = false;
            }
            if(!hasResult) {
                hasResult = true;
                try {
                    double cur = Functions.getParameter(params.get(1),
                                    functions, bind, properties).getNumber();
                    final double to = Functions.getParameter(params.get(2),
                                    functions, bind, properties).getNumber();
                    final double inc = Functions.getParameter(params.get(3),
                                    functions, bind, properties).getNumber();
                    for(; !result && (cur <= to); cur += inc) {
                        var.changeValue(Value.getValue(cur));
    //                    System.out.println(params.get(4));
                        result = result || Functions.getParameter(params.get(4),
                                        functions, bind, properties).getBoolean();
                    }
                } catch(Exception e) {
                    hasResult = false;
                }
            }
            
        } else if(params.size() == 3){
            try {
                var.changeValue(Value.V_NONE);
                result = Functions.getParameter(params.get(2), functions, bind,
                                properties).getBoolean();
                hasResult = true;
            } catch(Exception e) {
                hasResult = false;
            }
            if(!hasResult) {
                hasResult = true;
                try {
                    final Value v = Functions.getParameter(params.get(1),
                                    functions, bind, properties);
                    if(v.isTuple()) {
                        final Tuple tuple = v.getTuple();
                        for(final Iterator<Value> it = tuple.getEntries().iterator(); !result && it.hasNext();) {
                            var.changeValue(it.next());
        //                    System.out.println(params.get(2));
                            result = result || Functions.getParameter(
                                            params.get(2), functions, bind,
                                            properties).getBoolean();
                        }
                    } else {
                        var.changeValue(v);
                        result = Functions.getParameter(params.get(2),
                                        functions, bind, properties).getBoolean();
                    }
                } catch(Exception e) {
                    hasResult = false;
                }
            }
        } else {
            try {
                var.changeValue(Value.V_NONE);
                result = Functions.getParameter(params.get(1), functions, bind,
                                properties).getBoolean();
                hasResult = true;
            } catch(Exception e) {
                hasResult = false;
            }
            if(!hasResult) {
                hasResult = true;
                try {
                    for(final Iterator<Value> it = bind.getConstraint(var.getVariable()).iterator(); !result && it.hasNext();) {
                        var.changeValue(it.next());
    //                    System.out.println(params.get(1));
                        result = result || Functions.getParameter(params.get(1),
                                        functions, bind, properties).getBoolean();
                    }
                } catch(Exception e) {
                    hasResult = false;
                }
            }
        }
        
        var.changeValue(oldValue);
        return hasResult ? Value.getValue(result) : null;
    }
    
    
    public static Value exists1(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2 && params.size() != 3 && params.size() != 5) {
            throw new RuntimeException("The exists1 function expects two, three or five parameters: " + params
                            + "\nUsage: exists1(<var>, <condition>) the condition will be tested against each value the variable could be"
                            + "\n    note that the set of values depends on the simulator's optimizations etc. In expressions such as"
                            + "\n    \"X{0..5}:exists(X, <condition>)\" where X only appears in that context no problem appears"
                            + "\nor exists1(<var>, (t1, ..., tn), <condition>) the condition will be tested against each element of the tuple"
                            + "\nor exists1(<var>, <from>, <to>, <increment>, <condition>) the condition will be tested against each number "
                            + "    starting at <from> and incrementing by <increment> undtil <to> is reached");
        }
        final Value result = Functions.countTrue(params, functions, bind, properties);
        return result != null ? Value.getValue(result.getNumber() == 1) : null;
    }
    
    
    public static Value countTrue(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2 && params.size() != 3 && params.size() != 5) {
            throw new RuntimeException("The countTrue function expects two, three or five parameters: " + params
                            + "\nUsage: countTrue(<var>, <condition>) the condition will be tested against each value the variable could be"
                            + "\n    note that the set of values depends on the simulator's optimizations etc. In expressions such as"
                            + "\n    \"X{0..5}:exists(X, <condition>)\" where X only appears in that context no problem appears"
                            + "\nor countTrue(<var>, (t1, ..., tn), <condition>) the condition will be tested against each element of the tuple"
                            + "\nor countTrue(<var>, <from>, <to>, <increment>, <condition>) the condition will be tested against each number "
                            + "    starting at <from> and incrementing by <increment> undtil <to> is reached");
        }
        Value var = params.get(0).getToken().getValue();
//        if(!var.isVariable()) {
//            throw new RuntimeException("Function exists expects its first "
//                            + "parameter to be an unbound/free variable. "
//                            + "Found: " + var);
//        }
        final Value oldValue = var.copy();
        int result = 0;
        boolean hasResult = true;
        if(params.size() == 5) {
            try {
                double cur = Functions.getParameter(params.get(1), functions,
                                bind, properties).getNumber();
                final double to = Functions.getParameter(params.get(2),
                                functions, bind, properties).getNumber();
                final double inc = Functions.getParameter(params.get(3),
                                functions, bind, properties).getNumber();
                for(; cur <= to; cur += inc) {
                    var.changeValue(Value.getValue(cur));
//                    System.out.println(params.get(4));
                    final boolean val = Functions.getParameter(params.get(4),
                                    functions, bind, properties).getBoolean();
                    if(val) {
                        ++result;
                    }
                }
            } catch(Exception e) {
                hasResult = false;
            }
            
        } else if(params.size() == 3){
            try {
                final Value v = Functions.getParameter(params.get(1), functions,
                                bind, properties);
                if(v.isTuple()) {
                    final Tuple tuple = v.getTuple();
                    for(final Iterator<Value> it = tuple.getEntries().iterator(); it.hasNext();) {
                        var.changeValue(it.next());
    //                    System.out.println(params.get(2));
                        final boolean val = Functions.getParameter(
                                        params.get(2), functions, bind,
                                        properties).getBoolean();
                        if(val) {
                            ++result;
                        }
                    }
                } else {
                    var.changeValue(v);
                    final boolean val = Functions.getParameter(params.get(2),
                                    functions, bind, properties).getBoolean();
                    if(val) {
                        ++result;
                    }
                }
            } catch(Exception e) {
                hasResult = false;
            }
        } else {
            try {
                for(final Iterator<Value> it = bind.getConstraint(var.getVariable()).iterator(); it.hasNext();) {
                    var.changeValue(it.next());
//                    System.out.println(params.get(1));
                    boolean val = Functions.getParameter(params.get(1),
                                    functions, bind, properties).getBoolean();
                    if(val) {
                        ++result;
                    }
                }
            } catch(Exception e) {
                hasResult = false;
            }
        }
        
        var.changeValue(oldValue);
        return hasResult ? Value.getValue(result) : null;
    }
    
    
    public static Value ifthen(final List<Node> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() < 3) {
            throw new RuntimeException("The ifthen function expects at least three parameters: " + params
                            + "\nUsage: ifthen(<if>, <then>, {<elseif>, <then>,} <else>)");
        }
        try {
            int i = 0;
            boolean condition;
            do {
                condition = Functions.getParameter(params.get(i), functions,
                                bind, properties).getBoolean();
                i += 2;
            } while(!condition && (i < params.size()-1));
            return Functions.getParameter(params.get(condition ? i-1 : i),
                            functions, bind, properties);
        } catch(Exception e) {
            return null;
        }
    }
    
    
    public static Value dim(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The dim function expects one parameter: " + params
                            + "\nUsage: dim((t0, ..., tn)) = n+1");
        }
        final Value v = params.get(0);
        if(v.isTuple()) {
            return Value.getValue(v.getTuple().size());
        } else if(v.isString()) {
            return Value.getValue(v.getString().length());
        } else {
            return Value.getValue(1);
        }
    }
    
    
    public static Value sum(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The sum function expects one parameter: " + params
                            + "\nUsage: sum((t0, ..., tn)) = t0 + ... + tn");
        }
        final Tuple tuple = (params.get(0)).getTuple();
        Value result = Value.getValue(0.0f);
        for(int i = tuple.getEntries().size()-1; i >= 0; --i) {
            result = TreeComputer.performOp(Token.Operator.PLUS, result, tuple.getValue(i));
        }
        return result;
    }
    
    
    public static Value prod(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The prod function expects one parameter: " + params
                            + "\nUsage: prod((t0, ..., tn)) = t0 * ... * tn");
        }
        final Tuple tuple = (params.get(0)).getTuple();
        Value result = Value.getValue(1.0f);
        for(int i = tuple.getEntries().size()-1; i >= 0; --i) {
            result = TreeComputer.performOp(Token.Operator.TIMES, result, tuple.getValue(i));
        }
        return result;
    }
    
    
    public static Value reverse(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The reverse function expects one parameter: " + params
                            + "\nUsage: reverse((t0, ..., tn)) = (tn, ..., t0)");
        }
        final Tuple tuple = (params.get(0)).getTuple();
        final Tuple result = new Tuple();
        for(int i = tuple.getEntries().size()-1; i >= 0; --i) {
            result.addEntry(tuple.getValue(i));
        }
        return Value.getValue(result);
    }
    
    
    public static Value append(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The append function expects two parameter: " + params
                            + "\nUsage: append((t0, ..., tn), X) = (t0, ..., tn, X) or append((s0, ..., sn), (t0, ..., tn)) = (s0, ..., sn, t0, ..., tn, )");
        }
        Tuple tuple = (params.get(0)).getTuple();
        final Value entry = params.get(1);
        final Tuple result = new Tuple();
        for(int i = 0; i < tuple.getEntries().size(); ++i) {
            result.addEntry(tuple.getValue(i));
        }
        if(entry.isTuple()) {
            tuple = entry.getTuple();
            for(int i = 0; i < tuple.getEntries().size(); ++i) {
                result.addEntry(tuple.getValue(i));
            }
        } else {
            result.addEntry(entry);
        }
        return Value.getValue(result);
    }
    
    
    public static Value concatenate(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The concatenate function expects two parameter: " + params
                            + "\nUsage: concatenate((s0, ..., sn), (t0, ..., tn)) = (s0, ..., sn, t0, ..., tn, )");
        }
        final Tuple firstTuple = (params.get(0)).getTuple();
        final Tuple secondTuple = (params.get(1)).getTuple();
        final Tuple result = new Tuple();
        for(int i = 0; i < firstTuple.getEntries().size(); ++i) {
            result.addEntry(firstTuple.getValue(i));
        }
        for(int i = 0; i < secondTuple.getEntries().size(); ++i) {
            result.addEntry(secondTuple.getValue(i));
        }
        return Value.getValue(result);
    }
    
    
    public static Value remove(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The remove function expects two parameters: " + params
                            + "\nUsage: remove((t0, ..., tn), i) = (t0, ..., ti-1, ti+1, ..., tn)");
        }
        final Tuple tuple = (params.get(0)).getTuple();
        final int index = (int)(params.get(1)).getNumber();
        final Tuple result = new Tuple();
        for(int i = 0; i < tuple.getEntries().size(); ++i) {
            if(i != index) {
                result.addEntry(tuple.getValue(i));
            }
        }
        return Value.getValue(result);
    }
    
    
    public static Value replace(final List<Value> params) {
        if(params.size() != 3) {
            throw new RuntimeException("The replace function expects three parameters: " + params
                            + "\nUsage: replace((t0, ..., tn), i, X) = (t0, ..., ti-1, X, ti+1, ..., tn)");
        }
        final Tuple tuple = (params.get(0)).getTuple();
        final int index = (int)(params.get(1)).getNumber();
        final Value entry = params.get(2);
        final Tuple result = new Tuple();
        for(int i = 0; i < tuple.getEntries().size(); ++i) {
            if(i != index) {
                result.addEntry(tuple.getValue(i));
            } else {
                result.addEntry(entry);
            }
        }
        return Value.getValue(result);
    }
    
    
    public static Value map(final List<Value> params,
                    final Map<String, Function> functions, final Binding bind,
                    final Map<Value, Value> properties) {
        if(params.size() != 2) {
            throw new RuntimeException("The map function expects two parameters: " + params
                            + "\nUsage: map(X, (t0, ..., tn)) = (X(t0), ..., X(tn)) or map(X, Y) = X(Y), where X is a function name, f.e. \"sin\".");
        }
        final String function = params.get(0).getString();
        final Value value = params.get(1);
        if(value.isTuple()) {
            final Tuple tuple = value.getTuple();
            final Tuple result = new Tuple();
            final List<Node> operands = Pool.getPool().getList();
            final List<Value> vals = tuple.getEntries();
            for(int i = 0, n = vals.size(); i < n; ++i) {
                final Value val = vals.get(i);
                operands.add(new Node(null, val));
                result.addEntry(Functions.evaluate(function, operands, functions, bind, properties));
                operands.clear();
            }
            return Value.getValue(result);
        } else {
            final List<Node> operands = Pool.getPool().getList();
            operands.add(new Node(null, value));
            final Value result = Functions.evaluate(function, operands, functions, bind, properties);
            return result;
        }
    }
    
    
    public static Value proj(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The proj function expects two parameters: " + params
                            + "\nUsage: proj((t0, ..., tn), i) = ti");
        }
        final Value v = params.get(0);
        final Value index = params.get(1);
        if(v.isTuple()) {
            Tuple tuple = v.getTuple();
            return Value.getValue(tuple.getValue((int)index.getNumber()));
        } else if(v.isString()) {
            return Value.getValueNoVars(""+v.getString().charAt((int)index.getNumber()));
        } else if((int)index.getNumber() == 0) {
            return v;
        } else {
            return null;
        }
    }
    
    
    public static Value sin(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The sin function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.sin(v.getNumber()));
    }
    
    
    public static Value sinh(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The sinh function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.sinh(v.getNumber()));
//        final double x = v.getNumber();
//        return Value.getValue((Math.exp(x) - Math.exp(-x))/2);
    }
    
    
    public static Value pow(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The pow function expects two parameters: " + params);
        }
        final Value v1 = params.get(0);
        final Value v2 = params.get(1);
        return Value.getValue(Math.pow(v1.getNumber(), v2.getNumber()));
    }
    
    
    public static Value cos(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The cos function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.cos(v.getNumber()));
    }
    
    
    public static Value cosh(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The cosh function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.cosh(v.getNumber()));
//        final double x = v.getNumber();
//        return Value.getValue((Math.exp(x) + Math.exp(-x))/2);
    }
    
    
    public static Value cot(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The cot function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.tan(Math.PI/2 - v.getNumber()));
    }
    
    
    public static Value acot(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The acot function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue((Math.atan(1/v.getNumber()) + Math.PI));
    }
    
    
    public static Value coth(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The coth function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        final double ex = Math.exp(2*v.getNumber());
        if(ex == 1) {
            throw new RuntimeException("The coth function expects one parameter different from zero: " + params);
        }
        return Value.getValue((ex + 1)/(ex - 1));
    }
    
    
    public static Value abs(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The abs function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.abs(v.getNumber()));
    }
    
    
    public static Value max(final List<Value> params) {
        if(params.size() == 0) {
            throw new RuntimeException("The max function expects at least one parameter: " + params);
        }
        double max;
        if(params.size() == 1 && (params.get(0)).isTuple()) {
            final Tuple t = (params.get(0)).getTuple(); 
            max = t.getValue(0).getNumber();
            for(int i = 1; i < t.size(); i++) {
                final double d = t.getValue(i).getNumber();
                if(d > max) {
                    max = d;
                }
            }
        } else {
            max = (params.get(0)).getNumber();
            for(int i = 1; i < params.size(); i++) {
                final double d = (params.get(i)).getNumber();
                if(d > max) {
                    max = d;
                }
            }
        }
        return Value.getValue(max);
    }
    
    
    public static Value min(final List<Value> params) {
        if(params.size() == 0) {
            throw new RuntimeException("The min function expects at least one parameter: " + params);
        }
        double min;
        if(params.size() == 1 && (params.get(0)).isTuple()) {
            final Tuple t = (params.get(0)).getTuple(); 
            min = t.getValue(0).getNumber();
            for(int i = 1; i < t.size(); i++) {
                final double d = t.getValue(i).getNumber();
                if(d < min) {
                    min = d;
                }
            }
        } else {
            min = (params.get(0)).getNumber();
            for(int i = 1; i < params.size(); i++) {
                final double d = (params.get(i)).getNumber();
                if(d < min) {
                    min = d;
                }
            }
        }
        return Value.getValue(min);
    }
    
    
    public static Value rand(final List<Value> params) {
        if(params.size() != 0) {
            throw new RuntimeException("The rand function expects no parameters: " + params);
        }
        return Value.getValue(Math.random());
    }
    
    
    public static Value floor(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The floor function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.floor(v.getNumber()));
    }
    
    
    public static Value sqrt(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The sqrt function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.sqrt(v.getNumber()));
    }
    
    
    public static Value root(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The root function expects two parameter: " + params);
        }
        final Value deg = params.get(0);
        final Value v = params.get(1);
        return Value.getValue(Math.pow(v.getNumber(), 1.0/deg.getNumber()));
    }
    
    
    public static Value cb(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The cb function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(v.getNumber()*v.getNumber()*v.getNumber());
    }
    
    
    public static Value sqr(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The sqr function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(v.getNumber()*v.getNumber());
    }
    
    
    public static Value cbrt(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The cbrt function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.cbrt(v.getNumber()));
//        return Value.getValue(Math.pow(v.getNumber(), 1.0/3.0));
    }
    
    
    public static Value deg(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The deg function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.toDegrees(v.getNumber()));
    }
    
    
    public static Value rad(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The rad function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.toRadians(v.getNumber()));
    }
    
    
    public static Value round(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The round function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.round(v.getNumber()));
    }
    
    
    public static Value acos(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The acos function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.acos(v.getNumber()));
    }
    
    
    public static Value asin(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The asin function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.asin(v.getNumber()));
    }
    
    
    public static Value atan(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The atan function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.atan(v.getNumber()));
    }
    
    
    public static Value tan(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The tan function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.tan(v.getNumber()));
    }
    
    
    public static Value tanh(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The tanh function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.tanh(v.getNumber()));
//        final double ex = Math.exp(2*v.getNumber());
//        return Value.getValue((ex - 1)/(ex + 1));
    }
    
    
    public static Value sig(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The sig function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.signum(v.getNumber()));
//        return Value.getValue(v.getNumber() >= 0 ? 1 : -1);
    }
    
    
    public static Value exp(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The exp function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.exp(v.getNumber()));
    }
    
    
    public static Value factorial(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The factorial function expects one parameter: " + params);
        }
        long l = 1;
        for(int v = (int)params.get(0).getNumber(); v > 1; --v) {
            l *= v;
        }
        return Value.getValue("" + l);
    }
    
    
    public static Value ceil(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The ceil function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.ceil(v.getNumber()));
    }
    
    
    public static Value logn(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The logn function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.log(v.getNumber()));
    }
    
    
    public static Value log10(final List<Value> params) {
        if(params.size() != 1) {
            throw new RuntimeException("The log10 function expects one parameter: " + params);
        }
        final Value v = params.get(0);
        return Value.getValue(Math.log10(v.getNumber()));
//        return Value.getValue(Functions.log(10, v.getNumber()));
    }
    
    
    public static Value log(final List<Value> params) {
        if(params.size() != 2) {
            throw new RuntimeException("The log function expects two parameters: " + params);
        }
        final Value b = params.get(0);
        final Value v = params.get(1);
        return Value.getValue(Functions.log(b.getNumber(), v.getNumber()));
    }
    
    
    public static double log(double base, double x) {
        return (Math.log(x)/Math.log(base));
    }
    
    
    public static Value pi(final List<Value> params) {
        if(params.size() != 0) {
            throw new RuntimeException("The pi function expects no parameters: " + params);
        }
        return Value.getValue(Math.PI);
    }
    
    
    public static Value e(final List<Value> params) {
        if(params.size() != 0) {
            throw new RuntimeException("The e function expects no parameters: " + params);
        }
        return Value.getValue(Math.E);
    }
}
