/*
 * Project:  P-UMLaut
 * Filename: RPN.java
 * Created:  19.08.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.util;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import petruchio.sim.petrinettool.IPlaceType;
import petruchio.sim.petrinettool.PEPReader;
import petruchio.sim.pnmodel.net.ValueList;


public class RPN implements Cloneable {
    private List<Token> rpn;
    private Map<String, ValueList> constraints;
    
    public RPN() {
        this.rpn = Pool.getPool().getList();
        this.constraints = Pool.getPool().getMap();
    }
    
    public Token get(final int i) {
        return this.rpn.get(i);
    }
    
    public int size() {
        return this.rpn.size();
    }
    
    public Iterator<Token> iterator() {
        return this.rpn.iterator();
    }
    
    public int indexOf(final Object o) {
        return this.rpn.indexOf(o);
    }
    
    public void add(Token o) {
        this.rpn.add(o);
    }
    
    
    public void addRPN(final RPN theRPN) {
        this.rpn.addAll(theRPN.rpn);
    }
    
    
    public boolean isEmpty() {
        return this.rpn.isEmpty();
    }
    
    
    public boolean remove(final Object o) {
        return this.rpn.remove(o);
    }
    
    
    public void insertRPN(final int pos, final RPN subRPN) {
        this.rpn.addAll(pos, subRPN.rpn);
    }
    
    
    public void insert(final int pos, final Token o) {
        this.rpn.add(pos, o);
    }
    
    
    public Token removeLast() {
        if(this.isEmpty()) {
            throw new RuntimeException("RPN is empty.");
        }
        return this.rpn.remove(this.rpn.size()-1);
    }
    
    
    public List<Token> getList() {
        return this.rpn;
    }
    
    
    public Token getLast() {
        if(this.isEmpty()) {
            throw new RuntimeException("RPN is empty.");
        }
        return this.rpn.get(this.rpn.size()-1);
    }
    
    
    public Token remove(int i) {
        return this.rpn.remove(i);
    }
    
    
    public void addConstraint(String constr) {
        while(true) {
//            System.out.println(constr);
            constr = constr.replaceFirst("^\\s*(" + Token.Operator.AND.toString() + ")?\\s*", "");
            final int pos = constr.indexOf(Token.Operator.CURLY_BRACES_OPEN.toString());
            if(pos < 0) {
                return;
            }
            int c;
            int p;
            boolean isString = false;
            for(p = pos+1, c = 1; p < constr.length() && c != 0; p++) {
                if(Token.Operator.STRING_SEPARATOR.toString().equals(""+constr.charAt(p))) {
                    isString = !isString;
                } else if(!isString) {
                    if(Token.Operator.CURLY_BRACES_OPEN.toString().equals(""+constr.charAt(p))) {
                        c++;
                    } else if(Token.Operator.CURLY_BRACES_CLOSE.toString().equals(""+constr.charAt(p))) {
                        c--;
                    }
                }
            }
            
            final String var = constr.substring(0, pos).trim();
            final String type = constr.substring(pos, p).trim();
            //System.out.println(var + " => " + type);
            final IPlaceType theType = PEPReader.parseType(type);
            
            final ValueList theValues = Values.getValues(theType);
            ValueList oldConstraint = this.getConstraint(var);
            if(oldConstraint == null) {
                oldConstraint = Pool.getPool().getValueList(theValues);
                this.constraints.put(var, oldConstraint);
            } else {
                oldConstraint.retainAll(theValues);
            }
//            System.out.println(var + " in " + type + " = " + theType);
            
            constr = constr.substring(p);
        }
    }
    
    public Map<String, ValueList> getConstraints() {
        return this.constraints;
    }
    
    public ValueList getConstraint(final String var) {
        return this.constraints.get(var);
    }
    
    
    @Override
    public RPN clone() {
        RPN r = null;
        try {
            r = (RPN)super.clone();
            r.rpn = Pool.getPool().getList();
            r.constraints = Pool.getPool().getMap();
        } catch(final CloneNotSupportedException e) {
            throw new InternalError();
        }
        r.rpn.addAll(this.rpn);
        r.constraints.putAll(this.constraints);
        return r;
    }
    
    
    @Override
    public String toString() {
        final StringBuffer buf = new StringBuffer();
        for(Iterator<Token> it = this.rpn.iterator(); it.hasNext();) {
            final Token o = it.next();
            buf.append(o);
//            if(o.isValue()) {
//                buf.append(o.getValue().getNetValue());
//            } else if(o.isOperator()){
//                buf.append(o.getOperator());
//            } else if(o.isFunction()){
//                buf.append(o.getFunction());
//            }
            if(it.hasNext()) {
                buf.append(" ");
            }
        }
        //buf.append("\n\t");
        //buf.append(this.constraints);
        return buf.toString();
    }
    
    
    public String print() {
        return this.rpn.toString();
    }
}