/*
 * Project:  P-UMLaut
 * Filename: RPNComputer.java
 * Created:  25.07.2005
 * Author:   Tim Strazny
 */
package petruchio.sim.pnmodel.util;

import petruchio.sim.pnmodel.PNModel;
import petruchio.sim.pnmodel.net.Value;
import petruchio.sim.pnmodel.util.Token.Operator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class RPNComputer {
    public static final Collection<Operator> UNARY_OPERATORS;
    public static final Collection<Operator> ARITHMETIC_OPERATORS;
    public static final Collection<Operator> LOGICAL_OPERATORS;
    public static final Collection<Operator> COMPARISON_OPERATORS;
    public static final Collection<Operator> SEPARATORS;
    public static final Collection<Operator> PARENTHESESES;
    public static final Collection<Operator> IGNORE_DURING_INPUT;
    public static final Pattern OPERATOR;
    
    static {
        Collection<Operator> temp;
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.STRING_CONCATENATION);
        IGNORE_DURING_INPUT = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.UNARY_MINUS);
        temp.add(Token.Operator.NOT);
        temp.add(Token.Operator.BITWISE_NOT);
        UNARY_OPERATORS = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.UNARY_MINUS);
        temp.add(Token.Operator.PLUS);
        temp.add(Token.Operator.MINUS);
        temp.add(Token.Operator.TIMES);
        temp.add(Token.Operator.DIVIDE);
        temp.add(Token.Operator.MODULO);
        temp.add(Token.Operator.POWER);
        temp.add(Token.Operator.STRING_CONCATENATION);
        ARITHMETIC_OPERATORS = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.AND);
        temp.add(Token.Operator.OR);
        temp.add(Token.Operator.NOT);
        temp.add(Token.Operator.XOR);
        temp.add(Token.Operator.BITWISE_NOT);
        LOGICAL_OPERATORS = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.EQUAL);
        temp.add(Token.Operator.NOT_EQUAL);
        temp.add(Token.Operator.LESS);
        temp.add(Token.Operator.LESS_EQUAL);
        temp.add(Token.Operator.GREATER);
        temp.add(Token.Operator.GREATER_EQUAL);
        COMPARISON_OPERATORS = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.PARENTHESES_CLOSE);
        temp.add(Token.Operator.PARENTHESES_OPEN);
        temp.add(Token.Operator.BRACKETS_CLOSE);
        temp.add(Token.Operator.BRACKETS_OPEN);
        temp.add(Token.Operator.CURLY_BRACES_CLOSE);
        temp.add(Token.Operator.CURLY_BRACES_OPEN);
        PARENTHESESES = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.add(Token.Operator.CONSTRAINT_SEPARATOR);
//        temp.add(Token.Operator.DECIMAL_SEPARATOR);
        temp.add(Token.Operator.ESCAPE);
        temp.add(Token.Operator.PARENTHESES_OPEN);
        temp.add(Token.Operator.PARENTHESES_CLOSE);
        temp.add(Token.Operator.SEPARATOR);
        temp.add(Token.Operator.STRING_SEPARATOR);
        SEPARATORS = Collections.unmodifiableCollection(temp);
        
        temp = new ArrayList<Operator>();
        temp.addAll(ARITHMETIC_OPERATORS);
        temp.addAll(LOGICAL_OPERATORS);
        temp.addAll(COMPARISON_OPERATORS);
        temp.addAll(PARENTHESESES);
        temp.addAll(SEPARATORS);
        final StringBuffer ops = new StringBuffer();
        for(final Iterator<Operator> it = temp.iterator(); it.hasNext();) {
            final String op = it.next().toString();
            ops.append(RPNComputer.quote(op));
            if(it.hasNext()) {
                ops.append("|");
            }
        }
        OPERATOR = Pattern.compile(ops.toString());
    }
    
    public static String quote(String s) {
        s = PNModel.replaceAll(s, "\\", "\\\\");
        s = PNModel.replaceAll(s, ".", "\\.");
        s = PNModel.replaceAll(s, "+", "\\+");
        s = PNModel.replaceAll(s, "?", "\\?");
        s = PNModel.replaceAll(s, "*", "\\*");
        s = PNModel.replaceAll(s, "|", "\\|");
        s = PNModel.replaceAll(s, ")", "\\)");
        s = PNModel.replaceAll(s, "(", "\\(");
        s = PNModel.replaceAll(s, "]", "\\]");
        s = PNModel.replaceAll(s, "[", "\\[");
        s = PNModel.replaceAll(s, "}", "\\}");
        s = PNModel.replaceAll(s, "{", "\\{");
        s = PNModel.replaceAll(s, "^", "\\^");
        s = PNModel.replaceAll(s, "$", "\\$");
        return s;
    }
    
    public static List<Token> tokenize(final String expr) {
        final Map<String, Value> map = Pool.getPool().getMap();
        final List<Token> l = RPNComputer.tokenize(map, expr);
        return l;
    }
    
    public static List<Token> tokenize(final Map<String, Value> map,
                    final String expr) {
        if(expr == null) {
            return Pool.getPool().getList();
        }
        
        final List<Token> tokens = Pool.getPool().getList();
        final char[] txt = expr.toCharArray();
        final int pos = PNModel.getIndexOfConstraintThingy(expr);
        Matcher match;
        int i = 0;
        if(pos >= 0) {
            tokens.add(Token.getToken(Value.getValue(expr.substring(0, pos-1))));
            tokens.add(Token.getToken(Token.Operator.CONSTRAINT_SEPARATOR));
            i = pos;
        }
                
        boolean firstToken = true;
        boolean lastWasOp = false;
        while(i < txt.length) {
            final String txts = String.valueOf(txt[i]);
            match = RPNComputer.OPERATOR.matcher(expr.substring(i, txt.length));
            if(Character.isWhitespace(txt[i])) {
                i++;
                continue;
            } else if(txts.equals(Token.Operator.PARENTHESES_OPEN.toString())) {
                tokens.add(Token.getToken(Token.Operator.PARENTHESES_OPEN));
                i++;
                lastWasOp = false;
                firstToken = true;
                
            } else if(txts.equals(Token.Operator.PARENTHESES_CLOSE.toString())) {
                tokens.add(Token.getToken(Token.Operator.PARENTHESES_CLOSE));
                i++;
                lastWasOp = false;
                firstToken = false;
                
            } else if(txts.equals(Token.Operator.STRING_SEPARATOR.toString()) || (!match.lookingAt()
                    || RPNComputer.IGNORE_DURING_INPUT.contains(Token.Operator.getOperator(expr.substring(i+match.start(), i+match.end()))))) { // variable or value
                StringBuffer buf = new StringBuffer();
                if(txts.equals(Token.Operator.STRING_SEPARATOR.toString())) { // string
                    buf.append(txt[i]);
                    i++;
                    while(i < txt.length) {
                        buf.append(txt[i]);
                        if(("" + txt[i]).equals(Token.Operator.STRING_SEPARATOR.toString())
                                        && !(""+txt[i-1]).equals(Token.Operator.ESCAPE.toString())) {
                            ++i;
                            break;
                        } 
                        ++i;
                    }
                    String s = buf.toString();
                    s = PNModel.replaceAll(s, Token.Operator.ESCAPE.toString() + Token.Operator.STRING_SEPARATOR.toString(),
                                    Token.Operator.STRING_SEPARATOR.toString());
                    buf.replace(0, buf.length(), s);
                } else {
                    do {
                        buf.append(txt[i]);
                        i++;
                        match = RPNComputer.OPERATOR.matcher(expr.substring(i, txt.length));
                    } while((i < txt.length) && ((!match.lookingAt()
                            || RPNComputer.IGNORE_DURING_INPUT.contains(expr.substring(i+match.start(), i+match.end())))
                                    && !txts.equals(Token.Operator.PARENTHESES_OPEN.toString())
                                    && !txts.equals(Token.Operator.PARENTHESES_CLOSE.toString())
                                    && !txts.equals(Token.Operator.SEPARATOR.toString())));
                }
                final String token = buf.toString().trim();
                
                lastWasOp = false;
                firstToken = false;
                
                if(token.equals(Token.Operator.SEPARATOR.toString())) {
                    firstToken = true;
                    tokens.add(Token.getToken(Token.Operator.SEPARATOR));
                } else if((i < txt.length) && (""+txt[i]).equals(Token.Operator.PARENTHESES_OPEN.toString())) {
                    // function
                    tokens.add(Token.getToken(token));
                } else {
                    Value val = Value.getValue(token);
//                    System.out.println("Looking at " + val);
                    if(val.isVariable() && (map != null)) {
                        final Value v = map.get(val.getVariable());
                        if(i-token.length() >= pos) {
                            if(v == null) {
                                map.put(val.getVariable(), val);
    //                            System.out.println("Var " + val + " created.");
                            } else {
                                val = v;
    //                            System.out.println("Var " + val + " used.");
                            }
                        }
                    }
                    tokens.add(Token.getToken(val));
                }
                
                if((i < txt.length) && token.equals(Token.Operator.SEPARATOR.toString())) {
                    ++i;
                    firstToken = true;
                    tokens.add(Token.getToken(Token.Operator.SEPARATOR));
                }
                
            } else { // operators
                String op = "";
                int end = txt.length;
                while(!match.matches()) {
                    match = RPNComputer.OPERATOR.matcher(expr.substring(i, end--));
                }
                op = expr.substring(i, i+match.end());
                i += match.end();
                
                // handle the unary minus as a different operator
                if(op.equals(Token.Operator.MINUS.toString()) && (lastWasOp || firstToken)) {
                    op = Token.Operator.UNARY_MINUS.toString();
                    
                } else if(op.equals(Token.Operator.BITWISE_NOT.toString())) { // bitwise not => default not
                    op = Token.Operator.NOT.toString();
                }

                tokens.add(Token.getToken(op));
                lastWasOp = true;
                firstToken = false;
            }
        }
        
//        System.out.println(tokens);
        return tokens;
    }
    
    
    public static void buildTuple(final List<Token> tokens, int start, int to, RPN rpn) {
//        System.out.println("tuple: " + tokens.subList(start, to+1));
        int i = start + 1;
        int ctr = 1;
        rpn.add(Token.getToken(Token.Operator.LIST_END));
        int curStart = i;
        while((i <= to) && (ctr > 0)) {
            final Token token = tokens.get(i);
            if(token.isOperator()) {
                if((ctr == 1) && ((token.getOperator() == Token.Operator.SEPARATOR) || (token.getOperator() == Token.Operator.PARENTHESES_CLOSE))) {
//                    System.out.println("->" + tokens.subLists(curStart, i));
                    final RPN subRPN = RPNComputer.infixToRPN(tokens, curStart, i-1);
                    rpn.addRPN(subRPN);
                    curStart = i+1;
                    if(token.getOperator() == Token.Operator.PARENTHESES_CLOSE) {
                        --ctr;
                    }
                } else if(token.getOperator() == Token.Operator.PARENTHESES_OPEN) {
                    ++ctr;
                } else if(token.getOperator() == Token.Operator.PARENTHESES_CLOSE) {
                    --ctr;
                }
            }
            ++i;
        }
        
        //if((start-1 < 0) || tokens.get(start-1).isFunction()) {
            rpn.add(Token.getToken(Token.Operator.LIST_START));
        //}
    }
    
    
    public static RPN infixToRPN(final String expr) {
        final Map<String, Value> map = Pool.getPool().getMap();
        final RPN rpn = RPNComputer.infixToRPN(map, expr);
        return rpn;
    }
    
    
    public static RPN infixToRPN(final Map<String, Value> map, final String expr) {
        final List<Token> tokens = RPNComputer.tokenize(map, expr);
        final RPN rpn = RPNComputer.infixToRPN(tokens);
        return rpn;
    }
    
    
    public static RPN infixToRPN(final List<Token> tokens) {
        final RPN rpn = RPNComputer.infixToRPN(tokens, 0, tokens.size()-1);
//        System.out.println(rpn);
        return rpn;
    }
    
    public static RPN infixToRPN(final List<Token> tokens, final int from, final int to) {
        final Stack<Token> memory = new Stack<Token>();
        final RPN rpn = new RPN();
        int idx = from;
        
        // special: handle the : operator
        for(int i = from; i <= to; i++) {
            if(tokens.get(i).getOperator() == Token.Operator.CONSTRAINT_SEPARATOR) {
                idx = i+1;
                break;
            }
        }
        for(int i = from; i < idx-1; i++) {
            final Token t = tokens.get(i);
            if(t.isValue()) {
                rpn.addConstraint(t.getValue().getNetValue());
            } else {
                rpn.addConstraint(t.getOperator().toString());
            }
        }
        
        boolean nextToken = true;
//        System.out.println(tokens.subList(from, to+1));
        final int start = idx;
        while(idx <= to) {
            final Token last = (idx == start) ? Token.getToken(Token.Operator.AND) : tokens.get(idx-1);
            final Token next = tokens.get(idx);
//            System.out.println(token);
            
            final Token token;
            if(next.isOperator()
                    && last.isOperator()
                    && !RPNComputer.UNARY_OPERATORS.contains(next.getOperator())
                    && (RPNComputer.ARITHMETIC_OPERATORS.contains(next.getOperator())
                        || RPNComputer.COMPARISON_OPERATORS.contains(next.getOperator())
                        || RPNComputer.LOGICAL_OPERATORS.contains(next.getOperator()))
                    && (RPNComputer.ARITHMETIC_OPERATORS.contains(last.getOperator())
                        || RPNComputer.COMPARISON_OPERATORS.contains(last.getOperator())
                        || RPNComputer.LOGICAL_OPERATORS.contains(last.getOperator()))) {
                // encode prefix-operators as functions;
                token = new Token(next.getOperator().toString());
            } else {
                token = next;
            }
            
            if(token.isValue()) { // variable, value
                rpn.add(token);
                nextToken = true;
                
            } else if(token.getOperator() == Token.Operator.PARENTHESES_OPEN) {
                // check if it's a tuple
                int ctr = 1;
                boolean isTuple = false;
                int j = idx+1;
                while((j <= to) && (ctr > 0)) {
                    final Token curToken = tokens.get(j);
                    if(curToken.isOperator()) {
                        if(curToken.getOperator() == Token.Operator.PARENTHESES_OPEN) {
                            ++ctr;
                        } else if(curToken.getOperator() == Token.Operator.PARENTHESES_CLOSE) {
                            --ctr;
                        } else if((ctr == 1) && (curToken.getOperator() == Token.Operator.SEPARATOR)) {
                            isTuple = true;
                        }
                    }
                    ++j;
                }
                
                if(isTuple) {
//                    System.out.println("tuple: " + tokens.subList(idx, j));
                    RPNComputer.buildTuple(tokens, idx, j-1, rpn);
                    idx = j-1;
                } else {
                    if((idx-1 >= 0) && tokens.get(idx-1).isFunction()) {
                        rpn.add(Token.getToken(Token.Operator.LIST_END));
                    }
                    
                    memory.push(Token.getToken(Token.Operator.PARENTHESES_OPEN));
                    
                    if((idx-1 >= 0) && tokens.get(idx-1).isFunction()) {
                        memory.push(Token.getToken(Token.Operator.LIST_START));
                    }
                }
                nextToken = true;
            } else if(token.getOperator() == Token.Operator.PARENTHESES_CLOSE) {
                if(memory.isEmpty()) {
                    throw new RuntimeException("Unbalanced parentheses."
                                    + "\nTokens: " + tokens.subList(from, to+1)
                                    + "\nRPN so far: " + rpn
                                    + "\nMemory: " + memory
                                    + "\nToken " + (idx-from-1) + " => " + tokens.get(idx));
                }
//                System.out.println(memory);
                Token elem;
                while((elem = memory.pop()).getOperator() != Token.Operator.PARENTHESES_OPEN) {
                    if(memory.isEmpty()) {
                        throw new RuntimeException("Unbalanced parentheses."
                                        + "\nTokens: " + tokens.subList(from, to+1)
                                        + "\nRPN so far: " + rpn
                                        + "\nMemory: " + memory
                                        + "\nToken " + (idx-from-1) + " => " + tokens.get(idx));
                    }
                    rpn.add(elem);
                }
                nextToken = true;
                
            } else if(token.getOperator() == Token.Operator.BRACKETS_OPEN) {
                int pos = rpn.size()-1;
                if(!rpn.get(pos).isValue()) {
                    int c = 0;
                    boolean found = false;
                    while((pos >= 0) && (!found || (c != 0))) {
                        Token t = rpn.get(pos);
                        if(t.getOperator() == Token.Operator.LIST_START) {
                            found = true;
                            ++c;
                        } else if(t.getOperator() == Token.Operator.LIST_END) {
                            --c;
                        }
                        --pos;
                    }
                    ++pos;
                }
                rpn.insert(pos, Token.getToken(Token.Operator.LIST_END));
                
                int k = idx+1;
                int o = 1;
                while(k < tokens.size() && (o != 0)) {
                    if(tokens.get(k).getOperator() == Token.Operator.BRACKETS_OPEN) {
                        ++o;
                    } else if(tokens.get(k).getOperator() == Token.Operator.BRACKETS_CLOSE) {
                        --o;
                    }
                    k++;
                }
                final RPN r2 = RPNComputer.infixToRPN(tokens, idx+1, k-2);
                rpn.addRPN(r2);
                rpn.add(Token.getToken(Token.Operator.LIST_START));
                rpn.add(Token.getToken("proj"));
                idx = k-1;
                nextToken = true;
                
            } else if(token.getOperator() == Token.Operator.SEPARATOR) {
                while(memory.peek().getOperator() != Token.Operator.PARENTHESES_OPEN) {
                    rpn.add(memory.pop());
                }
                nextToken = true;
            } else if(memory.isEmpty()
                       || (token.isOperator() && !token.getOperator().isLeftAssociative())
                       || RPNComputer.higherPrecedence(token, memory.peek())) {
                memory.push(token);
                nextToken = true;
            } else {
                rpn.add(memory.pop());
                nextToken = false;
            }
//            System.out.println(">> Tokens: " + tokens.subList(from, to+1)
//                            + "\nRPN so far: " + rpn
//                            + "\nMemory: " + memory
//                            + "\nToken " + (idx-from) + " => " + tokens.get(idx));
            
            if(nextToken) {
                ++idx;
            }
        }
        
        
        while(!memory.isEmpty()) {
            final Token mem = memory.pop();
            if(mem.getOperator() == Token.Operator.PARENTHESES_OPEN) {
                throw new RuntimeException("Unbalanced parentheses."
                                + "\nTokens: " + tokens.subList(from, to+1)
                                + "\nRPN so far: " + rpn
                                + "\nMemory: " + memory);
            }
            rpn.add(mem);
        }
                
        return rpn;
    }
    
    
    public static int getArityOfOperator(final Operator op) {
        if(op == null) {
            return -1;
        } else if(RPNComputer.UNARY_OPERATORS.contains(op)) {
            return 1;
        } else {
            return 2;
        }
    }
    
    
    public static boolean isOperator(final String op) {
        return Token.Operator.getOperator(op) != null;
    }
    
    
    public static boolean higherPrecedence(final Token o1, final Token o2) {
        if(o2.isFunction()) { // nothing's higher than a function
            return false;
        }
        if(o1.isFunction()) { // nothing's higher than a function
            return true;
        }
        if(!o2.isOperator()) { // o2 is a value, nothing's higher than a value
            return false;
        }
        
        return Token.getPrecedence(o1.getOperator()) > Token.getPrecedence(o2.getOperator()); 
    }
    
    public static void main(final String[] args) {
        final StringBuffer buf = new StringBuffer();
        for(int i = 0; i < args.length; i++) {
            buf.append(args[i]);
            if(i < args.length - 1) {
                buf.append(" ");
            }
        }
        String guard = buf.toString();
        if(guard.length() == 0) {
            guard = "-abs(-max(X, 92837))&X=2";
            guard = "X{29} X {dot}: tt";
            guard = "X=(1, 2, 4*2)+5";
            guard = "5=2*(-3+logn((e()*e())**(x+x+x+1)))";
        }
        System.out.println("EXPRES:   " + guard);
        List<Token> tokens = RPNComputer.tokenize(guard);
        System.out.println("TOKENS:   " + tokens);
        RPN rpn = RPNComputer.infixToRPN(guard);
        System.out.println("RPN:      " + rpn);
        System.out.println("CONSTR:   " + rpn.getConstraints());
        final long start = System.currentTimeMillis();
        System.out.println("DURATION: " + (System.currentTimeMillis() - start) + " ms");
    }
}
